<?php

namespace app\controllers;

use app\models\Accident;
use app\models\AccidentSearch;
use app\models\Shift;
use Yii;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\AccessControl;

/**
 * AccidentController implements the CRUD actions for Accident model.
 */
class AccidentController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Accident models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new AccidentSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Accident model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Accident model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new Accident();

        if ($model->load(Yii::$app->request->post())) {

            $req_time = date('Y-m-d H:i:s');
            $shift = Shift::find()->where(['AND', ['<=', 'shift_start', $req_time], ['>=', 'shift_end', $req_time], ['=', 'ic_id', (string) $model->ic_id]])->one();
            $model->vehicle_no = strtoupper($model->vehicle_no);
            $model->customer_name = ucwords($model->customer_name);

            if ($shift->id) {
                $model->shift_id = $shift->id;
                if ($model->save()) {
                    return $this->redirect(['view', 'id' => $model->id]);
                }
            } else {
                Yii::$app->getSession()->setFlash('danger', 'No shift available for today');
            }
        }

        return $this->render('create', [
                    'model' => $model,
        ]);
    }

    public function actionImport() {
        
    }

    /**
     * Deletes an existing Accident model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Accident model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Accident the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = Accident::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

}
