<?php

namespace app\controllers;

use Yii;
use app\models\Apk;
use app\models\ApkSearch;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;

/**
 * ApkFileController implements the CRUD actions for Apk model.
 */
class ApkFileController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Apk models.
     * @return mixed
     */
    public function actionIndex()
    {
       
        // $dataProvider = new ActiveDataProvider([
        //     'query' => Apk::find()->orderBy(['upload_date' => SORT_DESC]),
        // ]);

        // return $this->render('index', [
        //     'dataProvider' => $dataProvider,
        // ]);
        $searchModel = new ApkSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Apk model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Apk model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    
    public function actionCreate()
    {
        // $model = new Apk();

        // if ($model->load(Yii::$app->request->post())) {

        //     $model->file = UploadedFile::getInstance($model, 'file');
        //     if ($model->file && $model->validate()) {
        //         $fileName = $model->apk_version . '.' . $model->file->extension;
        //         $model->file->saveAs('apk/' . $fileName);

        //         Apk::updateAll(['status' => 0]);

        //         $model->apk_file = 'apk/' . $fileName;
        //         $model->upload_date = date('Y-m-d H:i:s');
        //         $model->status = 1; 

        //         if ($model->save(false)) {
        //             return $this->redirect(['view', 'id' => $model->id]);
        //         }
        //     }
        // }

        // return $this->render('create', [
        //     'model' => $model,
        // ]);
        $model = new Apk();

            if ($model->load(Yii::$app->request->post())) {
            ini_set('post_max_size', '100M');
            ini_set('upload_max_filesize', '100M');
                // Check if the apk_version is unique before proceeding
                $existingModel = Apk::findOne(['apk_version' => $model->apk_version]);
                if ($existingModel) {
                    $model->addError('apk_version', 'This APK version already exists.');
                } else {
                    $model->file = UploadedFile::getInstance($model, 'file');
                    if ($model->file && $model->validate()) {
                        // $fileName = $model->apk_version . '.' . $model->file->extension;
                        // $model->file->saveAs('apk/' . $fileName);

                        $fileExtension = $model->file->extension;
                        $model->apk_version .= '.' . $fileExtension;

                        // Save the file to the server
                        $fileName = $model->apk_version;
                    
                         $model->file->saveAs('./apk/' . $fileName);

                        // Optionally, update status of all other versions to 0
                        Apk::updateAll(['status' => 0]);

                        $model->apk_file = './apk/' . $fileName;
                        $model->upload_date = date('Y-m-d H:i:s');
                        $model->status = 1; // Set the status to indicate this is the current version
                        
                         $model->download_count = 0;

                        if ($model->save(false)) {
                            return $this->redirect(['view', 'id' => $model->id]);
                        }
                    }
                }
            }

            return $this->render('create', [
                'model' => $model,
            ]);
    }

    /**
     * Updates an existing Apk model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Apk model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        // $this->findModel($id)->delete();
        $model = $this->findModel($id);

        $filePath = Yii::getAlias('@webroot/apk' . $model->apk_file);
        if (file_exists($filePath)) {
            unlink($filePath); 
        }

        $model->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Apk model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Apk the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Apk::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    
    public function actionDownload($filename)
    {
        
        $model = Apk::findOne(['apk_file' => './apk/'.$filename]);

        if ($model) {
          print_r($model->apk_file);
          
            //$path = './apk/'.$model->apk_file;
            $path = Yii :: getAlias('@webroot/apk/') . $filename;
             //$path = Yii::getAlias('@web'). $model->apk_file;
            
 //print_r($path);die;
 
            if (file_exists($path)) {
                $model->download_count += 1;
                $model->save(false);

                return Yii::$app->response->sendFile($path, $filename, [
                    'mimeType' => mime_content_type($path),
                    'inline' => false,
                ]);
            } else {
                throw new NotFoundHttpException('The requested file does not exist.');
            }
        } else {
            throw new NotFoundHttpException('The requested file does not exist.');
        }
    }


}
