<?php

namespace app\controllers;

use app\models\Booth;
use app\models\BoothDevices;
use app\models\BoothSearch;
use app\models\Ic;
use app\models\Barcode;
use webvimark\modules\UserManagement\models\User;
use Yii;
use yii\filters\AccessControl;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use app\models\IcHeadTellers;
use app\models\IcAts;
use app\models\IcHeadTellersWeekend;
use app\models\IcAtsAll;
use app\models\IcAtsWeekend;
use app\models\IcTs;
use app\models\IcManager;
use app\models\RegionAdmin;
use app\models\RegionManager;

/**
 * BoothController implements the CRUD actions for Booth model.
 */
class BoothController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {

        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update', 'delete', 'unassigned-device', 'booth-list', 'inactive', 'ic-booth-list', 'booth-log'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Booth models.
     * @return mixed
     */
    public function actionIndex() {

        $searchModel = new BoothSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Booth model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {

        $model = $this->findModel($id);
        $booth_devices = new BoothDevices();
        $all_devices = ArrayHelper::map(Yii::$app->db->createCommand('SELECT tbl_devices.id as id,tbl_devices.serial as serial,tbl_devices.name as device_name FROM tbl_booth_devices INNER JOIN tbl_devices ON tbl_devices.id=tbl_booth_devices.device_id WHERE tbl_devices.status=1 AND tbl_booth_devices.status=1 AND tbl_booth_devices.ic_id=' . $model->ic->id . ' AND tbl_booth_devices.booth_id is NULL ')->queryAll(), 'id', function ($model) {
                    return $model['device_name'] . ' ' . '(' . $model['serial'] . ' )';
                });

        $devices_for_booth = BoothDevices::find()->where(['ic_id' => $model->ic->id, 'status' => 1, 'booth_id' => $id])->all();
        $devices_for_booth_count = BoothDevices::find()->where(['ic_id' => $model->ic->id, 'status' => 1, 'booth_id' => $id])->count();

        if ($booth_devices->load(Yii::$app->request->post())) {

            $device_id = $booth_devices['device_id'];
            $today = date("Y-m-d H:i:s");
            $user_id = Yii::$app->user->id;

            Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET booth_id=' . $id . ',booth_assigned_on="' . $today . '", booth_assigned_by=' . $user_id . ' WHERE ic_id=' . $model->ic->id . ' AND device_id= ' . $device_id . ' AND status=1  ')->execute();

            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_user"')->queryOne();
            $new_version = $master_version['tbl_version'] + 1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_user"')->execute();
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('view', [
                    'model' => $model,
                    'all_devices' => $all_devices,
                    'devices_for_booth' => $devices_for_booth,
                    'booth_devices' => $booth_devices,
                    'devices_for_booth_count' => $devices_for_booth_count,
        ]);
    }

    /**
     * Creates a new Booth model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new Booth();

        $buttenEnable =1;
        if ($model->load(Yii::$app->request->post())) {
            $regionn = Ic::find()->select('region_id')->where(['id' => $model->ic_id])->one();
            $model->region_id = $regionn->region_id;

            if ($model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }
        }

        return $this->render('create', [
                    'model' => $model,
                    'buttenEnable'=>$buttenEnable,
        ]);
    }

    /**
     * Updates an existing Booth model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id) {
        $model = $this->findModel($id);

        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;
        
        $buttenEnable = 1;
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $buttenEnable = 1;
        } else if (User::hasRole('95')) {
            $buttenEnable = 0;
        } else if (User::hasRole('65')) {
            $buttenEnable = 0;
        } else if (User::hasRole('90')) {
            $buttenEnable = 1;
        }else if (User::hasRole('91')) {
            $buttenEnable = 0;
        }else if (User::hasRole('85')) {
            $buttenEnable = 0;
        }

        if ($model->status == Booth::STATUS_DEACTIVE) {
            Booth::updateAll(
                    [
                        'activated_on' => $today,
                        'activated_by' => $user_id
                    ],
                    ['id' => $id]
            );
            //Barcode::updateAll(['removed_on' => $today], ['removed_by' => $user_id], ['booth_id' => $id]);
            
        } elseif ($model->status == Booth::STATUS_ACTIVE) {
            Booth::updateAll(
                    [
                        'inactivated_on' => $today,
                        'inactivated_by' => $user_id
                    ],
                    ['id' => $id]
            );
            
        } elseif ($model->status == Booth::STATUS_CLOSE) {
            Booth::updateAll(
                    [
                        'inactivated_on' => $today,
                        'inactivated_by' => $user_id
                    ],
                    ['id' => $id]
            );
            
        }

        if ($model->load(Yii::$app->request->post()) && $model->save()) {

            if ($model->status == Booth::STATUS_ACTIVE || $model->status == Booth::STATUS_CLOSE) {
                // Update related Barcode records to inactive
                //Barcode::updateAll(['status' => Barcode::STATUS_INACTIVE], ['booth_id' => $model->id]);
                
                // Update related Barcode records to inactive and set removed_on/removed_by
                Barcode::updateAll(
                    [
                        'status' => Barcode::STATUS_INACTIVE,
                        'removed_on' => $today,
                        'removed_by' => $user_id,
                    ],
                    ['booth_id' => $model->id]
                );
                
            } else {
                // Update related Barcode records to active
                Barcode::updateAll(['status' => Barcode::STATUS_ACTIVE], ['booth_id' => $model->id]);
            }

            return $this->redirect(['view', 'id' => $model->id]);
        }


        return $this->render('update', [
                    'model' => $model,
                    'buttenEnable'=>$buttenEnable,
        ]);
    }

    /**
     * Deletes an existing Booth model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        try {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the Booth model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Booth the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {

            if (($model = Booth::findOne($id)) !== null) {
                return $model;
            }

            throw new NotFoundHttpException('The requested page does not exist.');
        }
        // REGION ADMIN
        if (User::hasRole('90')) {

            $db = \Yii::$app->db;
            $booth_ids = $db->createCommand("SELECT b.id
                FROM tbl_booth b 
                INNER JOIN tbl_region_admin rm ON b.region_id = rm.region_id
                WHERE rm.user_id =:user_id AND rm.status =1")->bindValue(':user_id', \Yii::$app->user->identity->id)->queryColumn();

            if (in_array($id, $booth_ids)) {
                $model = Booth::findOne(['id' => $id]);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
        }

        //REGION MANAGER        
        if (User::hasRole('91')) {

            $db = \Yii::$app->db;
            $booth_ids = $db->createCommand("SELECT b.id
                FROM tbl_booth b 
                INNER JOIN tbl_region_manager rm ON b.region_id = rm.region_id
                WHERE rm.user_id =:user_id AND rm.status =1")->bindValue(':user_id', \Yii::$app->user->identity->id)->queryColumn();

            if (in_array($id, $booth_ids)) {
                $model = Booth::findOne(['id' => $id]);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
        }

        // IC MANAGER 85
        if (User::hasRole('85')) {

            $db = \Yii::$app->db;
            $booth_ids = $db->createCommand("
                SELECT b.id
                FROM tbl_booth b 
                INNER JOIN tbl_ic_manager ic ON b.ic_id = ic.ic_id
                WHERE ic.user_id =:user_id AND ic.status =1")->bindValue(':user_id', \Yii::$app->user->identity->id)->queryColumn();

            if (in_array($id, $booth_ids)) {
                $model = Booth::findOne(['id' => $id]);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
        }

        // ATS 40, TS 30, HEAD TELLERS 25
        $roleToClassMapping = [
            '40' => [IcAts::class, IcAtsWeekend::class],
            '30' => [IcAtsAll::class, IcTs::class],
        ];

        foreach ($roleToClassMapping as $roleId => $classes) {
            if (User::hasRole($roleId)) {
                $ic_ids = [];
                foreach ($classes as $class) {
                    $ic_ids = array_merge($ic_ids, $class::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->column());
                }

                if (in_array($id, $ic_ids)) {
                    $model = Booth::findOne(['id' => $id]);
                    if ($model !== null) {
                        return $model;
                    }
                }

                throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
            }
        }
        //HEAD TELLER 25
        if (User::hasRole('25')) {


            $db = \Yii::$app->db;

            $booth_ids = $db->createCommand("
                SELECT b.id
                FROM tbl_booth b 
                INNER JOIN tbl_ic_head_tellers ht ON b.ic_id = ht.ic_id
                WHERE ht.user_id =:user_id AND ht.status =1
                UNION
                SELECT b.id
                FROM tbl_booth b 
                INNER JOIN tbl_ic_head_tellers_weekend htw ON b.ic_id = htw.ic_id
                WHERE htw.user_id =:user_id AND htw.status =1")->bindValue(':user_id', \Yii::$app->user->identity->id)->queryColumn();

            if (in_array($id, $booth_ids)) {

                $model = Booth::findOne(['id' => $id]);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionQuick() {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }

    public function actionUnassignedDevice() {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $device_id = ($_POST['device_id']);
        $booth_id = ($_POST['booth_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $row = BoothDevices::find()->where(['device_id' => $device_id, 'booth_id' => $booth_id, 'status' => 1])->one();
        $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,booth_remove_on="' . $today . '", booth_remove_by=' . $user_id . ' WHERE id=' . $row->id . '')->execute();
        $model = new BoothDevices();
        $model->device_id = $row->device_id;
        $model->ic_id = $row->ic_id;
        $model->status = 1;
        $model->ic_assigned_on = $row->ic_assigned_on;
        $model->ic_assigned_by = $row->ic_assigned_by;
        $model->save();

        return true;
    }

    public function actionInactive() {

        $searchModel = new BoothSearch();
        $dataProvider = $searchModel->searchInactive(Yii::$app->request->queryParams);

        return $this->render('inactive', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    public function actionBoothLog() {

        $searchModel = new BoothSearch();
        $dataProvider = $searchModel->searchBoothLog(Yii::$app->request->queryParams);

        return $this->render('booth-log', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /* public function actionBoothList($ic_id)
      {

      $count = Booth::find()
      ->where(['ic_id' => $ic_id])
      ->count();
      $booths = Booth::find()
      ->where(['ic_id' => $ic_id])
      ->all();
      if ($count > 0) {
      foreach ($booths as $booth) {
      echo "<option value='" . $booth->id . "'>" . $booth->name . "</option>";
      }
      } else {
      echo "<option value = ''>This interchange has no booth</option>";
      }

      } */

    public function actionBoothList($ic_id) {

        // Fetch Booths based on ic_id
        $booths = Booth::find()
                ->where(['ic_id' => $ic_id])
                ->all();

        // Prepare Booth options
        $boothOptions = "<option value=''>Select Booth</option>";
        foreach ($booths as $booth) {
            $boothOptions .= "<option value='" . $booth->id . "'>" . $booth->name . "</option>";
        }

        // Return Booth options
        return $boothOptions;
    }

    public function actionIcBoothList($region_id) {

        // Fetch ICs based on region_id
        $ics = Ic::find()
                ->where(['region_id' => $region_id])
                ->all();
  
        // Prepare IC options
        //$icOptions = "<option value='Any'>Any</option>";
        $icOptions = "<option value=''>Select Ic</option>";
        foreach ($ics as $ic) {
            $icOptions .= "<option value='" . $ic->id . "'>" . $ic->name . "</option>";
        }

        // Return JSON response
        return json_encode(['icOptions' => $icOptions]);
    }

}
