<?php

namespace app\controllers;

use app\common\components\Notification;
use app\models\CashCollect;
use app\models\CashCollectManualVehicleList;
use app\models\CashCollectMisusedVehicleList;
use app\models\CashCollectSearch;
use app\models\CashCollectSummary;
use app\models\CashCollectSummaryAts;
use app\models\CashCollectVehicleList;
use app\models\CashCollectWhitoutEscapeVehicleList;
use app\models\Devices;
use app\models\EntranceTicket;
use app\models\ExitTicket;
use app\models\Ic;
use app\models\IcUsers;
use app\models\LoginLogoutSession;
use app\models\MissIssuedTickets;
use app\models\MissIssuedTicketsEntrance;
use app\models\RegionAdmin;
use app\models\RegionManager;
use app\models\Shift;
use app\models\TicketTypeStatus;
use app\models\VehicleCategory;
use DateTime;
use webvimark\modules\UserManagement\models\User;
use Yii;
use yii\filters\AccessControl;
use yii\filters\VerbFilter;
use yii\helpers\Json;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use app\models\Towing;
use yii\web\BadRequestHttpException;
use yii\web\Response;
use yii\db\Query;
use app\models\TellerConfirm;
use app\models\SessionReportConfirm;
#use app\models\LoginForm;
use webvimark\modules\UserManagement\models\forms\LoginForm;

/**
 * DevicesController implements the CRUD actions for Devices model.
 */
class CashCollectController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'view-en', 'quick-en',
                            'create', 'update', 'delete', 'user-list', 'user-list-en', 'export-pdf', 'export-pdf-entrance',
                            'cash-collect-summary', 'get-summary', 'get-cash-balance',
                            'cash-collect-summary-pdf', 'cash-collect-summary-ats',
                            'get-summary-ats', 'cash-collect-summary-ats-pdf', 'cash-in', 'reset-cash-in', 'force-logout',
                            'reset-history', 'reset-count', 'remove',
                            'export-online', 'export-online-pdf'
                        ],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Devices models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new CashCollectSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Devices model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($user_id, $start_date, $end_date, $actual_session_id) {

        ini_set('memory_limit', '-1');
        $cash_collect = new CashCollect;
        $exit_ticket = new ExitTicket;
        $exit_ticket_es = new ExitTicket;
        $miss_issued_ticket = new MissIssuedTickets;

        $saveButtenEnable = 1;

        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $saveButtenEnable = 1;
        } else if (User::hasRole('95')) {
            $saveButtenEnable = 0;
        } else if (User::hasRole('65')) {
            $saveButtenEnable = 0;
        }

        if ($actual_session_id) {
            $actual_session_model = $this->findModel($actual_session_id);

            $towing_list = Towing::find()->where(['session_uuid' => $actual_session_model->session_uuid])->all();
            $total_towing = 0;
            foreach ($towing_list as $tlist) {

                $total_towing = $tlist->amount + $total_towing;
            }

            $mi_tickets_data = ExitTicket::find()
                    ->where(['session_uuid' => $actual_session_model->session_uuid, 'ticket_type_status_id' => 12])
                    ->orderBy(['id' => SORT_DESC])
                    ->all();

            $br_tickets_data = ExitTicket::find()
                    ->where(['session_uuid' => $actual_session_model->session_uuid, 'ticket_type_status_id' => 7])
                    ->orderBy(['id' => SORT_DESC])
                    ->all();

            $es_tickets_data = ExitTicket::find()
                    ->where(['session_uuid' => $actual_session_model->session_uuid, 'ticket_type_status_id' => 8])
                    ->orderBy(['id' => SORT_DESC])
                    ->all();

            $tickets_data = ExitTicket::find()
                    ->where(['session_uuid' => $actual_session_model->session_uuid])
                    ->orderBy(['id' => SORT_DESC])
                    ->all();
            $cash_collected_data_history = CashCollect::find()->where(['actual_session_id' => $actual_session_id])->orderBy([
                        'id' => SORT_DESC, //specify sort order ASC for ascending DESC for descending
                    ])->all();

            $total_amount = 0;

            $ticket_type_status = TicketTypeStatus::find()->all();
            $vehicle_categories = VehicleCategory::find()->all();
            $ticket_types_count = [];
            $vehicle_category_count = [];
            foreach ($ticket_type_status as $ticket_type) {
                $type_count = 0;
                $type_wise_amount = 0;
                foreach ($tickets_data as $ticket) {
                    if ($ticket_type->id == $ticket->ticket_type_status_id) {
                        $type_count++;
                        $type_wise_amount = $type_wise_amount + $ticket->amount;
                    }
                }
                array_push($ticket_types_count, ["name" => $ticket_type->name, "count" => $type_count, "amount" => $type_wise_amount]);
            }
            foreach ($vehicle_categories as $vehicle_category) {
                $type_count = 0;
                $category_wise_amount = 0;
                foreach ($tickets_data as $ticket) {
                    if ($vehicle_category->id == $ticket->vehicle_category) {
                        if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 12)) {
                            $type_count++;
                            if (!($ticket->ticket_type_status_id == 8)) {
                                $category_wise_amount = $category_wise_amount + $ticket->amount;
                            }
                        }
                    }
                }
                array_push($vehicle_category_count, ["name" => $vehicle_category->name, "count" => $type_count, "amount" => $category_wise_amount]);
            }

            foreach ($vehicle_category_count as $obj) {
                $all_categpory_ticket_count = $obj["count"] + $all_categpory_ticket_count;
            }

            foreach ($tickets_data as $ticket) {
                if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 8) && !($ticket->ticket_type_status_id == 12)) {
                    $total_amount = $total_amount + $ticket->amount;
                }
            }
            $total_ticket_count = count($tickets_data);
            $cash_collected_data = CashCollect::find()->where(['status' => 1, 'actual_session_id' => $actual_session_id])->one();
        }

        $new_start_date = new DateTime($start_date);
        $new_start_date = $new_start_date->format("Y-m-d 00:00:00");
        $new_end_date = new DateTime($end_date);
        $new_end_date = $new_end_date->format("Y-m-d 23:59:59");
        $dataLoginLogoutPairsProvider = LoginLogoutSession::find()->joinWith(['booth'])->where([
                    'and',
                    [
                        'user_id' => $user_id,
                        'tbl_booth.booth_type' => 2,
                    ],
                    ['between', 'login_time', $new_start_date, $new_end_date],
                ])->orderBy(['id' => SORT_DESC])->all();

        // submit cash
        if ($cash_collect->load(Yii::$app->request->post())) {

            $cash_collect->vehicle_count = json_encode($vehicle_category_count);
            $login_logout_session = LoginLogoutSession::find()->where(['id' => $actual_session_id])->one();
            $cash_collect->shift_id = $login_logout_session->shift_id;
            $ic_id = $login_logout_session->booth->ic->id;
            $actual_system_total = ((float) $cash_collect->total_system_amount - (float) $cash_collect->total_qr_amount);
            $bb = (float) $cash_collect->total_cash_amount - $actual_system_total;

            //$bb = $cash_collect->total_cash_amount - ($total_amount + $cash_collect->not_sync_amount);

            $cash_collect->excess = 0;
            $cash_collect->short = 0;
            if ($bb > 0) {
                $cash_collect->excess = $bb;
            } elseif ($bb < 0) {
                $cash_collect->short = -1 * $bb;
            }

            CashCollect::updateAll(['status' => 0], ['=', 'actual_session_id', $actual_session_id]);

            if ($cash_collect->save()) {
                if ($bb > 0) {

                    $notify1 = Yii::$app->SaveNotifications->send_to_ht(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                    $notify2 = Yii::$app->SaveNotifications->send_to_ats(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                    $notify3 = Yii::$app->SaveNotifications->send_to_superadmin(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                } elseif ($bb < 0) {

                    $notify1 = Yii::$app->SaveNotifications->send_to_ht(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                    $notify2 = Yii::$app->SaveNotifications->send_to_ats(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                    $notify3 = Yii::$app->SaveNotifications->send_to_superadmin(Notification::KEY_EXCESS_SHORT, $cash_collect->id, $ic_id);
                }

                //==========new change ============================
                $actual_session_model_new = $this->findModel($actual_session_id);

                $tickets_data_new = ExitTicket::find()->where([
                            'session_uuid' => $actual_session_model_new->session_uuid,
                        ])->orderBy(['id' => SORT_DESC])->all();

                $total_amount = 0;

                $ticket_type_status_new = TicketTypeStatus::find()->all();
                $vehicle_categories_new = VehicleCategory::find()->all();
                $ticket_types_count_new = [];
                $vehicle_category_count_new = [];
                $manual_ticket_count_new = [];
                $escape_ticket_count_new = [];
                $vehicle_category_count_without_escape_new = [];
                foreach ($ticket_type_status_new as $ticket_type) {
                    $type_count = 0;
                    $type_wise_amount = 0;
                    foreach ($tickets_data_new as $ticket) {
                        if ($ticket_type->id == $ticket->ticket_type_status_id) {
                            $type_count++;
                            $type_wise_amount = $type_wise_amount + $ticket->amount;
                        }
                    }
                    array_push($ticket_types_count_new, ["name" => $ticket_type->name, "id" => $ticket_type->id, "count" => $type_count, "amount" => $type_wise_amount]);
                }
                foreach ($vehicle_categories_new as $vehicle_category) {
                    $type_count = 0;
                    $category_wise_amount = 0;
                    $type_count_manual = 0;
                    $type_wise_amount_manual = 0;
                    $type_count_escape = 0;
                    $type_wise_amount_escape = 0;
                    $category_wise_amount_without_escape = 0;
                    foreach ($tickets_data_new as $ticket) {
                        if ($ticket->vehicle_category == $vehicle_category->id) {

                            //TICKET PRINT ON POS
                            if ($ticket->ticket_type_status_id != 7 && $ticket->ticket_type_status_id != 8) {
                                $type_count++;
                                $category_wise_amount = $category_wise_amount + $ticket->amount;
                            }
                            //WITHOUT ESCAPE TICKETS
                            if ($ticket->ticket_type_status_id != 8) {
                                $type_count_without_escape++;
                                $category_wise_amount_without_escape = $category_wise_amount_without_escape + $ticket->amount;
                            }

                            //MANUAL TICKET
                            if ($ticket->ticket_type_status_id == 7) {
                                $type_count_manual++;
                                $type_wise_amount_manual = $type_wise_amount_manual + $ticket->amount;
                            }
                            //ESCAPE TICKETS
                            if ($ticket->ticket_type_status_id == 8) {

                                $type_count_escape++;
                                $type_wise_amount_escape = $type_wise_amount_escape + $ticket->amount;
                            }
                        }
                    }
                    array_push($manual_ticket_count_new, ["name" => $vehicle_category->name, "count" => $type_count_manual, "amount" => $type_wise_amount_manual]);
                    array_push($escape_ticket_count_new, ["name" => $vehicle_category->name, "count" => $type_count_escape, "amount" => $type_wise_amount_escape]);

                    array_push($vehicle_category_count_new, ["name" => $vehicle_category->name, "count" => $type_count, "amount" => $category_wise_amount]);
                    array_push($vehicle_category_count_without_escape_new, ["name" => $vehicle_category->name, "count" => $type_count_without_escape, "amount" => $category_wise_amount_without_escape]);
                }

                foreach ($tickets_data_new as $ticket) {

                    if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 8)) {
                        $total_amount = $total_amount + $ticket->amount;
                    }
                }
                $total_ticket_count_new = count($tickets_data_new);

                $cash_collected_data = CashCollect::find()->where(['status' => 1, 'actual_session_id' => $actual_session_id])->one();
                $total_rev = 0;
                foreach ($vehicle_category_count_without_escape_new as $type) {
                    $total_rev = $total_rev + $type["amount"];
                }
//    print_r(json_encode($manual_ticket_count_new));die;
                //===========end===================================

                foreach ($vehicle_category_count_new as $vcc) {
                    $cashCollectVehicleList = new CashCollectVehicleList;
                    $cashCollectVehicleList->cash_collect_id = $cash_collect->id;
                    $cashCollectVehicleList->vehicle_type = $vcc['name'];
                    $cashCollectVehicleList->vehicle_count = $vcc['count'];
                    $cashCollectVehicleList->amount = $vcc['amount'];
                    $cashCollectVehicleList->save();
                }
                foreach ($manual_ticket_count_new as $vcc) {
                    $cashCollectVehicleList = new CashCollectManualVehicleList;
                    $cashCollectVehicleList->cash_collect_id = $cash_collect->id;
                    $cashCollectVehicleList->vehicle_type = $vcc['name'];
                    $cashCollectVehicleList->vehicle_count = $vcc['count'];
                    $cashCollectVehicleList->amount = $vcc['amount'];
                    $cashCollectVehicleList->save();
                }
                foreach ($escape_ticket_count_new as $vcc) {
                    $cashCollectVehicleList = new CashCollectMisusedVehicleList;
                    $cashCollectVehicleList->cash_collect_id = $cash_collect->id;
                    $cashCollectVehicleList->vehicle_type = $vcc['name'];
                    $cashCollectVehicleList->vehicle_count = $vcc['count'];
                    $cashCollectVehicleList->amount = $vcc['amount'];
                    $cashCollectVehicleList->save();
                }
                foreach ($vehicle_category_count_without_escape_new as $vcc) {
                    $cashCollectVehicleList = new CashCollectWhitoutEscapeVehicleList;
                    $cashCollectVehicleList->cash_collect_id = $cash_collect->id;
                    $cashCollectVehicleList->vehicle_type = $vcc['name'];
                    $cashCollectVehicleList->vehicle_count = $vcc['count'];
                    $cashCollectVehicleList->amount = $vcc['amount'];
                    $cashCollectVehicleList->save();
                }
                return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id, 'saveButtenEnable' => $saveButtenEnable]);
            }
        }

        // submit Escape Ticket
        if ($exit_ticket_es->load(Yii::$app->request->post()) && $exit_ticket_es->entrance_barcode == 'ESCAPE') {
           
            $exit_ticket_es->ticket_uuid = Yii::$app->Guid->getGUID();
            //if has logout time
            if ($actual_session_model->logout_time) {
                if ($actual_session_model->login_time <= $exit_ticket_es->exit_time && $actual_session_model->logout_time >= $exit_ticket_es->exit_time) {
                    
                    if ($exit_ticket_es->save()) {

                        Yii::$app->getSession()->setFlash('success', 'Escape ticket added');
                        return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id, 'saveButtenEnable' => $saveButtenEnable]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('danger', 'Please mind entrance time and exit time of escape ticket');
                }
            } else {
                if ($actual_session_model->login_time <= $exit_ticket_es->exit_time) {
                    if ($exit_ticket_es->save()) {
                        return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id, 'saveButtenEnable' => $saveButtenEnable]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('danger', 'Please mind entrance time and exit time of escape ticket');
                }
            }
        }
        // submit MISS Issued Ticket
        if ($miss_issued_ticket->load(Yii::$app->request->post())) {

            $exit_ticket = ExitTicket::find()->where(['id' => $miss_issued_ticket->exit_ticket_id])->one();
            $miss_issued_ticket->previous_ticket_type_status = $exit_ticket->ticket_type_status_id;
            if ($miss_issued_ticket->save()) {
                $res = Yii::$app->db->createCommand('UPDATE tbl_exit_ticket SET ticket_type_status_id=12 WHERE id=' . $miss_issued_ticket->exit_ticket_id . '')->execute();
                Yii::$app->getSession()->setFlash('success', 'Miss Issued Ticket added');
                return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id]);
            }
        }

        // submit Book Receipt Ticket
        if ($exit_ticket->load(Yii::$app->request->post()) && $exit_ticket_es->entrance_barcode != 'ESCAPE') {

            $exit_ticket->ticket_uuid = Yii::$app->Guid->getGUID();
            if ($actual_session_model->logout_time) {
                if ($actual_session_model->login_time <= $exit_ticket->exit_time && $actual_session_model->logout_time >= $exit_ticket->exit_time) {
                    
                    if ($exit_ticket->save()) {

                        Yii::$app->getSession()->setFlash('success', 'Book Receipt added');
                        return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id, 'saveButtenEnable' => $saveButtenEnable]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('danger', 'Please mind entrance time and exit time of book receipt ticket');
                }
            } else {
                if ($actual_session_model->login_time <= $exit_ticket->exit_time) {
                    if ($exit_ticket->save()) {
                        return $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id, 'saveButtenEnable' => $saveButtenEnable]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('danger', 'Please mind entrance time and exit time of book receipt ticket');
                }
            }
            //
            // $exit_ticket->ticket_uuid = getGUID();
            // if($exit_ticket->save()){
            //      return $this->redirect(['view', 'user_id' => $user_id,'start_date'=>$start_date,'end_date'=>$end_date,'actual_session_id'=>$actual_session_model->id]);
            //  }
        }
//print_r($cash_collected_data);die
        return $this->render('view', [
                    'model' => $this->findUser($user_id),
                    'miss_issued_ticket' => $miss_issued_ticket,
                    'dataLoginLogoutPairsProvider' => $dataLoginLogoutPairsProvider,
                    'start_date' => $start_date,
                    'end_date' => $end_date,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_amount' => ($total_amount + $total_towing),
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'cash_collected_data' => $cash_collected_data,
                    //'balance'=>($cash_collected_data->total_system_amount - $cash_collected_data->total_cash_amount),
                    'balance' => ($total_amount - ($cash_collected_data->total_cash_amount)),
                    'cash_collected_data_history' => $cash_collected_data_history,
                    'exit_ticket' => $exit_ticket,
                    'exit_ticket_es' => $exit_ticket_es,
                    'miss_issued_ticket' => $miss_issued_ticket,
                    'all_categpory_ticket_count' => $all_categpory_ticket_count,
                    'towing_list' => $towing_list,
                    'total_towing' => $total_towing,
                    'mi_tickets_data' => $mi_tickets_data,
                    'br_tickets_data' => $br_tickets_data,
                    'es_tickets_data' => $es_tickets_data,
                    'saveButtenEnable' => $saveButtenEnable
        ]);
    }

    protected function findModel($id) {
        if (($model = LoginLogoutSession::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    protected function findModelCashCollect($id) {
        if (($model = CashCollect::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    protected function findUser($id) {
        if (($model = User::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionUserList() {

        //  $all_users = Yii::$app->db->createCommand('SELECT id,username,full_name FROM user WHERE designation_id =5 ')->queryAll();
        $all_users = Yii::$app->db->createCommand('SELECT id,username,full_name FROM user WHERE designation_id =5')->queryAll();

        if (!Yii::$app->user->isSuperadmin) {

            //HEAD TELLERS 25
            //filter ther users related to IC for
            if (User::hasRole('25')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_head_tellers WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_head_tellers_weekend WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //ATS 40
            //filter ther users related to IC for
            if (User::hasRole('40')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats_weekend WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }
            //TS 30
            //filter ther users related to IC for
            if (User::hasRole('30')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats_all WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ts WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //IC Manager
            //filter ther users related to IC for
            if (User::hasRole('85')) {

                $ic_manager = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_manager WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_manager) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_manager], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //REGION ADMIN 90
            //filter ther users related to IC for
            if (User::hasRole('90')) {

                $regions = RegionAdmin::find()->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

                $ar_region = [];
                foreach ($regions as $region) {
                    array_push($ar_region, $region->region_id);
                }
                //  $ics = Ic::find()->select('id')->where(['region_id' => $region->region_id])->all();
                $ics = Ic::find()->select('id')->where(['in', 'region_id', $ar_region])->all();

                $ar = [];
                foreach ($ics as $ic) {
                    array_push($ar, $ic->id);
                }

                if ($ics) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //REGION MANAGER 91
            //filter there users related to IC for
            if (User::hasRole('91')) {

                $regions = RegionManager::find()->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

                $ar_region = [];
                foreach ($regions as $region) {
                    array_push($ar_region, $region->region_id);
                }

                $ics = Ic::find()->select('id')->where(['in', 'region_id', $ar_region])->all();

                $ar = [];
                foreach ($ics as $ic) {
                    array_push($ar, $ic->id);
                }

                if ($ics) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }
        }

        $today = date("Y-m-d");
        $date = new DateTime($today);
        $date->modify("-1 day");
        $start_date = $date->format("Y-m-d");

        $end_date = $today;

        return $this->render('user-list', [
                    'all_users' => $all_users,
                    'start_date' => $start_date,
                    'end_date' => $end_date,
        ]);
    }

    public function actionUserListEn() {

        //  $all_users = Yii::$app->db->createCommand('SELECT id,username,full_name FROM user WHERE designation_id =5  ')->queryAll();
        $all_users = Yii::$app->db->createCommand('SELECT id,username,full_name FROM user WHERE designation_id =5 ')->queryAll();

        if (!Yii::$app->user->isSuperadmin) {

            //HEAD TELLERS 25
            //filter ther users related to IC for
            if (User::hasRole('25')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_head_tellers WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_head_tellers_weekend WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //ATS 40
            //filter ther users related to IC for
            if (User::hasRole('40')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats_weekend WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }
            //TS 30
            //filter ther users related to IC for
            if (User::hasRole('30')) {

                $ic_list_ats = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ats_all WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();
                $ic_list_ats_weekend = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_ts WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_list_ats) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
                if ($ic_list_ats_weekend) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_list_ats_weekend], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //IC Manager
            //filter ther users related to IC for
            if (User::hasRole('85')) {

                $ic_manager = Yii::$app->db->createCommand('SELECT ic_id FROM tbl_ic_manager WHERE user_id ="' . \Yii::$app->user->identity->id . '" AND status=1 ')->queryAll();

                if ($ic_manager) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ic_manager], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }


            //REGION ADMIN 90
            //filter there users related to IC for
            if (User::hasRole('90')) {

                $regions = RegionAdmin::find()->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

                $ar_region = [];
                foreach ($regions as $region) {
                    array_push($ar_region, $region->region_id);
                }
                //  $ics = Ic::find()->select('id')->where(['region_id' => $region->region_id])->all();
                $ics = Ic::find()->select('id')->where(['in', 'region_id', $ar_region])->all();

                $ar = [];
                foreach ($ics as $ic) {
                    array_push($ar, $ic->id);
                }

                if ($ics) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }

            //REGION MANAGER 91
            //filter there users related to IC for
            if (User::hasRole('91')) {

                $regions = RegionManager::find()->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

                $ar_region = [];
                foreach ($regions as $region) {
                    array_push($ar_region, $region->region_id);
                }

                $ics = Ic::find()->select('id')->where(['in', 'region_id', $ar_region])->all();

                $ar = [];
                foreach ($ics as $ic) {
                    array_push($ar, $ic->id);
                }

                if ($ics) {
                    $ic_users = IcUsers::find()->select('user_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                    $all_users = User::find()->select('id,username,full_name')->where(['AND', ['in', 'id', $ic_users], ['designation_id' => 5]])->all();
                }
            }
        }

        $today = date("Y-m-d");
        $date = new DateTime($today);
        $date->modify("-1 day");
        $start_date = $date->format("Y-m-d");

        $end_date = $today;

        return $this->render('user-list-en', [
                    'all_users' => $all_users,
                    'start_date' => $start_date,
                    'end_date' => $end_date,
        ]);
    }

    public function actionCashIn($id) {

        return $this->render('cash-in', [
                    'model' => $this->findModelCashCollect($id),
        ]);
    }

    //Reset History
    public function actionResetHistory($actual_session_id) {
        $resetHistories = CashCollect::find()
                ->where(['actual_session_id' => $actual_session_id])
                ->with(['lastResetUser'])
                ->orderBy(['last_reset_at' => SORT_DESC])
                ->all();

        return $this->render('reset-history', [
                    'resetHistories' => $resetHistories,
        ]);
    }

    public function actionQuick() {
        $user_id = ($_POST['user_id']);
        $start_date = ($_POST['start_date']);
        $end_date = ($_POST['end_date']);
        $this->redirect(['view', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => ""]);
    }

    public function actionExportPdf($user_id, $actual_session_id) {

        $actual_session_model = $this->findModel($actual_session_id);

        $tickets_data = ExitTicket::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $total_amount = 0;

        $ticket_type_status = TicketTypeStatus::find()->all();
        $vehicle_categories = VehicleCategory::find()->where(['not in', 'id', [99, 30]])->all();
        $ticket_types_count = [];
        $vehicle_category_count = [];
        $manual_ticket_count = [];
        $escape_ticket_count = [];
        $vehicle_category_count_without_escape = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket_type->id == $ticket->ticket_type_status_id) {
                    $type_count++;
                    $type_wise_amount = $type_wise_amount + $ticket->amount;
                }
            }
            array_push($ticket_types_count, ["name" => $ticket_type->name, "id" => $ticket_type->id, "count" => $type_count, "amount" => $type_wise_amount]);
        }
        foreach ($vehicle_categories as $vehicle_category) {
            $type_count = 0;
            $category_wise_amount = 0;
            $type_count_manual = 0;
            $type_wise_amount_manual = 0;
            $type_count_escape = 0;
            $type_wise_amount_escape = 0;
            $category_wise_amount_without_escape = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket->vehicle_category == $vehicle_category->id) {

                    //TICKET PRINT ON POS
                    if ($ticket->ticket_type_status_id != 7 && $ticket->ticket_type_status_id != 8) {
                        if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 12)) {
                            $type_count++;
                            $category_wise_amount = $category_wise_amount + $ticket->amount;
                        }
                        // $type_count++;
                        // $category_wise_amount  = $category_wise_amount + $ticket->amount;
                    }
                    //WITHOUT ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id != 8 && !($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 12)) {

                        $type_count_without_escape++;
                        $category_wise_amount_without_escape = $category_wise_amount_without_escape + $ticket->amount;
                    }

                    //MANUAL TICKET
                    if ($ticket->ticket_type_status_id == 7) {
                        $type_count_manual++;
                        $type_wise_amount_manual = $type_wise_amount_manual + $ticket->amount;
                    }
                    //ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id == 8) {
                        $type_count_escape++;
                        $type_wise_amount_escape = $type_wise_amount_escape + $ticket->amount;
                    }
                }
            }
            array_push($manual_ticket_count, ["name" => $vehicle_category->name, "count" => $type_count_manual, "amount" => $type_wise_amount_manual]);
            array_push($escape_ticket_count, ["name" => $vehicle_category->name, "count" => $type_count_escape, "amount" => $type_wise_amount_escape]);

            array_push($vehicle_category_count, ["name" => $vehicle_category->name, "count" => $type_count, "amount" => $category_wise_amount]);
            array_push($vehicle_category_count_without_escape, ["name" => $vehicle_category->name, "count" => $type_count_without_escape, "amount" => $category_wise_amount_without_escape]);
        }
        //    print_r(json_encode($manual_ticket_count));die;

        foreach ($tickets_data as $ticket) {

            if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 8) && !($ticket->ticket_type_status_id == 12)) {
                $total_amount = $total_amount + $ticket->amount;
            }
        }
        $total_ticket_count = count($tickets_data);

        $cash_collected_data = CashCollect::find()->where(['status' => 1, 'actual_session_id' => $actual_session_id])->one();
        $total_rev = 0;
        foreach ($vehicle_category_count_without_escape as $type) {

            $total_rev = $total_rev + $type["amount"];
        }

        $user = $this->findUser($user_id);
        $mpdf = new \Mpdf\Mpdf([
            // 'orientation' => 'L',
            'setAutoTopMargin' => 'pad',
        ]);

        $mpdf->setFooter('Report generated by ' . Yii::$app->getUser()->username . '<br> on ' . date('Y-m-d H:i:s') . ' || Page {PAGENO} of {nb}');
        $mpdf->WriteHTML($this->renderPartial('pdf', [
                    'user' => $user,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_amount' => $total_amount,
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'cash_collected_data' => $cash_collected_data,
                    'manual_ticket_count' => $manual_ticket_count,
                    'escape_ticket_count' => $escape_ticket_count,
                    'vehicle_category_count_without_escape' => $vehicle_category_count_without_escape,
                    'total_rev' => $total_rev,
                    // 'balance'=>($cash_collected_data->total_system_amount - $cash_collected_data->total_cash_amount),
                    // 'balance'=>($total_rev - $cash_collected_data->total_cash_amount),
                    'balance' => ($total_amount - ($cash_collected_data->total_cash_amount + $cash_collected_data->total_qr_amount)),
        ]));
        $mpdf->Output('Cash Collect Summary Report (Teller-' . $user->username . ')' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

    public function actionCashCollectSummary() {
        $model = new CashCollectSummary();

        if (Yii::$app->request->isPost) {
            $model->load(Yii::$app->request->post());
            $sDate = date('Y-m-d', strtotime("-2 days"));
            $eDate = date('Y-m-d', strtotime("+1 days"));

            if ($model->ic == 'Any') {
                $results = Yii::$app->db->createCommand("
                SELECT S.*, IC.name AS ic_name
                  FROM tbl_shift S
                  INNER JOIN tbl_ic IC ON S.ic_id = IC.id
                  WHERE DATE(S.shift_start) BETWEEN :start_date AND :end_date
            ")
                        ->bindValue(':start_date', $sDate)
                        ->bindValue(':end_date', $eDate)
                        ->queryAll();
            } else {
                $results = Yii::$app->db->createCommand("
                SELECT S.*, IC.name AS ic_name
                FROM tbl_shift S
                INNER JOIN tbl_ic IC ON S.ic_id = IC.id
                WHERE S.ic_id = :ic_id
                AND DATE(S.shift_start) BETWEEN :start_date AND :end_date
            ")
                        ->bindValue(':ic_id', $model->ic)
                        ->bindValue(':start_date', $sDate)
                        ->bindValue(':end_date', $eDate)
                        ->queryAll();
            }

            // Preload IC options for the selected region
            $icOptions = $model->region ? IC::getICList($model->region) : [];

            return $this->render('cash-collect-summary', [
                        'model' => $model,
                        'results' => $results,
                        'icOptions' => $icOptions,
            ]);
        } else {
            return $this->render('cash-collect-summary', [
                        'model' => $model,
                        'icOptions' => [],
            ]);
        }
    }

    public function actionGetSummary() {
        $shift_id = Yii::$app->request->post('shift_id');
        //$shift_id = 136739;

        setcookie("cash-collect-shift-id", $shift_id, time() + 86400);

        $db = Yii::$app->db;

        $ff = $db->createCommand("SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY','')) ")->execute();

        //$shift_id = $_POST['shift_id'];
        //setcookie("cash-collect-shift-id", $shift_id, time() + 86400);

        $vehicle_category_count = Yii::$app->db->createCommand("
    SELECT VL.vehicle_type as name, SUM(VL.vehicle_count) as vCount, SUM(VL.amount) as vAmount
    FROM tbl_cash_collect CC
    INNER JOIN cash_collect_vehicle_list VL ON VL.cash_collect_id = CC.id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id
    GROUP BY VL.vehicle_type
    ORDER BY VL.id")
                ->bindValue(':shift_id', $shift_id)
                ->queryAll();

        $manual_ticket_count = Yii::$app->db->createCommand("
    SELECT VL.vehicle_type as name, SUM(VL.vehicle_count) as vCount, SUM(VL.amount) as vAmount
    FROM tbl_cash_collect CC
    INNER JOIN cash_collect_manual_vehicle_list VL ON VL.cash_collect_id = CC.id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id
    GROUP BY VL.vehicle_type
    ORDER BY VL.id")
                ->bindValue(':shift_id', $shift_id)
                ->queryAll();

        $escape_ticket_count = Yii::$app->db->createCommand("
    SELECT VL.vehicle_type as name, SUM(VL.vehicle_count) as vCount, SUM(VL.amount) as vAmount
    FROM tbl_cash_collect CC
    INNER JOIN cash_collect_misused_vehicle_list VL ON VL.cash_collect_id = CC.id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id
    GROUP BY VL.vehicle_type
    ORDER BY VL.id")
                ->bindValue(':shift_id', $shift_id)
                ->queryAll();

        $vehicle_category_count_without_escape = Yii::$app->db->createCommand("
    SELECT VL.vehicle_type as name, SUM(VL.vehicle_count) as vCount, SUM(VL.amount) as vAmount
    FROM tbl_cash_collect CC
    INNER JOIN cash_collect_whitout_escape_vehicle_list VL ON VL.cash_collect_id = CC.id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id
    GROUP BY VL.vehicle_type
    ORDER BY VL.id")
                ->bindValue(':shift_id', $shift_id)
                ->queryAll();

        $cashBalance = Yii::$app->db->createCommand("
    SELECT SUM(CC.total_system_amount) as sysTotal, SUM(CC.total_cash_amount) as cashTotal, SUM(CC.total_qr_amount) AS qrTotal
    FROM tbl_cash_collect CC
    WHERE CC.status = 1 AND CC.shift_id = :shift_id")
                ->bindValue(':shift_id', $shift_id)
                ->queryOne();

        $comments = Yii::$app->db->createCommand("
    SELECT CONCAT(LS.login_time,' - ',LS.logout_time) as sess, U.username as uName, CC.comment as comment
    FROM tbl_cash_collect CC
    INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
    INNER JOIN `user` U ON U.id = LS.user_id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id")
                ->bindValue(':shift_id', $shift_id)
                ->queryAll();

        $totalVehicleCount = Yii::$app->db->createCommand("
    SELECT SUM(VL.vehicle_count) as vCount
    FROM tbl_cash_collect CC
    INNER JOIN cash_collect_vehicle_list VL ON VL.cash_collect_id = CC.id
    WHERE CC.status = 1 AND CC.shift_id = :shift_id")
                ->bindValue(':shift_id', $shift_id)
                ->queryScalar();

        // Get cash reset count   
        $cash_reset = Yii::$app->db->createCommand("
    SELECT COUNT(*) 
    FROM tbl_cash_collect CC 
    WHERE CC.status = 0 AND CC.shift_id = :shift_id")
                ->bindValue(':shift_id', $shift_id)
                ->queryScalar();

// Get towing count    
        /* $towing_ticket = Yii::$app->db->createCommand(
          "SELECT COUNT(TW.id) AS twCount, IFNULL(SUM(TW.amount),0) AS twAmount
          FROM tbl_towing TW
          WHERE TW.shift_id = :shift_id")
          ->bindValue(':shift_id', $shift_id)
          ->queryScalar(); */

        /* $total_towing_amount = $towing_ticket['twAmount'];
          $total_towing_count = $towing_ticket['twCount']; */

        $towing_list = Towing::find()->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $total_towing_amount = 0;
        $towing_count = 0;

        foreach ($towing_list as $tlist) {
            $total_towing_amount += $tlist->amount;
            $towing_count++;
        }




// Duplicate exit ticket
        $duplicate_ticket = Yii::$app->db->createCommand("
            WITH DuplicateTickets AS (
                SELECT 
                    EX1.entrance_serial,
                    COUNT(EX1.id) AS serialCount
                FROM 
                    tbl_exit_ticket EX1
                WHERE 
                    EX1.exit_time <= NOW() + INTERVAL 2 HOUR 
                    AND DATE(EX1.exit_time) = CURRENT_DATE
                    AND EX1.shift_id = :shift_id
                GROUP BY 
                    EX1.entrance_serial
                HAVING 
                    COUNT(EX1.id) > 1
            ),
            FirstTicketCounts AS (
                SELECT 
                    EX2.entrance_serial,
                    COUNT(EX2.id) AS firstTicketCount
                FROM 
                    tbl_exit_ticket EX2
                WHERE 
                    EX2.id IN (
                        SELECT MIN(EX3.id)
                        FROM tbl_exit_ticket EX3
                        WHERE EX3.exit_time <= NOW() + INTERVAL 2 HOUR 
                        AND DATE(EX3.exit_time) = CURRENT_DATE
                        AND EX3.shift_id = :shift_id
                        GROUP BY EX3.entrance_serial
                    )
                GROUP BY 
                    EX2.entrance_serial
            )
            SELECT 
                (SUM(DT.serialCount) - COALESCE(SUM(FTC.firstTicketCount), 0)) AS tCount
            FROM 
                DuplicateTickets DT
            LEFT JOIN 
                FirstTicketCounts FTC ON DT.entrance_serial = FTC.entrance_serial
        ")->bindValue(':shift_id', $shift_id)
                ->queryScalar();

// Get special ticket types data
        $tickets_data = ExitTicket::find()
                ->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $ticket_type_status = TicketTypeStatus::find()->all();
        $ticket_types_count = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket->ticket_type_status_id == $ticket_type->id) {
                    $type_count++;
                    $type_wise_amount += $ticket->amount;
                }
            }
            $ticket_types_count[] = [
                "name" => $ticket_type->name,
                "count" => $type_count,
                "amount" => $type_wise_amount
            ];
        }

        $sysTotal = $cashBalance['sysTotal'];
        $cashTotal = $cashBalance['cashTotal'];
        $qrTotal = $cashBalance['qrTotal'];
//$balance = $cashTotal - $sysTotal;
//$balance = $cashBalance['cashTotal'] - $cashBalance['sysTotal'] + $cashBalance['qrTotal'];
        $balance = $cashBalance['cashTotal'] + $cashBalance['qrTotal'] - $cashBalance['sysTotal'];

        ob_start(); // Start output buffering
// Your logic here
        echo "<table border='1' style='width:100%; border-collapse:collapse;'>"; // Table styling
        if ($vehicle_category_count) {
            echo "<thead>";
            echo "<tr>";
            echo "<th style='width:150px; text-align:left;'><b>Item</b></th>"; // Adjust width for 'Item' column
            foreach ($vehicle_category_count as $type) {
                echo "<th style='width:100px; text-align:center;'><b>" . $type["name"] . "</b></th>"; // Adjust width for each category column
            }
            echo "<th style='width:120px; text-align:center;'><b>Grand Total</b></th>"; // Adjust width for 'Grand Total' column
            echo "</tr>";
            echo "</thead>";
            echo "<tbody>";

            // Row: No of Vehicles
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>No of Vehicles</td>";
            $total_vehicles_by_pos = 0;
            foreach ($vehicle_category_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vCount"]) . "</td>";
                $total_vehicles_by_pos += $type["vCount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_vehicles_by_pos) . "</td>";
            echo "</tr>";

            // Row: Manual Vehicles
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Manual Vehicles</td>";
            $total_manual_vehicles = 0;
            foreach ($manual_ticket_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vCount"]) . "</td>";
                $total_manual_vehicles += $type["vCount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_manual_vehicles) . "</td>";
            echo "</tr>";

            // Row: Escape Vehicles
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Escape Vehicles</td>";
            $total_misused_vehicles = 0;
            foreach ($escape_ticket_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vCount"]) . "</td>";
                $total_misused_vehicles += $type["vCount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_misused_vehicles) . "</td>";
            echo "</tr>";

            // Row: System Rev
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>System Rev</td>";
            $total_sys = 0;
            foreach ($vehicle_category_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vAmount"], 2, '.', ',') . "</td>";
                $total_sys += $type["vAmount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_sys, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Row: Manual Rev
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Manual Rev</td>";
            $total_manual = 0;
            foreach ($manual_ticket_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vAmount"], 2, '.', ',') . "</td>";
                $total_manual += $type["vAmount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_manual, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Row: Escape Rev
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Escape Rev</td>";
            $total_escape = 0;
            foreach ($escape_ticket_count as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vAmount"], 2, '.', ',') . "</td>";
                $total_escape += $type["vAmount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_escape, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Row: Total Rev
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Total Rev</td>";
            $total_rev = 0;
            foreach ($vehicle_category_count_without_escape as $type) {
                echo "<td style='text-align:right; width:100px;'>" . number_format($type["vAmount"], 2, '.', ',') . "</td>";
                $total_rev += $type["vAmount"];
            }
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_rev, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Row: Cash Reset Count
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Cash Reset Count</td>";
            echo "<td colspan='" . count($vehicle_category_count) . "' style='text-align:center;'></td>";
            echo "<td style='text-align:right; width:120px;'>" . number_format($cash_reset) . "</td>";
            echo "</tr>";

            // Row: Duplicate Ticket
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Duplicate Ticket</td>";
            echo "<td colspan='" . count($vehicle_category_count) . "'></td>";
            echo "<td style='text-align:right; width:120px;'>" . number_format($duplicate_ticket) . "</td>";
            echo "</tr>";

            // Row: Towing Count and Amount
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Towing Count</td>";
            echo "<td colspan='" . count($vehicle_category_count) . "'></td>";
            echo "<td style='text-align:right; width:120px;'>" . number_format($towing_count) . "</td>";
            echo "</tr>";
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Towing Amount</td>";
            echo "<td colspan='" . count($vehicle_category_count) . "'></td>";
            echo "<td style='text-align:right; width:120px;'>" . number_format($total_towing_amount, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Row: Sign by teller
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Sign by teller</td>";
            echo "<td colspan='2'>&nbsp;</td>";
            echo "<td style='text-align:left; width:100px;'>Date</td>";
            echo "<td colspan='2'></td>";
            echo "<td colspan='3' style='text-align:left;'>Excess :</td>";
            echo "<td colspan='2' style='text-align:right; width:120px;'>";
            if ($balance > 0) {
                echo "<p style='color:green;font-size:12px;margin: 6px 0;text-transform: uppercase;'>" . number_format($balance, 2, '.', ',') . "</p>";
            } else {
                echo '0';
            }
            echo "</td>";
            echo "</tr>";

            // Row: Check by HT
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Check by HT</td>";
            echo "<td colspan='2'>&nbsp;</td>";
            echo "<td style='text-align:left; width:100px;'>Date</td>";
            echo "<td colspan='2'>&nbsp;</td>";
            echo "<td colspan='3' style='text-align:left;'>Short :</td>";
            echo "<td colspan='2' style='text-align:right; width:120px;'>";
            if ($balance < 0) {
                echo "<p style='color:red;font-size:12px;margin: 6px 0;text-transform: uppercase;'>" . number_format(abs($balance), 2, '.', ',') . "</p>";
            } else {
                echo '0';
            }
            echo "</td>";
            echo "</tr>";

            // Row: Check by ATS
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Check by ATS</td>";
            echo "<td colspan='2'>&nbsp;</td>";
            echo "<td style='text-align:left; width:100px;'>Date</td>";
            echo "<td colspan='2'>&nbsp;</td>";
            echo "<td colspan='3' style='text-align:left;'>QR Payment :</td>";
            echo "<td colspan='2' style='text-align:right; width:120px;'>" . number_format($qrTotal, 2, '.', ',') . "</td>";
            echo "</tr>";

            // Grand Total
            echo "<tr>";
            echo "<td style='text-align:left; width:150px;'>Grand Total</td>";
            echo "<td colspan='8'>&nbsp;</td>";
            echo "<td colspan='2' style='text-align:right; width:120px;'>" . number_format($total_rev + $balance + $total_towing_amount, 2, '.', ',') . "</td>";
            echo "</tr>";

            echo "<tr><td colspan='" . (count($vehicle_category_count) + 2) . "'>&nbsp;</td></tr>";

            // Section: Special Ticket Types
            echo "<tr><td colspan='" . (count($vehicle_category_count) + 2) . "'>&nbsp;</td></tr>";

            echo "<tr>";
            echo "<th colspan='3' style='text-align:left; width:150px;'>Ticket Type</th>";
            echo "<th style='text-align:left; width:100px;'>Count</th>";
            echo "<th style='text-align:left; width:120px;'>Amount</th>";
            echo "</tr>";

            foreach ($ticket_types_count as $ticket_type) {
                echo "<tr>";
                echo "<td colspan='3' style='text-align:left; width:150px;'>" . $ticket_type["name"] . "</td>";
                echo "<td style='text-align:right; width:100px;'>" . number_format($ticket_type["count"]) . "</td>";
                echo "<td style='text-align:right; width:120px;'>" . number_format($ticket_type["amount"], 2, '.', ',') . "</td>";
                echo "</tr>";
            }
        } else {
            echo "<tr>";
            echo "<td colspan='" . (count($vehicle_category_count) + 2) . "' style='text-align:center; color:#856404; background-color: #fff3cd;'>No Record Found!!</td>";
            echo "</tr>";
        }
        echo "</tbody>";
        echo "</table>";

        return ob_get_clean(); // Flush the output buffer and send output
    }

    public function actionGetCashBalance() {
        $shift_id = $_POST['shift_id'];

        $result = Yii::$app->db->createCommand("SELECT sum(CC.total_system_amount) as sysTotal,SUM(CC.total_cash_amount) as cashTotal,SUM(CC.total_qr_amount) AS qrTotal
            FROM tbl_cash_collect CC
            WHERE CC.status = 1 AND CC.shift_id = $shift_id")->queryOne();
        // print_r($result);die;
        //$balance = $result['cashTotal'] - $result['sysTotal'] + $result['qrTotal'] ;
        $balance = $result['cashTotal'] + $result['qrTotal'] - $result['sysTotal'];
        $array = ['sysTotal' => $result['sysTotal'], 'cashTotal' => $result['cashTotal'], 'qrTotal' => $result['qrTotal'], 'balance' => $balance];
        if (!is_null($result['sysTotal'])) {
            print_r(json::encode($array));
        }
    }

    public function actionCashCollectSummaryPdf() {
        $shift_id = $_COOKIE["cash-collect-shift-id"];

        // print_r($shift_id);die;

        $db = Yii::$app->db;

        $ff = $db->createCommand("SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY','')) ")->execute();

        $vehicle_category_count = Yii::$app->db->createCommand("SELECT VL.vehicle_type as name,SUM(VL.vehicle_count) as vCount,SUM(VL.amount) as vAmount
            FROM tbl_cash_collect CC
            INNER JOIN cash_collect_vehicle_list VL ON VL.cash_collect_id = CC.id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id
            GROUP BY VL.vehicle_type
            ORDER by VL.id")->queryAll();
        $manual_ticket_count = Yii::$app->db->createCommand("SELECT VL.vehicle_type as name,SUM(VL.vehicle_count) as vCount,SUM(VL.amount) as vAmount
            FROM tbl_cash_collect CC
            INNER JOIN cash_collect_manual_vehicle_list VL ON VL.cash_collect_id = CC.id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id
            GROUP BY VL.vehicle_type
            ORDER by VL.id")->queryAll();
        $escape_ticket_count = Yii::$app->db->createCommand("SELECT VL.vehicle_type as name,SUM(VL.vehicle_count) as vCount,SUM(VL.amount) as vAmount
            FROM tbl_cash_collect CC
            INNER JOIN cash_collect_misused_vehicle_list VL ON VL.cash_collect_id = CC.id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id
            GROUP BY VL.vehicle_type
            ORDER by VL.id")->queryAll();
        $vehicle_category_count_without_escape = Yii::$app->db->createCommand("SELECT VL.vehicle_type as name,SUM(VL.vehicle_count) as vCount,SUM(VL.amount) as vAmount
            FROM tbl_cash_collect CC
            INNER JOIN cash_collect_whitout_escape_vehicle_list VL ON VL.cash_collect_id = CC.id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id
            GROUP BY VL.vehicle_type
            ORDER by VL.id")->queryAll();

        $cashBalance = Yii::$app->db->createCommand("SELECT sum(CC.total_system_amount) as sysTotal,SUM(CC.total_cash_amount) as cashTotal,SUM(CC.total_qr_amount) AS qrTotal
            FROM tbl_cash_collect CC
            WHERE CC.status = 1 AND CC.shift_id = $shift_id")->queryOne();

        $comments = Yii::$app->db->createCommand("SELECT CONCAT (LS.login_time,' - ',LS.logout_time) as sess,U.username as uName,CC.comment as comment
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN `user` U ON U.id = LS.user_id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id")->queryAll();

        $totalVehivleCount = Yii::$app->db->createCommand("SELECT SUM(VL.vehicle_count) as vCount
            FROM tbl_cash_collect CC
            INNER JOIN cash_collect_vehicle_list VL ON VL.cash_collect_id = CC.id
            WHERE CC.status = 1 AND CC.shift_id = $shift_id")->queryScalar();

        $sysTotal = $cashBalance['sysTotal'];
        $cashTotal = $cashBalance['cashTotal'];
        $qrTotal = $cashBalance['qrTotal'];
        //$balance = $cashTotal - $sysTotal;
        //$balance = $cashBalance['cashTotal'] - $cashBalance['sysTotal'] + $cashBalance['qrTotal'];
        $balance = $cashBalance['cashTotal'] + $cashBalance['qrTotal'] - $cashBalance['sysTotal'];

        // Get cash reset count   
        $cash_reset = Yii::$app->db->createCommand("
            SELECT COUNT(*) 
            FROM tbl_cash_collect CC 
            WHERE CC.status = 0 AND CC.shift_id = :shift_id")
                ->bindValue(':shift_id', $shift_id)
                ->queryScalar();

        // Get towing count    
        /* $towing_ticket = Yii::$app->db->createCommand(
          "SELECT COUNT(*)
          FROM tbl_towing TW
          WHERE TW.shift_id = :shift_id")
          ->bindValue(':shift_id', $shift_id)
          ->queryScalar(); */


        $towing_list = Towing::find()->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $total_towing_amount = 0;
        $towing_count = 0;

        foreach ($towing_list as $tlist) {
            $total_towing_amount += $tlist->amount;
            $towing_count++;
        }

        // Duplicate exit ticket
        $duplicate_ticket = Yii::$app->db->createCommand("
            WITH DuplicateTickets AS (
                SELECT 
                    EX1.entrance_serial,
                    COUNT(EX1.id) AS serialCount
                FROM 
                    tbl_exit_ticket EX1
                WHERE 
                    EX1.exit_time <= NOW() + INTERVAL 2 HOUR 
                    AND DATE(EX1.exit_time) = CURRENT_DATE
                    AND EX1.shift_id = :shift_id
                GROUP BY 
                    EX1.entrance_serial
                HAVING 
                    COUNT(EX1.id) > 1
            ),
            FirstTicketCounts AS (
                SELECT 
                    EX2.entrance_serial,
                    COUNT(EX2.id) AS firstTicketCount
                FROM 
                    tbl_exit_ticket EX2
                WHERE 
                    EX2.id IN (
                        SELECT MIN(EX3.id)
                        FROM tbl_exit_ticket EX3
                        WHERE EX3.exit_time <= NOW() + INTERVAL 2 HOUR 
                        AND DATE(EX3.exit_time) = CURRENT_DATE
                        AND EX3.shift_id = :shift_id
                        GROUP BY EX3.entrance_serial
                    )
                GROUP BY 
                    EX2.entrance_serial
            )
            SELECT 
                (SUM(DT.serialCount) - COALESCE(SUM(FTC.firstTicketCount), 0)) AS tCount
            FROM 
                DuplicateTickets DT
            LEFT JOIN 
                FirstTicketCounts FTC ON DT.entrance_serial = FTC.entrance_serial
        ")->bindValue(':shift_id', $shift_id)
                ->queryScalar();

        // Get special ticket types data
        $tickets_data = ExitTicket::find()
                ->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $ticket_type_status = TicketTypeStatus::find()->all();
        $ticket_types_count = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket->ticket_type_status_id == $ticket_type->id) {
                    $type_count++;
                    $type_wise_amount += $ticket->amount;
                }
            }
            $ticket_types_count[] = [
                "name" => $ticket_type->name,
                "count" => $type_count,
                "amount" => $type_wise_amount
            ];
        }

        $mpdf = new \Mpdf\Mpdf();
        $mpdf->setFooter('Report generated by ' . Yii::$app->getUser()->username . '<br> on ' . date('Y-m-d H:i:s') . ' || Page {PAGENO} of {nb}');
        $mpdf->WriteHTML($this->renderPartial('cash-collect-summary-pdf', [
                    'vehicle_category_count' => $vehicle_category_count,
                    'manual_ticket_count' => $manual_ticket_count,
                    'escape_ticket_count' => $escape_ticket_count,
                    'vehicle_category_count_without_escape' => $vehicle_category_count_without_escape,
                    'cashBalance' => $cashBalance,
                    'sysTotal' => $sysTotal,
                    'cashTotal' => $cashTotal,
                    'qrTotal' => $qrTotal,
                    'balance' => $balance,
                    'comments' => $comments,
                    'totalVehivleCount' => $totalVehivleCount,
                    'ticket_types_count' => $ticket_types_count,
                    'cash_reset' => $cash_reset,
                    'total_towing_amount' => $total_towing_amount,
                    'towing_count' => $towing_count,
                    'duplicate_ticket' => $duplicate_ticket,
                    'shift_id' => $shift_id,
        ]));
        $mpdf->Output('Cash Collect Summary Report ' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

    public function actionCashCollectSummaryAts() {
        $model = new CashCollectSummaryAts;
        if (Yii::$app->request->post()) {

            $model->load(Yii::$app->request->post());

            if ($model->ic == 'Any') {

                $results = Yii::$app->db->createCommand("SELECT S.*, IC.name AS ic_name
            FROM tbl_shift S
            INNER JOIN tbl_ic IC ON S.ic_id = IC.id
            WHERE S.schedule_id IN (SELECT SC.id
                FROM tbl_schedule SC
                WHERE SC.date = '" . $model->date . "')")->queryAll();
            } else {

                $results = Yii::$app->db->createCommand("SELECT S.*, IC.name AS ic_name
            FROM tbl_shift S
            INNER JOIN tbl_ic IC ON S.ic_id = IC.id
            WHERE S.ic_id = $model->ic AND S.schedule_id IN (SELECT SC.id
                FROM tbl_schedule SC
                WHERE SC.date = '" . $model->date . "')")->queryAll();
            }

            // Preload IC options for the selected region
            $icOptions = $model->region ? IC::getICList($model->region) : [];

            return $this->render('cash-collect-summary-ats', ['model' => $model, 'results' => $results, 'icOptions' => $icOptions,]);
        } else {

            return $this->render('cash-collect-summary-ats', ['model' => $model, 'icOptions' => [],]);
        }
    }

    public function actionGetSummaryAts() {
        $shift_id = $_POST['shift_id'];
        setcookie("cash-collect-shift-id-ats", $shift_id, time() + 86400);

        $shift_type = Shift::find()->where(['id' => $shift_id])->one()->shift_name;
        if ($shift_type == 'EVENING SHIFT') {
            $day_type = 'Only Night';
            $shift_type_2 = 'NIGHT SHIFT';
        } else {
            $day_type = 'Only Day';
            $shift_type_2 = 'MORNING SHIFT';
        }

        /* $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,(CC.total_system_amount) as cash,IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short
          FROM tbl_cash_collect CC
          INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
          INNER JOIN tbl_booth B ON B.id = LS.booth_id
          INNER JOIN user U ON U.id = LS.user_id
          WHERE CC.shift_id = $shift_id
          ORDER BY B.name,LS.login_time")->queryAll(); */

        /*  $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,(CC.total_system_amount+CC.not_sync_amount ) as cash,IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short
          FROM tbl_cash_collect CC
          INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
          INNER JOIN tbl_booth B ON B.id = LS.booth_id
          INNER JOIN user U ON U.id = LS.user_id
          INNER JOIN tbl_shift S ON S.id = $shift_id
          WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')
          ORDER BY B.name,LS.login_time")->queryAll(); */

        /*   $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,

          (SELECT SUM(amount) FROM tbl_exit_ticket et WHERE et.session_uuid = LS.session_uuid AND et.ticket_type_status_id NOT IN (2,8,12)) as cash,

          IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short
          FROM tbl_cash_collect CC
          INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id

          INNER JOIN tbl_booth B ON B.id = LS.booth_id
          INNER JOIN user U ON U.id = LS.user_id
          INNER JOIN tbl_shift S ON S.id = 59714
          WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')

          ORDER BY B.name,LS.login_time")->queryAll();




          $towing_charge_during_shift = Yii::$app->db->createCommand("SELECT IFNULL(SUM(T.amount),0)
          FROM tbl_towing T
          WHERE T.shift_id = $shift_id")->queryScalar();
         */

        $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,(CC.total_system_amount+CC.not_sync_amount ) as cash,IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short,IFNULL(CC.total_qr_amount,0) as qr_amount,LS.session_uuid
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryAll();

        $data_incomplete = Yii::$app->db->createCommand("SELECT COUNT(*) FROM `tbl_login_logout_session` LS 
    
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE LS.id NOT IN 
            (SELECT tbl_cash_collect.actual_session_id from tbl_cash_collect WHERE tbl_cash_collect.status=1) 
            AND B.booth_type=2 AND LS.confirmed_status=0 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $session_incomplete = Yii::$app->db->createCommand("SELECT COUNT(*) FROM `tbl_login_logout_session` LS 
    
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE LS.id NOT IN 
            (SELECT tbl_cash_collect.actual_session_id from tbl_cash_collect WHERE tbl_cash_collect.status=1) 
            AND B.booth_type=2 AND LS.confirmed_status=0 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $cash_reset = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status = 0 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $inactive_ic_ticket = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_exit_ticket EX 
            INNER JOIN tbl_ic IC1 ON IC1.id = EX.entrance_ic
            INNER JOIN tbl_booth B ON B.id = EX.booth_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE IC1.status = 0  AND EX.created_on >= ADDTIME(S.shift_start,'-00:15:00.0000') AND EX.created_on <= ADDTIME(S.shift_start,'-00:15:00.0000')")->queryScalar();

        $cash_income = Yii::$app->db->createCommand("SELECT SUM(CC.total_cash_amount) AS cash_income 
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $qr_income = Yii::$app->db->createCommand("SELECT SUM(CC.total_qr_amount) AS qr_income 
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $towing_ticket = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_towing T
            WHERE T.shift_id = $shift_id")->queryScalar();

        $duplicate_ticket = Yii::$app->db->createCommand("
    WITH DuplicateTickets AS (
        SELECT 
            EX1.entrance_serial,
            COUNT(EX1.id) AS serialCount
        FROM 
            tbl_exit_ticket EX1
        WHERE 
            EX1.exit_time <= NOW() + INTERVAL 2 HOUR 
            AND DATE(EX1.exit_time) = CURRENT_DATE
            AND EX1.shift_id = :shift_id
        GROUP BY 
            EX1.entrance_serial
        HAVING 
            COUNT(EX1.id) > 1
    ),
    FirstTicketCounts AS (
        SELECT 
            EX2.entrance_serial,
            COUNT(EX2.id) AS firstTicketCount
        FROM 
            tbl_exit_ticket EX2
        WHERE 
            EX2.id IN (
                SELECT MIN(EX3.id)
                FROM tbl_exit_ticket EX3
                WHERE EX3.exit_time <= NOW() + INTERVAL 2 HOUR 
                AND DATE(EX3.exit_time) = CURRENT_DATE
                AND EX3.shift_id = :shift_id
                GROUP BY EX3.entrance_serial
            )
        GROUP BY 
            EX2.entrance_serial
    )
    SELECT 
        (SUM(DT.serialCount) - COALESCE(SUM(FTC.firstTicketCount), 0)) AS tCount
    FROM 
        DuplicateTickets DT
    LEFT JOIN 
        FirstTicketCounts FTC ON DT.entrance_serial = FTC.entrance_serial
")->bindValue(':shift_id', $shift_id)->queryScalar();
        if (empty($duplicate_ticket)) {
            $duplicate_ticket = 0;
        }

        // Get special ticket types data
        $tickets_data = ExitTicket::find()
                ->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $ticket_type_status = TicketTypeStatus::find()->all();
        $ticket_types_count = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket->ticket_type_status_id == $ticket_type->id) {
                    $type_count++;
                    $type_wise_amount += $ticket->amount;
                }
            }
            $ticket_types_count[] = [
                "name" => $ticket_type->name,
                "count" => $type_count,
                "amount" => $type_wise_amount
            ];
        }

        $accident_damage_claims_amount = Yii::$app->db->createCommand("SELECT IFNULL(SUM(A.amount),0)
            FROM tbl_accident A
            WHERE A.shift_id = $shift_id")->queryScalar();
            

        $booth_name = '';
        $i = 0;
        $total_booth_collection_amount = 0;
        $total_excess_and_short = 0;
        $total_amount = 0;
        $excess_and_short = 0;
        $ttl = 0;
        $grand_total_booth_collection_amount = 0;
        $grand_total_excess_and_short = 0;
        $html = '';
        $total_qr_income = 0;
        $total_towing_charge_during_shift = 0;

        foreach ($data as $d) {

            $towing_charge_during_shift = Yii::$app->db->createCommand("SELECT IFNULL(SUM(T.amount),0)
                            FROM tbl_towing T
                            WHERE T.shift_id = $shift_id AND T.session_uuid ='" . $d['session_uuid'] . "'")->queryScalar();

            $html .= '<tr>';
            $html .= '<td>' . $d['name'] . '</td>';
            $html .= '<td>' . $d['username'] . '</td>';
            $html .= '<td>' . $d['timerange'] . '</td>';
            $cash_user_fee = $d['cash'] - ($d['qr_amount'] + $towing_charge_during_shift);

            $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency($cash_user_fee) . '</td>';

            if ($d['excess'] == 0 && $d['short'] == 0) {
                $html .= '<td style="text-align:right;">0</td>';
                $excess_and_short = 0;
            } else {
                if ($d['excess'] != 0) {
                    $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency($d['excess']) . '</td>';
                    $excess_and_short = $d['excess'];
                    $excess_and_short = (float) $excess_and_short;
                }
                if ($d['short'] != 0) {
                    $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency(- $d['short']) . '</td>';
                    $excess_and_short = $d['short'];
                    $excess_and_short = (float) $excess_and_short * -1;
                }
            }
            $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency($d['qr_amount']) . '</td>';
            $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency($towing_charge_during_shift) . '</td>';
            $ttl = (float) $d['cash'] + $excess_and_short;
            $html .= '<td style="text-align:right;">' . Yii::$app->formatter->asCurrency($ttl) . '</td>';
            $html .= '<td></td>';
            $html .= '<td></td>';

            $total_booth_collection_amount += (float) $cash_user_fee;
            $total_excess_and_short += $excess_and_short;
            $total_amount += $ttl;
            $total_qr_income += $d['qr_amount'];
            $total_towing_charge_during_shift += $towing_charge_during_shift;

            $grand_total_booth_collection_amount += (float) $d['cash'];
            $grand_total_excess_and_short += $excess_and_short;
            $html .= '</tr>';

            ++$i;
        }
        $html .= '<tr>';
        $html .= '<td colspan="3" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">Total Amount</td>';
        $html .= '<td style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_booth_collection_amount) . '</td>';
        $html .= '<td style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_excess_and_short) . '</td>';
        $html .= '<td style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_qr_income) . '</td>';
        $html .= '<td style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_towing_charge_during_shift) . '</td>';
        $html .= '<td style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_amount) . '</td>';
        $html .= '<td></td>';
        $html .= '<td></td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="10" style="text-align:center;font-weight: bold;">SUMMARY OF ' . $shift_type_2 . '</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;">Total Booth Revenue (Rs.)</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;">' . $day_type . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($total_amount) . '</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;">Accident Damage Claims Amount (Rs.)</td>';
        $html .= '<td colspan="1" style="text-align:left;font-weight: bold;">Receipt No</td>';
        $html .= '<td colspan="1" style="text-align:center;font-weight: bold;">' . $day_type . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($accident_damage_claims_amount) . '</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;background-color: #f1f4f7;">Total Amount of Shift</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . $day_type . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . Yii::$app->formatter->asCurrency($total_amount + $accident_damage_claims_amount) . '</td>';
        $html .= '</tr>';

        // Close the table and return the HTML
        $html .= '<tr>';
        $html .= '<td colspan="10" style="text-align:center;font-weight: bold;"> SPECIAL TICKET TYPES</td>';
        $html .= '</tr>';

        // Add the special ticket types counts to the summary
        foreach ($ticket_types_count as $ticket_type) {
            $html .= '<tr>';
            $html .= '<td colspan="4" style="text-align:left;font-weight: bold;">' . $ticket_type["name"] . ' </td>';
            $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . '' . '</td>';
            $html .= '<td colspan="4" style="text-align:right;font-weight: bold;">' . $ticket_type["count"] . '</td>';
            $html .= '</tr>';

            // $html .= '<tr>';
            // $html .= '<td colspan="4" style="text-align:left;font-weight: bold;">' . $ticket_type["name"] . ' Amount (Rs.)</td>';
            // $html .= '<td colspan="1" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">'.''.'</td>';
            // $html .= '<td colspan="3" style="text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($ticket_type["amount"]) . '</td>';
            // $html .= '</tr>';
        }
        /////////////////////

        $html .= '<tr>';
        $html .= '<td colspan="10" style="text-align:center;font-weight: bold;">TICKET COUNT TYPES</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;background-color: #f1f4f7;">Total Incomplete sessions in Shift</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . '' . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . $data_incomplete . '</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;background-color: #f1f4f7;">Incomplete Session Count</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . '' . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . $session_incomplete . '</td>';
        $html .= '</tr>';

        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;background-color: #f1f4f7;">Cash Reset Count</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . '' . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . $cash_reset . '</td>';
        $html .= '</tr>';


        $html .= '<tr>';
        $html .= '<td colspan="4" style="text-align:left;font-weight: bold;background-color: #f1f4f7;">Duplicate Ticket Count</td>';
        $html .= '<td colspan="2" style="text-align:center;font-weight: bold;background-color: #f1f4f7;">' . '' . '</td>';
        $html .= '<td colspan="4" style="text-align:right;font-weight: bold;background-color: #f1f4f7;">' . $duplicate_ticket . '</td>';
        $html .= '</tr>';

        if ($shift_type == 'EVENING SHIFT') {
            $morning = Yii::$app->db->createCommand("SELECT SUM(CC.total_system_amount) AS total_system_amount,SUM(CC.not_sync_amount) AS not_sync_amount,SUM(CC.excess) AS excess,SUM(CC.short) AS short,SUM(IFNULL(CC.total_qr_amount,0)) as qr_amount
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = (SELECT S2.id
                    FROM tbl_shift S
                    INNER JOIN tbl_shift S2 ON S2.schedule_id = S.schedule_id AND S2.ic_id = S.ic_id AND S2.shift_name = 'MORNING SHIFT' AND S2.id != S.id
                    WHERE S.id = $shift_id)
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')
            ORDER BY B.name,LS.login_time")->queryAll();

            $towing_charge_during_morning_shift = Yii::$app->db->createCommand("SELECT IFNULL(SUM(T.amount),0)
            FROM tbl_towing T
            INNER JOIN tbl_login_logout_session LS ON LS.session_uuid = T.session_uuid
            INNER JOIN tbl_cash_collect cs ON LS.id =cs.actual_session_id
            WHERE T.shift_id = (SELECT S2.id
                    FROM tbl_shift S
                    INNER JOIN tbl_shift S2 ON S2.schedule_id = S.schedule_id AND S2.ic_id = S.ic_id AND S2.shift_name = 'MORNING SHIFT' AND S2.id != S.id
                    WHERE S.id = $shift_id) AND cs.status =1")->queryScalar();

            $accident_damage_claims_during_morning_shift = Yii::$app->db->createCommand("SELECT IFNULL(SUM(A.amount),0)
            FROM tbl_accident A
            WHERE A.shift_id = (SELECT S2.id
                    FROM tbl_shift S
                    INNER JOIN tbl_shift S2 ON S2.schedule_id = S.schedule_id AND S2.ic_id = S.ic_id AND S2.shift_name = 'MORNING SHIFT' AND S2.id != S.id
                    WHERE S.id = $shift_id)")->queryScalar();

            $cash_amount = (float) $morning[0]['total_system_amount'];
            $not_sync_amount = (float) $morning[0]['not_sync_amount'];
            $excess = (float) $morning[0]['excess'];
            $short = (float) $morning[0]['short'] * -1;
            $qr_amount_morning_shift = (float) $morning[0]['qr_amount'];

            $total_qr_amount = $total_qr_income+ $qr_amount_morning_shift;
            $grand_total_booth_collection_amount += ($cash_amount + $not_sync_amount);
            $grand_total_booth_collection_amount = $grand_total_booth_collection_amount - $total_qr_amount;
            $grand_total_excess_and_short += ($excess + $short);
            $total_towimg_charge = $total_towing_charge_during_shift + $towing_charge_during_morning_shift;
            $total_accident_damage_amount = $accident_damage_claims_amount + $accident_damage_claims_during_morning_shift;
            $toBank = $grand_total_booth_collection_amount + $grand_total_excess_and_short;
            $Total_revenu_amount = $toBank + $total_towimg_charge + $total_accident_damage_amount;

            $html .= '<tr colspan="10" style="text-align:left;font-weight: bold;height:10px;">';
            $html .= '</tr>';

            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total User Fee Collection  Without Excess/Short Money Amount (Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($grand_total_booth_collection_amount) . '</td>';
            $html .= '</tr>';
            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total Excess/Short Money Amount (Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($grand_total_excess_and_short) . '</td>';
            $html .= '</tr>';
            $html .= '</tr>';
            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total QR Amount (Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($total_qr_amount) . '</td>';

            $html .= '</tr>';
            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total User Fee Collection With Excess/Short Money Amount (Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($toBank) . '</td>';

            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total Towing Charges Collection Amount(Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($total_towimg_charge) . '</td>';
            $html .= '</tr>';
            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">Total Accident Damage Claims Amount(Rs.)</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($total_accident_damage_amount) . '</td>';
            $html .= '</tr>';
            $html .= '<tr colspan="10" style="text-align:left;font-weight: bold;height:10px;"></tr>';
            $html .= '<tr>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:left;font-weight: bold;">TOTAL REVENUE DEPOSITED AT THE BANK</td>';
            $html .= '<td colspan="2" style="background-color: #f1f4f7;text-align:center;font-weight: bold;">Day & Night</td>';
            $html .= '<td colspan="4" style="background-color: #f1f4f7;text-align:right;font-weight: bold;">' . Yii::$app->formatter->asCurrency($Total_revenu_amount) . '</td>';
            $html .= '</tr>';
        }
        print_r($html);
    }

    public function actionCashCollectSummaryAtsPdf() {
        $shift_id = $_COOKIE["cash-collect-shift-id-ats"];

        /* $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,(CC.total_system_amount) as cash,IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short
          FROM tbl_cash_collect CC
          INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
          INNER JOIN tbl_booth B ON B.id = LS.booth_id
          INNER JOIN user U ON U.id = LS.user_id
          WHERE CC.shift_id = $shift_id
          ORDER BY B.name,LS.login_time")->queryAll(); */

        $data = Yii::$app->db->createCommand("SELECT B.name,U.username,CONCAT(time(LS.login_time),' - ',time(LS.logout_time)) as timerange,(CC.total_system_amount+CC.not_sync_amount ) as cash,IFNULL(CC.excess,0) AS excess,IFNULL(CC.short,0) as short,IFNULL(CC.total_qr_amount,0) as qr_amount,LS.session_uuid
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000') 
            ORDER BY B.name,LS.login_time")->queryAll();

        $session_incomplete = Yii::$app->db->createCommand("SELECT COUNT(*) FROM `tbl_login_logout_session` LS 
                
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE LS.id NOT IN 
            (SELECT tbl_cash_collect.actual_session_id from tbl_cash_collect WHERE tbl_cash_collect.status=1) 
            AND B.booth_type=2 AND LS.confirmed_status=0 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $cash_reset = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status = 0 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $inactive_ic_ticket = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_exit_ticket EX 
            INNER JOIN tbl_ic IC1 ON IC1.id = EX.entrance_ic
            INNER JOIN tbl_booth B ON B.id = EX.booth_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE IC1.status = 0  AND EX.created_on >= ADDTIME(S.shift_start,'-00:15:00.0000') AND EX.created_on <= ADDTIME(S.shift_start,'-00:15:00.0000')")->queryScalar();

        $cash_income = Yii::$app->db->createCommand("SELECT SUM(CC.total_cash_amount) AS cash_income 
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $qr_income = Yii::$app->db->createCommand("SELECT SUM(CC.total_qr_amount) AS qr_income 
            FROM tbl_cash_collect CC
            INNER JOIN tbl_login_logout_session LS ON LS.id = CC.actual_session_id
            INNER JOIN tbl_booth B ON B.id = LS.booth_id
            INNER JOIN user U ON U.id = LS.user_id
            INNER JOIN tbl_shift S ON S.id = $shift_id
            WHERE  CC.status=1 AND S.ic_id = B.ic_id AND LS.login_time >= ADDTIME(S.shift_start,'-00:15:00.0000') AND LS.login_time <= ADDTIME(S.shift_end,'-00:15:00.0000')  
            ORDER BY B.name,LS.login_time")->queryScalar();

        $towing_ticket = Yii::$app->db->createCommand("SELECT COUNT(*) FROM tbl_towing T
            WHERE T.shift_id = $shift_id")->queryScalar();

        $towing_charge_during_shift = Yii::$app->db->createCommand("SELECT IFNULL(SUM(T.amount),0)
            FROM tbl_towing T
            WHERE T.shift_id = $shift_id")->queryScalar();

        $accident_damage_claims_amount = Yii::$app->db->createCommand("SELECT IFNULL(SUM(A.amount),0)
            FROM tbl_accident A
            WHERE A.shift_id = $shift_id")->queryScalar();

        $duplicate_ticket = Yii::$app->db->createCommand("
            WITH DuplicateTickets AS (
                SELECT 
                    EX1.entrance_serial,
                    COUNT(EX1.id) AS serialCount
                FROM 
                    tbl_exit_ticket EX1
                WHERE 
                    EX1.exit_time <= NOW() + INTERVAL 2 HOUR 
                    AND DATE(EX1.exit_time) = CURRENT_DATE
                    AND EX1.shift_id = :shift_id
                GROUP BY 
                    EX1.entrance_serial
                HAVING 
                    COUNT(EX1.id) > 1
            ),
            FirstTicketCounts AS (
                SELECT 
                    EX2.entrance_serial,
                    COUNT(EX2.id) AS firstTicketCount
                FROM 
                    tbl_exit_ticket EX2
                WHERE 
                    EX2.id IN (
                        SELECT MIN(EX3.id)
                        FROM tbl_exit_ticket EX3
                        WHERE EX3.exit_time <= NOW() + INTERVAL 2 HOUR 
                        AND DATE(EX3.exit_time) = CURRENT_DATE
                        AND EX3.shift_id = :shift_id
                        GROUP BY EX3.entrance_serial
                    )
                GROUP BY 
                    EX2.entrance_serial
            )
            SELECT 
                (SUM(DT.serialCount) - COALESCE(SUM(FTC.firstTicketCount), 0)) AS tCount
            FROM 
                DuplicateTickets DT
            LEFT JOIN 
                FirstTicketCounts FTC ON DT.entrance_serial = FTC.entrance_serial
        ")->bindValue(':shift_id', $shift_id)->queryScalar();
        if (empty($duplicate_ticket)) {
            $duplicate_ticket = 0;
        }
        // Get special ticket types data
        $tickets_data = ExitTicket::find()
                ->where(['shift_id' => $shift_id])
                ->orderBy(['id' => SORT_DESC])
                ->all();

        $ticket_type_status = TicketTypeStatus::find()->all();
        $ticket_types_count = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket->ticket_type_status_id == $ticket_type->id) {
                    $type_count++;
                    $type_wise_amount += $ticket->amount;
                }
            }
            $ticket_types_count[] = [
                "name" => $ticket_type->name,
                "count" => $type_count,
                "amount" => $type_wise_amount
            ];
        }


        $mpdf = new \Mpdf\Mpdf();
        $mpdf->setFooter('Report generated by ' . Yii::$app->getUser()->username . '<br> on ' . date('Y-m-d H:i:s') . ' || Page {PAGENO} of {nb}');
        $mpdf->WriteHTML($this->renderPartial('cash-collect-summary-ats-pdf', [
                    'data' => $data,
                    'session_incomplete' => $session_incomplete,
                    'inactive_ic_ticket' => $inactive_ic_ticket,
                    'cash_reset' => $cash_reset,
                    'cash_income' => $cash_income,
                    'qr_income' => $qr_income,
                    'towing_ticket' => $towing_ticket,
                    'accident_damage_claims_amount' => $accident_damage_claims_amount,
                    'duplicate_ticket' => $duplicate_ticket,
                    'ticket_types_count' => $ticket_types_count,
                    'shift_id' => $shift_id,
        ]));
        $mpdf->Output('Cash Collect Summary Report ' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

    public function beforeAction($action) {
        $this->enableCsrfValidation = false;
        return parent::beforeAction($action);
    }

    public function actionResetCashIn() {

        // $id = ($_POST['cash_in_id']);
        // $result = Yii::$app->db->createCommand('UPDATE tbl_cash_collect SET status = 0 WHERE id=' . $id . '')
        //     ->execute();

        $id = Yii::$app->request->post('cash_in_id');

        $result = Yii::$app->db->createCommand('UPDATE tbl_cash_collect SET status = 0 WHERE id=:id')
                ->bindValue(':id', $id)
                ->execute();

        return true;
    }

    public function actionResetCount() {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;

        $id = Yii::$app->request->post('id');
        $userId = Yii::$app->user->id;
        $resetAt = date('Y-m-d H:i:s');

        // $result = Yii::$app->db->createCommand('UPDATE tbl_cash_collect SET reset_count = reset_count + 1 WHERE id = :id')
        //         ->bindValue(':id', $id)
        //         ->execute();

        $result = Yii::$app->db->createCommand()
                ->update('tbl_cash_collect', [
                    'reset_count' => new \yii\db\Expression('reset_count + 1'),
                    'last_reset_at' => $resetAt,
                    'last_reset_by' => $userId,
                        ], ['id' => $id])
                ->execute();
    }

    public function actionForceLogout() {

        $id = ($_POST['login_logout_session_id']);
        $date_now = date("Y-m-d H:i:s");

        $result = Yii::$app->db->createCommand('UPDATE tbl_login_logout_session SET
        status = 0,
        logout_time ="' . $date_now . '",
        logout_type = "BY_ADMIN"
        WHERE id=' . $id . '')
                ->execute();

        return true;
    }

    public function actionViewEn($user_id, $start_date, $end_date, $actual_session_id) {
        ini_set('memory_limit', '-1');
        $miss_issued_ticket = new MissIssuedTicketsEntrance;

        $saveButtenEnable = 1;

        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $saveButtenEnable = 1;
        } else if (User::hasRole('95')) {
            $saveButtenEnable = 0;
        } else if (User::hasRole('65')) {
            $saveButtenEnable = 0;
        }

        if ($actual_session_id) {

            $actual_session_model = $this->findModel($actual_session_id);
            $tickets_data = EntranceTicket::find()
                    ->where(['session_uuid' => $actual_session_model->session_uuid])
                    ->orderBy(['id' => SORT_DESC])
                    ->all();
            $total_ticket_count = count($tickets_data);

            $ticket_types_count = [];
            $vehicle_category_count = [];

            $vehicle_category_count = Yii::$app->db->createCommand('SELECT (SELECT name FROM tbl_vehicle_category WHERE id =vehicle_category) as vehicle_category_name,vehicle_category,COUNT(vehicle_category) as count FROM `tbl_entrance_ticket` WHERE session_uuid="' . $actual_session_model->session_uuid . '"  group by vehicle_category')->queryAll();
            $ticket_types_count = Yii::$app->db->createCommand('SELECT (SELECT name FROM tbl_ticket_type_status WHERE id =ticket_type_status_id) as type,ticket_type_status_id,COUNT(ticket_type_status_id) as count FROM `tbl_entrance_ticket` WHERE session_uuid="' . $actual_session_model->session_uuid . '" group by ticket_type_status_id')->queryAll();
        }

        $new_start_date = new DateTime($start_date);
        $new_start_date = $new_start_date->format("Y-m-d 00:00:00");
        $new_end_date = new DateTime($end_date);
        $new_end_date = $new_end_date->format("Y-m-d 23:59:59");
        $dataLoginLogoutPairsProvider = LoginLogoutSession::find()->joinWith(['booth'])->where([
                    'and',
                    [
                        'user_id' => $user_id,
                        'tbl_booth.booth_type' => 1,
                    ],
                    ['between', 'login_time', $new_start_date, $new_end_date],
                ])->orderBy(['id' => SORT_DESC])->all();

        // submit cash
        //    if ($cash_collect->load(Yii::$app->request->post())  ) {
//         $cash_collect->vehicle_count = $vehicle_category_count;
        //         $login_logout_session = LoginLogoutSession::find()->where(['id' => $actual_session_id])->one();
        //         $cash_collect->shift_id = $login_logout_session->shift_id;
        //        $ic_id = $login_logout_session->booth->ic->id;
//         CashCollect::updateAll(['status' => 0], ['=', 'actual_session_id', $actual_session_id]);
//         if($cash_collect->save()){
//             return $this->redirect(['view-en', 'user_id' => $user_id,'start_date'=>$start_date,'end_date'=>$end_date,'actual_session_id'=>$actual_session_model->id]);
//        }
        //     }
        // submit MISS Issued Ticket
        if ($miss_issued_ticket->load(Yii::$app->request->post())) {

            $entrance_ticket = EntranceTicket::find()->where(['id' => $miss_issued_ticket->entrance_ticket_id])->one();
            $miss_issued_ticket->previous_ticket_type_status = $entrance_ticket->ticket_type_status_id;

            if ($miss_issued_ticket->save()) {
                $res = Yii::$app->db->createCommand('UPDATE tbl_entrance_ticket SET ticket_type_status_id=12 WHERE id=' . $miss_issued_ticket->entrance_ticket_id . '')->execute();
                Yii::$app->getSession()->setFlash('success', 'Miss Issued Ticket added');
                return $this->redirect(['view-en', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => $actual_session_model->id]);
            }
        }

        return $this->render('view-en', [
                    'model' => $this->findUser($user_id),
                    'dataLoginLogoutPairsProvider' => $dataLoginLogoutPairsProvider,
                    'start_date' => $start_date,
                    'end_date' => $end_date,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'exit_ticket' => $exit_ticket,
                    'miss_issued_ticket' => $miss_issued_ticket,
                    'all_categpory_ticket_count' => $all_categpory_ticket_count,
                    'saveButtenEnable' => $saveButtenEnable,
        ]);
    }

    public function actionQuickEn() {
        $user_id = ($_POST['user_id']);
        $start_date = ($_POST['start_date']);
        $end_date = ($_POST['end_date']);
        $this->redirect(['view-en', 'user_id' => $user_id, 'start_date' => $start_date, 'end_date' => $end_date, 'actual_session_id' => ""]);
    }

    public function actionExportPdfEntrance($user_id, $actual_session_id) {

        $actual_session_model = $this->findModel($actual_session_id);

        $tickets_data = EntranceTicket::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $total_ticket_count = count($tickets_data);

        $ticket_types_count = [];
        $vehicle_category_count = [];

        $vehicle_category_count = Yii::$app->db->createCommand('SELECT (SELECT name FROM tbl_vehicle_category WHERE id =vehicle_category) as vehicle_category_name,vehicle_category,COUNT(vehicle_category) as count FROM `tbl_entrance_ticket` WHERE session_uuid="' . $actual_session_model->session_uuid . '" AND ticket_type_status_id NOT IN(12)  group by vehicle_category')->queryAll();
        $ticket_types_count = Yii::$app->db->createCommand('SELECT (SELECT name FROM tbl_ticket_type_status WHERE id =ticket_type_status_id) as type,ticket_type_status_id,COUNT(ticket_type_status_id) as count FROM `tbl_entrance_ticket` WHERE session_uuid="' . $actual_session_model->session_uuid . '" group by ticket_type_status_id')->queryAll();

        $user = $this->findUser($user_id);

        $mpdf = new \Mpdf\Mpdf([
            // 'orientation' => 'L',
            'setAutoTopMargin' => 'pad',
        ]);

        $mpdf->setFooter('Report generated by ' . Yii::$app->getUser()->username . '<br> on ' . date('Y-m-d H:i:s') . ' || Page {PAGENO} of {nb}');
        $mpdf->WriteHTML($this->renderPartial('pdf-entrance', [
                    'user' => $user,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_amount' => $total_amount,
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'total_rev' => $total_rev,
                    'balance' => ($total_amount - $cash_collected_data->total_cash_amount),
        ]));
        $mpdf->Output('Cash Collect ENTRANCE Summary Report (Teller-' . $user->username . ')' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

    public function actionRemove() {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $id = Yii::$app->request->post('id');
        if ($id) {
            $model = ExitTicket::findOne($id);
            if ($model && $model->delete()) {
                return ['success' => true];
            }
        }
        return ['success' => false];
    }

    public function actionExportOnline($user_id, $actual_session_id) {
        $teller_confirm = new TellerConfirm();
        $actual_session_model = $this->findModel($actual_session_id);

        $SessionReportConfirm = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                    'active' => 1
                ])->orderBy(['id' => SORT_DESC])->one();

        $SessionReportConfirmAll = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $tickets_data = ExitTicket::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $total_amount = 0;
//print_r($tickets_data);die;

        $ticket_type_status = TicketTypeStatus::find()->all();
        $vehicle_categories = VehicleCategory::find()->where(['not in', 'id', [99, 4]])->all();
        $ticket_types_count = [];
        $vehicle_category_count = [];
        $manual_ticket_count = [];
        $escape_ticket_count = [];
        $misissued_ticket_count = [];
        $manual_serial_ticket_count = [];
        $reprint_ticket_count = [];
        $lost_ticket_count = [];

        $vehicle_category_count_without_escape = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket_type->id == $ticket->ticket_type_status_id) {
                    $type_count++;
                    $type_wise_amount = $type_wise_amount + $ticket->amount;
                }
            }
            array_push($ticket_types_count, ["name" => $ticket_type->name, "id" => $ticket_type->id, "count" => $type_count, "amount" => $type_wise_amount]);
        }
        foreach ($vehicle_categories as $vehicle_category) {
            $type_count = 0;
            $category_wise_amount = 0;
            $type_count_manual = 0;
            $type_wise_amount_manual = 0;
            $type_count_escape = 0;
            $type_wise_amount_escape = 0;

            $type_count_misissued = 0;
            $type_wise_amount_misissued = 0;

            $type_count_manual_serial = 0;
            $type_wise_amount_manual_serial = 0;

            $type_count_reprint = 0;
            $type_wise_amount_reprint = 0;

            $type_count_lost = 0;
            $type_wise_amount_lost = 0;

            $scanned_nfc_normal = 0;
            $scanned_nfc_manual = 0;

            $category_wise_amount_without_escape = 0;

            $total_against_category = [];
            $cat1_count = [];

            foreach ($tickets_data as $ticket) {
                if ($ticket->vehicle_category == $vehicle_category->id) {

                    //TICKET PRINT ON POS
                    if ($ticket->ticket_type_status_id != 8) {
                        if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 3) && !($ticket->ticket_type_status_id == 7) && !($ticket->ticket_type_status_id == 6) && !($ticket->ticket_type_status_id == 10) && !($ticket->ticket_type_status_id == 12)) {
                            $type_count++;
                            $category_wise_amount = $category_wise_amount + $ticket->amount;
                            if ($ticket->ticket_type_status_id == 5) {
                                $scanned_nfc_normal++;
                            }
                            // if ($ticket->ticket_type_status_id == 6) {
                            //    $scanned_nfc_manual++;
                            //  }
                            // if($ticket->ticket_type_status_id == 10){
                            //     $scanned_nfc_manual++;
                            // }
                        }
                        // $type_count++;
                        // $category_wise_amount  = $category_wise_amount + $ticket->amount;
                    }
                    //WITHOUT ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id != 8 && !($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 12)) {

                        $type_count_without_escape++;
                        $category_wise_amount_without_escape = $category_wise_amount_without_escape + $ticket->amount;
                    }

                    //MANUAL TICKET
                    if ($ticket->ticket_type_status_id == 7) {
                        $type_count_manual++;
                        $type_wise_amount_manual = $type_wise_amount_manual + $ticket->amount;
                    }
                    //ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id == 8) {
                        $type_count_escape++;
                        $type_wise_amount_escape = $type_wise_amount_escape + $ticket->amount;
                    }
                    //MISISSUED TICKETS
                    if ($ticket->ticket_type_status_id == 12) {
                        $type_count_misissued++;
                        $type_wise_amount_misissued = $type_wise_amount_misissued + $ticket->amount;
                    }

                    //MANUAL ENTER TICKETS
                    if ($ticket->ticket_type_status_id == 10) {
                        $type_count_manual_serial++;
                        $type_wise_amount_manual_serial = $type_wise_amount_manual_serial + $ticket->amount;
                    }

                    //REPRINT TICKETS
                    if ($ticket->ticket_type_status_id == 2) {
                        $type_count_reprint++;
                        $type_wise_amount_reprint = $type_wise_amount_reprint + $ticket->amount;
                    }

                    //LOST TICKETS
                    if (($ticket->ticket_type_status_id == 3) || ($ticket->ticket_type_status_id == 6)) {
                        $type_count_lost++;
                        $type_wise_amount_lost = $type_wise_amount_lost + $ticket->amount;
                    }
                }
            }


            //Book Rece = 7
            array_push($manual_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_manual, "amount" => $type_wise_amount_manual, "nfc_normal" => 0, "nfc_manual" => $type_count_manual]);
            array_push($escape_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_escape, "amount" => $type_wise_amount_escape]);
            array_push($misissued_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_misissued, "amount" => $type_wise_amount_misissued]);
            array_push($manual_serial_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_manual_serial, "amount" => $type_wise_amount_manual_serial, "nfc_normal" => 0, "nfc_manual" => $type_count_manual_serial]);
            array_push($reprint_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_reprint, "amount" => $type_wise_amount_reprint]);
            array_push($lost_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_lost, "amount" => $type_wise_amount_lost, "nfc_normal" => 0, "nfc_manual" => $type_count_lost]);

            array_push($vehicle_category_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count, "amount" => $category_wise_amount, "nfc_normal" => $scanned_nfc_normal, "nfc_manual" => $scanned_nfc_manual]);
            array_push($vehicle_category_count_without_escape, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_without_escape, "amount" => $category_wise_amount_without_escape]);
        }
        //    print_r(json_encode($manual_ticket_count));die;

        foreach ($tickets_data as $ticket) {

            if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 8) && !($ticket->ticket_type_status_id == 12)) {
                $total_amount = $total_amount + $ticket->amount;
            }
        }
        $total_ticket_count = count($tickets_data);

        $cash_collected_data = CashCollect::find()->where(['status' => 1, 'actual_session_id' => $actual_session_id])->one();
        $total_rev = 0;
        foreach ($vehicle_category_count_without_escape as $type) {

            $total_rev = $total_rev + $type["amount"];
        }

        $user = $this->findUser($user_id);

        if ($teller_confirm->load(Yii::$app->request->post()) && $teller_confirm->type == 'TELLER_CONFIRM') {
            //validate username and password of teller for confirmation

            $model = new LoginForm();
            $model->username = $teller_confirm->username;
            $model->password = $teller_confirm->password;

            if ($model->login2()) {
                //Save to the data base

                $session_report = new SessionReportConfirm();
                $session_report->session_uuid = $teller_confirm->session_uuid;
                $session_report->teller_id = $user->id;
                $session_report->teller_confirm_status = 1;
                $session_report->teller_confirm_time = date("Y-m-d H:i:s");

                $session_report->validate();
                // print_r($session_report->getErrors());die;
                $session_report->save();
            }
        }
        if ($teller_confirm->load(Yii::$app->request->post()) && $teller_confirm->type == 'HEAD_TELLER_CONFIRM') {
            //Update status in db Head teller confirmation
            $res = Yii::$app->db->createCommand('UPDATE tbl_session_report_confirm 
            SET final_confirm_by=' . Yii::$app->user->id . ',
            final_confirm_status=1,
            final_confirm_time ="' . date("Y-m-d H:i:s") . '"
            WHERE session_uuid="' . $teller_confirm->session_uuid . '"
            ')->execute();
        }

        $SessionReportConfirm = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                    'active' => 1
                ])->orderBy(['id' => SORT_DESC])->one();

        $SessionReportConfirmAll = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $towing_list = Towing::find()->where(['session_uuid' => $actual_session_model->session_uuid])->all();
        $total_towing = 0;
        $towing_count = count($towing_list);
        foreach ($towing_list as $tlist) {

            $total_towing = $tlist->amount + $total_towing;
        }


        return $this->render('pdf-online', [
                    'user' => $user,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_amount' => $total_amount,
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'cash_collected_data' => $cash_collected_data,
                    'manual_ticket_count' => $manual_ticket_count,
                    'escape_ticket_count' => $escape_ticket_count,
                    'misissued_ticket_count' => $misissued_ticket_count,
                    'manual_serial_ticket_count' => $manual_serial_ticket_count,
                    'reprint_ticket_count' => $reprint_ticket_count,
                    'lost_ticket_count' => $lost_ticket_count,
                    'vehicle_category_count_without_escape' => $vehicle_category_count_without_escape,
                    'total_rev' => $total_rev,
                    'teller_confirm' => $teller_confirm,
                    // 'balance'=>($cash_collected_data->total_system_amount - $cash_collected_data->total_cash_amount),
                    // 'balance'=>($total_rev - $cash_collected_data->total_cash_amount),
                    //'balance' => ($total_amount - $cash_collected_data->total_cash_amount),
                    'balance' => ($cash_collected_data->total_cash_amount - $total_amount),
                    'SessionReportConfirm' => $SessionReportConfirm,
                    'SessionReportConfirmAll' => $SessionReportConfirmAll,
                    'towing_count' => $towing_count,
                    'total_towing' => $total_towing,
                    'grand_total' => ($cash_collected_data->total_system_amount + $cash_collected_data->total_qr_amount)
        ]);
    }

    public function actionExportOnlinePdf($user_id, $actual_session_id) {
        $teller_confirm = new TellerConfirm();
        $actual_session_model = $this->findModel($actual_session_id);

        $SessionReportConfirm = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                    'active' => 1
                ])->orderBy(['id' => SORT_DESC])->one();

        $SessionReportConfirmAll = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $tickets_data = ExitTicket::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $total_amount = 0;

        $ticket_type_status = TicketTypeStatus::find()->all();
        $vehicle_categories = VehicleCategory::find()->where(['not in', 'id', [99, 4]])->all();
        $ticket_types_count = [];
        $vehicle_category_count = [];
        $manual_ticket_count = [];
        $escape_ticket_count = [];
        $misissued_ticket_count = [];
        $manual_serial_ticket_count = [];
        $reprint_ticket_count = [];
        $lost_ticket_count = [];

        $vehicle_category_count_without_escape = [];
        foreach ($ticket_type_status as $ticket_type) {
            $type_count = 0;
            $type_wise_amount = 0;
            foreach ($tickets_data as $ticket) {
                if ($ticket_type->id == $ticket->ticket_type_status_id) {
                    $type_count++;
                    $type_wise_amount = $type_wise_amount + $ticket->amount;
                }
            }
            array_push($ticket_types_count, ["name" => $ticket_type->name, "id" => $ticket_type->id, "count" => $type_count, "amount" => $type_wise_amount]);
        }
        foreach ($vehicle_categories as $vehicle_category) {
            $type_count = 0;
            $category_wise_amount = 0;
            $type_count_manual = 0;
            $type_wise_amount_manual = 0;
            $type_count_escape = 0;
            $type_wise_amount_escape = 0;

            $type_count_misissued = 0;
            $type_wise_amount_misissued = 0;

            $type_count_manual_serial = 0;
            $type_wise_amount_manual_serial = 0;

            $type_count_reprint = 0;
            $type_wise_amount_reprint = 0;

            $type_count_lost = 0;
            $type_wise_amount_lost = 0;

            $scanned_nfc_normal = 0;
            $scanned_nfc_manual = 0;

            $category_wise_amount_without_escape = 0;

            $total_against_category = [];
            $cat1_count = [];

            foreach ($tickets_data as $ticket) {
                if ($ticket->vehicle_category == $vehicle_category->id) {

                    //TICKET PRINT ON POS
                    if ($ticket->ticket_type_status_id != 8) {
                        if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 3) && !($ticket->ticket_type_status_id == 7) && !($ticket->ticket_type_status_id == 10) && !($ticket->ticket_type_status_id == 12) && !($ticket->ticket_type_status_id == 6)) {
                            $type_count++;
                            $category_wise_amount = $category_wise_amount + $ticket->amount;
                            if ($ticket->ticket_type_status_id == 5) {
                                $scanned_nfc_normal++;
                            }
                            //if ($ticket->ticket_type_status_id == 6) {
                            //    $scanned_nfc_manual++;
                            // }
                            // if($ticket->ticket_type_status_id == 10){
                            //     $scanned_nfc_manual++;
                            // }
                        }
                        // $type_count++;
                        // $category_wise_amount  = $category_wise_amount + $ticket->amount;
                    }
                    //WITHOUT ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id != 8 && !($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 12)) {

                        $type_count_without_escape++;
                        $category_wise_amount_without_escape = $category_wise_amount_without_escape + $ticket->amount;
                    }

                    //MANUAL TICKET
                    if ($ticket->ticket_type_status_id == 7) {
                        $type_count_manual++;
                        $type_wise_amount_manual = $type_wise_amount_manual + $ticket->amount;
                    }
                    //ESCAPE TICKETS
                    if ($ticket->ticket_type_status_id == 8) {
                        $type_count_escape++;
                        $type_wise_amount_escape = $type_wise_amount_escape + $ticket->amount;
                    }
                    //MISISSUED TICKETS
                    if ($ticket->ticket_type_status_id == 12) {
                        $type_count_misissued++;
                        $type_wise_amount_misissued = $type_wise_amount_misissued + $ticket->amount;
                    }

                    //MANUAL ENTER TICKETS
                    if ($ticket->ticket_type_status_id == 10) {
                        $type_count_manual_serial++;
                        $type_wise_amount_manual_serial = $type_wise_amount_manual_serial + $ticket->amount;
                    }

                    //REPRINT TICKETS
                    if ($ticket->ticket_type_status_id == 2) {
                        $type_count_reprint++;
                        $type_wise_amount_reprint = $type_wise_amount_reprint + $ticket->amount;
                    }

                    //LOST TICKETS
                    if (($ticket->ticket_type_status_id == 3) || ($ticket->ticket_type_status_id == 6)) {
                        $type_count_lost++;
                        $type_wise_amount_lost = $type_wise_amount_lost + $ticket->amount;
                    }
                }
            }

            //Book Rece = 7
            array_push($manual_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_manual, "amount" => $type_wise_amount_manual, "nfc_normal" => 0, "nfc_manual" => $type_count_manual]);
            array_push($escape_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_escape, "amount" => $type_wise_amount_escape]);
            array_push($misissued_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_misissued, "amount" => $type_wise_amount_misissued]);
            array_push($manual_serial_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_manual_serial, "amount" => $type_wise_amount_manual_serial, "nfc_normal" => 0, "nfc_manual" => $type_count_manual_serial]);
            array_push($reprint_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_reprint, "amount" => $type_wise_amount_reprint]);
            array_push($lost_ticket_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_lost, "amount" => $type_wise_amount_lost, "nfc_normal" => 0, "nfc_manual" => $type_count_lost]);

            array_push($vehicle_category_count, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count, "amount" => $category_wise_amount, "nfc_normal" => $scanned_nfc_normal, "nfc_manual" => $scanned_nfc_manual]);
            array_push($vehicle_category_count_without_escape, ["v_category__id" => $vehicle_category->id, "name" => $vehicle_category->name, "count" => $type_count_without_escape, "amount" => $category_wise_amount_without_escape]);
        }
        //    print_r(json_encode($manual_ticket_count));die;

        foreach ($tickets_data as $ticket) {

            if (!($ticket->ticket_type_status_id == 2) && !($ticket->ticket_type_status_id == 8) && !($ticket->ticket_type_status_id == 12)) {
                $total_amount = $total_amount + $ticket->amount;
            }
        }
        $total_ticket_count = count($tickets_data);

        $cash_collected_data = CashCollect::find()->where(['status' => 1, 'actual_session_id' => $actual_session_id])->one();
        $total_rev = 0;
        foreach ($vehicle_category_count_without_escape as $type) {

            $total_rev = $total_rev + $type["amount"];
        }

        $user = $this->findUser($user_id);

        if ($teller_confirm->load(Yii::$app->request->post()) && $teller_confirm->type == 'TELLER_CONFIRM') {
            //validate username and password of teller for confirmation

            $model = new LoginForm();
            $model->username = $teller_confirm->username;
            $model->password = $teller_confirm->password;

            if ($model->login2()) {
                //Save to the data base

                $session_report = new SessionReportConfirm();
                $session_report->session_uuid = $teller_confirm->session_uuid;
                $session_report->teller_id = $user->id;
                $session_report->teller_confirm_status = 1;
                $session_report->teller_confirm_time = date("Y-m-d H:i:s");

                $session_report->validate();
                // print_r($session_report->getErrors());die;
                $session_report->save();
            }
        }
        if ($teller_confirm->load(Yii::$app->request->post()) && $teller_confirm->type == 'HEAD_TELLER_CONFIRM') {
            //Update status in db Head teller confirmation
            $res = Yii::$app->db->createCommand('UPDATE tbl_session_report_confirm 
            SET final_confirm_by=' . Yii::$app->user->id . ',
            final_confirm_status=1,
            final_confirm_time ="' . date("Y-m-d H:i:s") . '"
            WHERE session_uuid="' . $teller_confirm->session_uuid . '"
            ')->execute();
        }

        $SessionReportConfirm = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                    'active' => 1
                ])->orderBy(['id' => SORT_DESC])->one();

        $SessionReportConfirmAll = SessionReportConfirm::find()->where([
                    'session_uuid' => $actual_session_model->session_uuid,
                ])->orderBy(['id' => SORT_DESC])->all();

        $towing_list = Towing::find()->where(['session_uuid' => $actual_session_model->session_uuid])->all();
        $total_towing = 0;
        $towing_count = count($towing_list);
        foreach ($towing_list as $tlist) {

            $total_towing = $tlist->amount + $total_towing;
        }

        //pdf

        $user = $this->findUser($user_id);

        $mpdf = new \Mpdf\Mpdf([
            // 'orientation' => 'L',
            'setAutoTopMargin' => 'pad',
        ]);

        $mpdf->setFooter('Report generated by ' . Yii::$app->getUser()->username . '<br> on ' . date('Y-m-d H:i:s') . ' || Page {PAGENO} of {nb}');
        $mpdf->WriteHTML($this->renderPartial('pdf-online-report', [
                    'user' => $user,
                    'actual_session_model' => $actual_session_model,
                    'tickets_data' => $tickets_data,
                    'actual_session_id' => $actual_session_id,
                    'total_amount' => $total_amount,
                    'total_ticket_count' => $total_ticket_count,
                    'cash_collect' => $cash_collect,
                    'vehicle_category_count' => $vehicle_category_count,
                    'ticket_types_count' => $ticket_types_count,
                    'cash_collected_data' => $cash_collected_data,
                    'manual_ticket_count' => $manual_ticket_count,
                    'escape_ticket_count' => $escape_ticket_count,
                    'misissued_ticket_count' => $misissued_ticket_count,
                    'manual_serial_ticket_count' => $manual_serial_ticket_count,
                    'reprint_ticket_count' => $reprint_ticket_count,
                    'lost_ticket_count' => $lost_ticket_count,
                    'vehicle_category_count_without_escape' => $vehicle_category_count_without_escape,
                    'total_rev' => $total_rev,
                    'teller_confirm' => $teller_confirm,
                    'balance' => ($cash_collected_data->total_cash_amount - $total_amount),
                    'SessionReportConfirm' => $SessionReportConfirm,
                    'SessionReportConfirmAll' => $SessionReportConfirmAll,
                    'towing_count' => $towing_count,
                    'total_towing' => $total_towing,
                    'grand_total' => ($cash_collected_data->total_system_amount + $cash_collected_data->total_qr_amount)
        ]));
        $mpdf->Output('Cash Collect Report (Teller-' . $user->username . ')' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

}
