<?php

namespace app\controllers;

use app\models\BoothDevices;
use app\models\Devices;
use app\models\DevicesSearch;
use app\models\DeviceStatus;
use app\models\Ic;
use app\models\IcAts;
use app\models\IcAtsAll;
use app\models\IcAtsWeekend;
use app\models\IcHeadTellers;
use app\models\IcHeadTellersWeekend;
use app\models\IcTs;
use app\models\LoginLogout;
use app\models\IcManager;
use app\models\RegionAdmin;
use app\models\RegionManager;
use webvimark\modules\UserManagement\models\User;
use Yii;
use yii\filters\AccessControl;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * DevicesController implements the CRUD actions for Devices model.
 */
class DevicesController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update', 'delete', 'activity', 'unassigned-device'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Devices models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new DevicesSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Devices model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {

        $device_journey = BoothDevices::find()->where(['device_id' => $id, 'status' => 0])->andWhere(['not', ['booth_id' => null]])->limit(50)->orderBy(['id' => SORT_DESC])->orderBy([
            'id' => SORT_DESC,

        ])->all();
        $device_active_on = BoothDevices::find()->where(['device_id' => $id, 'status' => 1])->one();
        $device_login_logout = LoginLogout::find()->where(['device_id' => $id])->limit(50)->orderBy([
            'id' => SORT_DESC,

        ])->all();
        $device_status = DeviceStatus::find()->where(['device_id' => $id, 'status' => 1])->one();

        $booth_device_model = new BoothDevices();

        if ($booth_device_model->load(Yii::$app->request->post())) {

            // $booth= Booth::find()->where(['id'=>$booth_device_model->booth_id])->one();
            $today = date("Y-m-d H:i:s");
            $user_id = Yii::$app->user->id;
            $model = new BoothDevices();
            $model->device_id = $booth_device_model->device_id;
            $model->booth_id = $booth_device_model->booth_id;
            $model->ic_id = $booth_device_model->ic_id;
            $model->status = 1;
            $model->ic_assigned_on = $today;
            $model->ic_assigned_by = $user_id;

            $model->booth_assigned_on = $today;
            $model->booth_assigned_by = $user_id;

            $device_id = $booth_device_model->device_id;

            //  $today = date("Y-m-d H:i:s");
            $user_id = Yii::$app->user->id;

            $row = BoothDevices::find()->where(['booth_id' => $booth_device_model->booth_id, 'status' => 1])->one();
            if ($row->booth_id) {
                $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,ic_remove_on="' . $today . '",ic_remove_by=' . $user_id . ',booth_remove_on="' . $today . '", booth_remove_by=' . $user_id . ' WHERE booth_id =' . $booth_device_model->booth_id . '')->execute();

            } else {
                $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,ic_remove_on="' . $today . '",ic_remove_by=' . $user_id . ' WHERE device_id=' . $device_id . '')->execute();

            }

            if ($model->save()) {
                $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_user"')->queryOne();
                $new_version = $master_version['tbl_version'] + 1;
                $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_user"')->execute();

                return $this->redirect(['view', 'id' => $model->device_id]);
            }

        }
        return $this->render('view', [
            'model' => $this->findModel($id),
            'device_journey' => $device_journey,
            'device_active_on' => $device_active_on,
            'device_login_logout' => $device_login_logout,
            'booth_device_model' => $booth_device_model,
            'device_status' => $device_status,
        ]);
    }
    public function actionActivity()
    {
        $all_devices = Devices::find()->all();

        // HEAD TELLERS 25
        if (User::hasRole('25')) {

            $head_teller = IcHeadTellers::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();
            $head_teller_weekend = IcHeadTellersWeekend::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

            if ($head_teller) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }
            if ($head_teller_weekend) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller_weekend], ['status' => 1]])->all();

                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();

            }

        }

        // ATS 40
        if (User::hasRole('40')) {

            $head_teller = IcAts::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

            $head_teller_weekend = IcAtsWeekend::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();
            if ($head_teller) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }
            if ($head_teller_weekend) {
                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller_weekend], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }

        }
        // TS 30
        if (User::hasRole('30')) {

            $head_teller = IcAtsAll::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

            $head_teller_weekend = IcTs::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();
            if ($head_teller) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }
            if ($head_teller_weekend) {
                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $head_teller_weekend], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }

        }
        
        //IC MANAGER 85
        if (User::hasRole('85')) {

            $ic_manager = IcManager::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();

            if ($ic_manager) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $ic_manager], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }

        }
        
        
        // REGION ADMIN 90
        if (User::hasRole('90')) {

            $regions = RegionAdmin::find()->select('region_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();
            $reg = [];
            foreach ($regions as $region) {
                array_push($reg, $region->region_id);
            }
            $ics = Ic::find()->select('id')->where(['region_id' => $reg])->all();
            $ar = [];
            foreach ($ics as $ic) {
                array_push($ar, $ic->id);
            }

            if ($ics) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }

        }
        
        // REGION MANAGER 91
        if (User::hasRole('91')) {

            $regions = RegionManager::find()->select('region_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->all();
            $reg = [];
            foreach ($regions as $region) {
                array_push($reg, $region->region_id);
            }
            $ics = Ic::find()->select('id')->where(['region_id' => $reg])->all();
            $ar = [];
            foreach ($ics as $ic) {
                array_push($ar, $ic->id);
            }

            if ($ics) {

                $device_ids = BoothDevices::find()->select('device_id as id')->where(['AND', ['in', 'ic_id', $ar], ['status' => 1]])->all();
                $all_devices = Devices::find()->where(['in', 'id', $device_ids])->all();
            }

        }

        return $this->render('activity', [
            'all_devices' => $all_devices,

        ]);
    }

    /**
     * Creates a new Devices model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Devices();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Devices model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Devices model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        try {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the Devices model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Devices the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {

            if (($model = Devices::findOne($id)) !== null) {
                return $model;
            }
    
            throw new NotFoundHttpException('The requested page does not exist.');
            
        }

        if (User::hasRole('90')) {
            
            $region_ids = RegionAdmin::find()
            ->select('region_id')
            ->where(['user_id' => Yii::$app->user->identity->id, 'status' => 1])
            ->column();

            $ic_ids = Ic::find()
                ->select('id')
                ->where(['region_id' => $region_ids])
                ->column();

            $device_ids = BoothDevices::find()
                ->select('device_id')
                ->where(['ic_id' => $ic_ids, 'status' => 1])
                ->column();

            if (in_array($id, $device_ids)) {
                $model = Devices::findOne($id);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned device.');
        }

        if (User::hasRole('91')) {
            
            $region_ids = RegionManager::find()
            ->select('region_id')
            ->where(['user_id' => Yii::$app->user->identity->id, 'status' => 1])
            ->column();

            $ic_ids = Ic::find()
                ->select('id')
                ->where(['region_id' => $region_ids])
                ->column();

            $device_ids = BoothDevices::find()
                ->select('device_id')
                ->where(['ic_id' => $ic_ids, 'status' => 1])
                ->column();

            if (in_array($id, $device_ids)) {
                $model = Devices::findOne($id);
                if ($model !== null) {
                    return $model;
                }
            }

            throw new NotFoundHttpException('The requested page does not exist for your assigned device.');
        }

        if (User::hasRole('85')) {
            
            $ic_manager = IcManager::find()
                ->select('ic_id')
                ->where([
                    'user_id' => \Yii::$app->user->identity->id, 
                    'status' => 1
                ])
                ->column(); 

                $device_ids = BoothDevices::find()
                    ->select('device_id')
                    ->where(['ic_id' => $ic_manager, 'status' => 1])
                    ->column();

                if (in_array($id, $device_ids)) {
                    $model = Devices::findOne($id);
                    if ($model !== null) {
                        return $model;
                    }
                }

                throw new NotFoundHttpException('The requested page does not exist for your assigned device.');
        }

        // ATS 40, TS 30, HEAD TELLERS 25
        $roleToClassMapping = [
            '40' => [IcAts::class, IcAtsWeekend::class],
            '30' => [IcAtsAll::class, IcTs::class],
            '25' => [IcHeadTellers::class, IcHeadTellersWeekend::class]
        ];

        foreach ($roleToClassMapping as $roleId => $classes) {
            if (User::hasRole($roleId)) {
                $ic_ids = [];
                foreach ($classes as $class) {
                    $ic_ids = array_merge($ic_ids, $class::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->column());
                }

                if (!empty($ic_ids)) {
                    $model = Devices::find()
                        ->innerJoin(BoothDevices::tableName(), 'BoothDevices.device_id = Devices.id')
                        ->where(['BoothDevices.ic_id' => $ic_ids, 'Devices.id' => $id, 'BoothDevices.status' => 1])
                        ->one();
        
                    if ($model !== null) {
                        return $model;
                    }
                }

                throw new NotFoundHttpException('The requested page does not exist for your assigned device.');
            }
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
    public function actionUnassignedDevice()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $device_id = ($_POST['device_id']);

        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $row = BoothDevices::find()->where(['device_id' => $device_id, 'status' => 1])->one();
        if ($row->booth_id) {
            $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,ic_remove_on="' . $today . '",ic_remove_by=' . $user_id . ',booth_remove_on="' . $today . '", booth_remove_by=' . $user_id . ' WHERE device_id=' . $device_id . '')->execute();

        } else {
            $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,ic_remove_on="' . $today . '",ic_remove_by=' . $user_id . ' WHERE device_id=' . $device_id . '')->execute();

        }

        if ($res) {
            return true;
        } else {
            return false;
        }

    }
}
