<?php

namespace app\controllers;

use app\models\EmailList;
use app\models\EmailListSearch;
use Yii;
use yii\filters\AccessControl;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

/**
 * EmailListController implements the CRUD actions for EmailList model.
 */
class EmailListController extends Controller
{

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update', 'delete', 'send'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all EmailList models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new EmailListSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single EmailList model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new EmailList model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new EmailList();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing EmailList model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing EmailList model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the EmailList model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return EmailList the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = EmailList::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionSend()
    {

        $all_emails = Yii::$app->db->createCommand('SELECT * FROM email_list')->queryAll();
        $all_outlets = Yii::$app->db->createCommand('SELECT * FROM outlet')->queryAll();

        function getManager($date)
        {

            $manager = Yii::$app->db->createCommand('SELECT
                (SELECT name FROM outlet WHERE id =weekend_roster.outlet_id) as outlet,
                schedule.date,
                employee.title,
                employee.FullName,
                (SELECT type FROM tenure WHERE tenure.id =employee.tenure) as tenure,
                (SELECT username FROM user WHERE user.id=user_shift.user_id) as username,
                employee.email,employee.contact_number
                FROM user_shift
                INNER JOIN shift ON user_shift.shift_id = shift.id
                INNER JOIN schedule ON shift.schedulid = schedule.id
                INNER JOIN weekend_roster ON schedule.weekend_rosterid = weekend_roster.id
                INNER JOIN employee ON employee.user_id = user_shift.user_id
WHERE weekend_roster.outlet_id IN (SELECT id FROM outlet) AND schedule.date ="' . $date . '" AND( employee.tenure =2 OR employee.tenure =9) ORDER BY outlet ASC')->queryAll();

            return ($manager);
        }

        //when automated please uncomment this
        $today = date("Y-m-d");
        $date1 = strtotime("+1 day", strtotime($today));
        $date_saturday = date("Y-m-d", $date1);
        $date2 = strtotime("+1 day", strtotime($date_saturday));
        $date_sunday = date("Y-m-d", $date2);

        //uncomment this two lines when automated
        // $date_saturday = '2019-03-09';
        // $date_sunday = '2019-03-10';

        $allBM_and_DBM_saturday = getManager($date_saturday);
        $allBM_and_DBM_sunday = getManager($date_sunday);

        $_content = $this->renderPartial('_email_templete', array(
            'date_saturday' => $date_saturday,
            'date_sunday' => $date_sunday,
            'allBM_and_DBM_saturday' => $allBM_and_DBM_saturday,
            'allBM_and_DBM_sunday' => $allBM_and_DBM_sunday,

        ));

        foreach ($all_emails as $email) {

            Yii::$app->mailer->compose()
                ->setFrom(Yii::$app->params['adminEmail'])
                ->setTo($email['email'])
                ->setSubject('BM & DBM DETAILS')
                ->setTextBody('BM and DBM Details')
                ->setHtmlBody($_content)
                ->send();
        }

        Yii::$app->getSession()->setFlash('success', 'Success! Weekend Roster, BM and DBM details are shooted to the given email list');
        return $this->render('send', [
        ]);
    }

}
