<?php

namespace app\controllers;

use app\models\EscapeTickets;
use app\models\EscapeTicketsSearch;
use app\models\Shift;
use Yii;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\AccessControl;

/**
 * EscapeTicketsController implements the CRUD actions for EscapeTickets model.
 */
class EscapeTicketsController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all EscapeTickets models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new EscapeTicketsSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single EscapeTickets model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new EscapeTickets model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new EscapeTickets();

        if ($model->load(Yii::$app->request->post())) {

            $req_time = date('Y-m-d H:i:s');
            $shift = Shift::find()->where(['AND', ['<=', 'shift_start', $req_time], ['>=', 'shift_end', $req_time], ['=', 'ic_id', (string) $model->exit_ic]])->one();
            $ref_shift = Shift::find()->where(['AND', ['<=', 'shift_start', $model->exit_time], ['>=', 'shift_end', $model->exit_time], ['=', 'ic_id', (string) $model->exit_ic]])->one();

            $model->shift_id = (string) $shift->id;
            $model->ref_shift_id = (string) $ref_shift->id;
            $model->validate();
            print_r($model->getErrors());
            die;
            if ($model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }
        }

        return $this->render('create', [
                    'model' => $model,
        ]);
    }

    /**
     * Updates an existing EscapeTickets model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id) {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
                    'model' => $model,
        ]);
    }

    /**
     * Deletes an existing EscapeTickets model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the EscapeTickets model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return EscapeTickets the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = EscapeTickets::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

}
