<?php

namespace app\controllers;

use Yii;
use app\models\ExitTicket;
use app\models\ExitTicketSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\web\Response;

/**
 * ExitTicketController implements the CRUD actions for ExitTicket model.
 */
class ExitTicketController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update', 'delete', 'ssp', 'view-ticket', 'update-ticket', 'service'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all ExitTicket models.
     * @return mixed
     */
     public function actionIndex()
      {
      $searchModel = new ExitTicketSearch();
      $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

      return $this->render('index', [
      'searchModel' => $searchModel,
      'dataProvider' => $dataProvider,
      ]);
      }
     

    public function actionService() {
        $searchModel = new ExitTicketSearch();
        // Pass a default value for vehicle_category
        $dataProvider = $searchModel->searchService(Yii::$app->request->queryParams, 20);

        return $this->render('service', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single ExitTicket model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    //cash collect view
    public function actionViewTicket($id) {
        return $this->render('viewTicket', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ExitTicket model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new ExitTicket();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }
        /* if ($model->load(Yii::$app->request->post())) {
          $exitTime = $model->exit_time;
          $entranceSerial = $model->entrance_serial;

          // Check for duplicates within 24 hours before the exit time
          if ($model->hasDuplicateEntranceTicket($entranceSerial, $exitTime, 24)) {
          Yii::$app->session->setFlash('error', 'Duplicate entrance ticket found within 24 hours before the exit time.');
          return $this->render('create', [
          'model' => $model,
          ]);
          }

          if ($model->save()) {
          return $this->redirect(['view', 'id' => $model->id]);
          }
          } */


        return $this->render('create', [
                    'model' => $model,
        ]);
    }

    /**
     * Updates an existing ExitTicket model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id) {
        $model = $this->findModel($id);

        if (!in_array($model->ticket_type_status_id, [7, 8, 12])) {
            throw new NotFoundHttpException('Only Mis-issued, Book Receipt, and Escape tickets can be updated.');
        }


        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view-ticket', 'id' => $model->id]);
            //return Yii::$app->getResponse()->redirect(Yii::$app->request->referrer ?: Yii::$app->homeUrl);
        }
        if ($model->ticket_type_status_id == 7) {

            return $this->render('update_book_reciept', [
                        'model' => $model,
            ]);
        }else if ($model->ticket_type_status_id == 8) {

            return $this->render('update_escap', [
                        'model' => $model,
            ]);
        } else if ($model->ticket_type_status_id == 12) {

            return Yii::$app->getResponse()->redirect(Yii::$app->request->referrer ?: Yii::$app->homeUrl); 
        }else {

            return $this->render('update', [
                        'model' => $model,
            ]);
        }
    }

    public function actionUpdateTicket($id) {
        $model = $this->findModel($id); // Make sure this method retrieves the model instance

        if ($model === null) {
            throw new \yii\web\NotFoundHttpException('The requested page does not exist.');
        }

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
                    'model' => $model,
        ]);
    }

    /**
     * Deletes an existing ExitTicket model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        //$this->findModel($id)->delete();
        //return $this->redirect(['index']);

        $model = $this->findModel($id);

        if (in_array($model->ticket_type_status_id, [7, 8, 12])) {
            $model->delete();
            return Yii::$app->getResponse()->redirect(Yii::$app->request->referrer ?: Yii::$app->homeUrl);
        }

        throw new NotFoundHttpException('Only Mis-issued, Book Receipt and Escape tickets can be deleted.');
    }

    /**
     * Finds the ExitTicket model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ExitTicket the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = ExitTicket::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionQuick() {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }

//    public function actionIndex() {
//
//        return $this->render('ssp', [
//        ]);
//    }

}
