<?php

namespace app\controllers;

use app\models\Fare;
use app\models\TimeSlot;
use app\models\VehicleCategory;
use Yii;
use yii\filters\VerbFilter;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use app\models\Route;
use app\models\FareChanged;
use yii\filters\AccessControl;


/**
 * FareController implements the CRUD actions for Fare model.
 */
class FareController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update','delete','all'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Fare models.
     * @return mixed
     */

    public function actionIndex()
    {
        $vehicle_categories = VehicleCategory::find()->where(['status'=>VehicleCategory::STATUS_FARE_HAVE])->all();

        return $this->render('index', [
            'vehicle_categories' => $vehicle_categories,

        ]);
    }

    /**
     * Displays a single Fare model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($vehicle_category_id)
    {

        //   print_r(Yii::$app->request->post());die;
        $vehicle_category = VehicleCategory::findOne($vehicle_category_id);
        $time_slots = TimeSlot::find()->all();
        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();

        return $this->render('view', [
            'ic' => $ic,
            'time_slots' => $time_slots,
            'vehicle_category' => $vehicle_category,
        ]);
    }
    public function actionAll()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;

        $fares = $_POST["fares"];

        $vehicle_category_id = $_POST["vehicle_category_id"];
        $time_slot_id = $_POST["time_slot_id"];
        $day_name = $_POST["day"];
        $days = ['MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY'];
        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();
        $new_records = [];
        for ($i = 0; $i < sizeof($ic); $i++) {

            for ($j = 0; $j < $i; $j++) {

                //uncomment this code snip for create all routes againtas to the interchanges(only for one direction)

                // $route = new Route();
                // $route->source=$ic[$i]['id'];
                // $route->destination=$ic[$j]['id'];
                // $route->direction = 1;
                // $route->distance=100;
                // $route->max_time = "04:30:00";
                // $route->save();

                // $routeF = new Route();
                // $routeF->source=$ic[$j]['id'];
                // $routeF->destination=$ic[$i]['id'];
                // $routeF->direction = 2;
                // $routeF->distance=100;
                // $routeF->max_time = "04:30:00";
                // $routeF->save();


                $route_id = Yii::$app->db->createCommand('SELECT id FROM tbl_route WHERE source=' . $ic[$i]['id'] . ' AND destination=' . $ic[$j]['id'] . '')->queryOne();
                $route_id_2 = Yii::$app->db->createCommand('SELECT id FROM tbl_route WHERE source=' . $ic[$j]['id'] . ' AND destination=' . $ic[$i]['id'] . '')->queryOne();
                // check the fares for all days
                if ($day_name == "ALL_DAY") {

                    foreach ($days as $d) {
                        // check the user selected days
                        if ($fares["CHECKBOX_" . $d] == $d) {
                            $f_amount = $fares["val" . $ic[$i]["id"] . $ic[$j]["id"]];
                            //find the related fare is exists in the db
                            $exist_fare = Yii::$app->db->createCommand('SELECT * FROM tbl_fare WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $d . '" AND route_id=' . $route_id['id'] . ' AND status=1')->queryOne();
                            $exist_fare_2 = Yii::$app->db->createCommand('SELECT * FROM tbl_fare WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $d . '" AND route_id=' . $route_id_2['id'] . ' AND status=1')->queryOne();
                           
                           
                            if (!($exist_fare["final_amount"] == $f_amount) && !($exist_fare_2["final_amount"] == $f_amount)) {
                                if(isset($exist_fare["id"])){
                                    //set the exist data tuple status as 0 ,deactivate
                                    Yii::$app->db->createCommand('UPDATE tbl_fare SET status=0 WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $d . '" AND route_id=' . $route_id['id'] . ' ')->execute();
                                    }
                                    if(isset($exist_fare_2["id"])){
                                        //set the exist data tuple status as 0 ,deactivate
                                        Yii::$app->db->createCommand('UPDATE tbl_fare SET status=0 WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $d . '" AND route_id=' . $route_id_2['id'] . ' ')->execute();
                                        }
                                $new_fare = new Fare();
                                $new_fare->day = $d;
                                $new_fare->time_slot_id = $time_slot_id;
                                $new_fare->vehicle_category_id = $vehicle_category_id;

                                $new_fare->fare = $ic[$i]['id'];
                                $new_fare->discount = $ic[$j]['id'];

                                $new_fare->remarks = "remark";
                                $new_fare->final_amount = $f_amount;
                                $new_fare->status = 1;
                                $new_fare->route_id = $route_id['id'];
                                $new_fare->validate();

                                // save the new fares for user selected days
                                $new_fare->save();                               
                                array_push($new_records, $new_fare->id);

                                // Save the new fares as other direction
                                $new_fare_2 = new Fare();
                                $new_fare_2->day = $d;
                                $new_fare_2->time_slot_id = $time_slot_id;
                                $new_fare_2->vehicle_category_id = $vehicle_category_id;

                                $new_fare_2->fare = $ic[$j]['id'];
                                $new_fare_2->discount = $ic[$i]['id'];

                                $new_fare_2->remarks = "remark";
                                $new_fare_2->final_amount = $f_amount;
                                $new_fare_2->status = 1;
                                $new_fare_2->route_id = $route_id_2['id'];
                                $new_fare_2->validate();

                                // save the new fares for user selected days
                                $new_fare_2->save();                               
                                array_push($new_records, $new_fare_2->id);

                            }

                        }

                    }
                   

                } else {
                    $f_amount = $fares["val" . $ic[$i]['id'] . $ic[$j]['id']];
                    //find the related fare is exists in the db
                    $exist_fare = Yii::$app->db->createCommand('SELECT * FROM tbl_fare WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $day_name . '" AND route_id=' . $route_id['id'] . ' AND status=1')->queryOne();
                    $exist_fare_2 = Yii::$app->db->createCommand('SELECT * FROM tbl_fare WHERE vehicle_category_id=' . $vehicle_category_id . ' AND time_slot_id=' . $time_slot_id . ' AND day="' . $day_name . '" AND route_id=' . $route_id_2['id'] . ' AND status=1')->queryOne();

                    //if exists check the fare is not equal
                    if (!($exist_fare["final_amount"] == $f_amount) && !($exist_fare_2["final_amount"] == $f_amount)) {
                        if(isset($exist_fare["id"])){
                            //set the exist data tuple status as 0 ,deactivate
                       Yii::$app->db->createCommand('UPDATE tbl_fare SET status=0 WHERE id=' . $exist_fare["id"] . ' ')->execute();
                       }
                       if(isset($exist_fare_2["id"])){
                        //set the exist data tuple status as 0 ,deactivate
                   Yii::$app->db->createCommand('UPDATE tbl_fare SET status=0 WHERE id=' . $exist_fare_2["id"] . ' ')->execute();
                   }
                      
                        $new_fare = new Fare();
                        $new_fare->day = $day_name;
                        $new_fare->time_slot_id = $time_slot_id;
                        $new_fare->vehicle_category_id = $vehicle_category_id;

                        $new_fare->fare = $ic[$i]['id'];
                        $new_fare->discount = $ic[$j]['id'];

                        $new_fare->remarks = "remark";
                        $new_fare->final_amount = $f_amount;
                        $new_fare->status = 1;
                        $new_fare->route_id = $route_id['id'];
                        $new_fare->validate();
                        // save the new data tuple as status 1;activate
                        $new_fare->save();
                        
                        array_push($new_records, $new_fare->id);

                        $new_fare_2 = new Fare();
                        $new_fare_2->day = $day_name;
                        $new_fare_2->time_slot_id = $time_slot_id;
                        $new_fare_2->vehicle_category_id = $vehicle_category_id;

                        $new_fare_2->fare = $ic[$j]['id'];
                        $new_fare_2->discount = $ic[$i]['id'];

                        $new_fare_2->remarks = "remark";
                        $new_fare_2->final_amount = $f_amount;
                        $new_fare_2->status = 1;
                        $new_fare_2->route_id = $route_id_2['id'];
                        $new_fare_2->validate();
                        // save the new data tuple as status 1;activate
                        $new_fare_2->save();
                        
                        array_push($new_records, $new_fare_2->id);

                    }

                }
            }
        }

     
        $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_fare"')->queryOne();
      
        $new_version = $master_version['tbl_version']+1;
        
 //  $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_fare"')->execute();
       
        $fare_changed = new FareChanged();
        $fare_changed->created_on = date("Y-m-d H:i:s");
        $fare_changed->newly_inserted = json_encode($new_records);
        $fare_changed->master_tbl_fare_version =  $new_version ;
        $fare_changed->save();
   
        
        return true;
    }

    /**
     * Creates a new Fare model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Fare();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Fare model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Fare model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Fare model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Fare the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Fare::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
}
