<?php

namespace app\controllers;

use Yii;
use app\models\Ic;
use app\models\IcSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\Booth;
use app\models\BoothSearch;
use yii\filters\AccessControl;
use app\models\Devices;
use app\models\BoothDevices;
use yii\helpers\ArrayHelper;
use yii\web\Response;
use app\models\Route;
use app\models\IcUsers;
use app\models\IcUsersSearch;
use app\models\IcHeadTellers;
use app\models\IcAts;
use app\models\IcHeadTellersWeekend;
use app\models\IcUsersAll;

use app\models\IcAtsAll;
use app\models\IcAtsWeekend;
use app\models\IcTs;
use app\models\Shift;

use app\models\Schedule;
use app\models\IcManager;
use app\models\RegionAdmin;
use app\models\RegionManager;
use yii\helpers\Json;
use webvimark\modules\UserManagement\models\User;



/**
 * IcController implements the CRUD actions for Ic model.
 */
class IcController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'shift-list',
                        'create', 'update','delete','unassigned-device','remove-ht','remove-htw','get-immediate-ic',
                        'get-immediate-ic-for-select','remove-ats','remove-ats-all','remove-atsw','remove-ts','ic-list','remove-icm'
                    ],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Ic models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new IcSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Ic model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {   
        $buttenEnable = 1;
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $buttenEnable = 1;
        } else if (User::hasRole('95')) {
            $buttenEnable = 0;
        } else if (User::hasRole('65')) {
            $buttenEnable = 0;
        } else if (User::hasRole('90')) {
            $buttenEnable = 1;
        }else if (User::hasRole('91')) {
            $buttenEnable = 1;
        }
        
        $BoothModel = new Booth();
        $searchBoothModel = new BoothSearch();
        $booth_devices = new BoothDevices();
        $ic_user = new IcUsers();
        $add_ic_user_model = new IcUsers();
        $remove_ic_user_model = new IcUsers();

        $ic_head_teller_model = new IcHeadTellers();
        $ic_head_teller_model_weekend = new IcHeadTellersWeekend();
        $ic_ats_model = new IcAts();
        $ic_ats_model_all = new IcAtsAll();
        $ic_ats_model_weekend = new IcAtsWeekend();
        $ic_ts_model = new IcTs();
        $ic_manager_model = new IcManager();



        $searchIcUsersModel = new IcUsersSearch();
        $dataUsersProvider = $searchIcUsersModel->searchByIc(Yii::$app->request->queryParams);
        $dataBoothProvider = $searchBoothModel->searchBooth(Yii::$app->request->queryParams);
       
        $all_devices =ArrayHelper::map(Yii::$app->db->createCommand('SELECT tbl_devices.id as id,tbl_devices.serial as serial,tbl_devices.name as device_name FROM tbl_devices WHERE tbl_devices.status =1 AND tbl_devices.id NOT IN (SELECT device_id FROM tbl_booth_devices WHERE tbl_booth_devices.status=1 AND tbl_booth_devices.ic_id is NOT NULL)')->queryAll(), 'id', function($model) {
            return $model['device_name'].' '.'('.$model['serial'].' )';
        });
        $user_list_for_add =ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_users WHERE tbl_ic_users.status= '.IcUsers::STATUS_ACTIVE.') AND user.designation_id=5')->queryAll(), 'id', 'username');
        $user_list_for_remove = ArrayHelper::map(Yii::$app->db->createCommand('SELECT tbl_ic_users.user_id as id,user.username as username FROM tbl_ic_users INNER JOIN user ON  tbl_ic_users.user_id = user.id WHERE tbl_ic_users.status='.IcUsers::STATUS_ACTIVE.' AND tbl_ic_users.ic_id='.$id.' ')->queryAll(), 'id', 'username');
        $head_teller_list =ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_head_tellers WHERE tbl_ic_head_tellers.status=1 AND tbl_ic_head_tellers.ic_id='.$id.') AND AI.name = 25')->queryAll(), 'id', 'username');
        $head_teller_list_weekend =ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_head_tellers_weekend WHERE tbl_ic_head_tellers_weekend.status=1 AND tbl_ic_head_tellers_weekend.ic_id='.$id.') AND AI.name = 25')->queryAll(), 'id', 'username');
   
        $ats_list=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_ats WHERE tbl_ic_ats.status=1 AND tbl_ic_ats.ic_id='.$id.') AND AI.name = 40')->queryAll(), 'id', 'username');
        $ats_list_all=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_ats_all WHERE tbl_ic_ats_all.status=1 AND tbl_ic_ats_all.ic_id='.$id.') AND AI.name = 40')->queryAll(), 'id', 'username');
        $ats_list_weekend=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_ats_weekend WHERE tbl_ic_ats_weekend.status=1 AND tbl_ic_ats_weekend.ic_id='.$id.') AND AI.name = 40')->queryAll(), 'id', 'username');
        $ts_list=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_ts WHERE tbl_ic_ts.status=1 AND tbl_ic_ts.ic_id='.$id.') AND AI.name = 30')->queryAll(), 'id', 'username');

        //$ic_manager_list=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_manager WHERE tbl_ic_manager.status=1 AND tbl_ic_manager.ic_id='.$id.') AND user.designation_id=29')->queryAll(), 'id', 'username');
        
        $ic_manager_list=ArrayHelper::map(Yii::$app->db->createCommand('SELECT user.id as id,user.username as username FROM user INNER JOIN auth_assignment AA ON AA.user_id = user.id
            INNER JOIN auth_item AI ON AI.name = AA.item_name WHERE user.status =1 AND user.id NOT IN (SELECT user_id FROM tbl_ic_manager WHERE tbl_ic_manager.status=1 AND tbl_ic_manager.ic_id='.$id.') AND AI.name = 85')->queryAll(), 'id', 'username');

       
        $devices_for_ic = BoothDevices::find()->where(['ic_id' => $id,'status'=>1])->all();
        $devices_for_ic_count = BoothDevices::find()->where(['ic_id' => $id,'status'=>1])->count();
        //count booths 
        $countOfBooths = Booth::find()
            ->where(['ic_id' => $id])
            ->count();
          //count users 
        $countOfUsers = IcUsers::find()
            ->where(['ic_id' => $id,'status'=>IcUsers::STATUS_ACTIVE])
            ->count();   

        if ($BoothModel->load(Yii::$app->request->post()) ) {

           
            $regionn= Ic::find()->select('region_id')->where(['id' => $BoothModel->ic_id])->one();
            $BoothModel->region_id = $regionn->region_id;
            if($BoothModel->save()){
                 return $this->redirect(['view', 'id' => $BoothModel->ic_id]);
            }

           
        }
        // add devices to the ic
        if ($booth_devices->load(Yii::$app->request->post())  ) {

           
            foreach ($booth_devices['device_id'] as $device_id) {

                $model = new BoothDevices();
                $model->device_id = $device_id;
                $model->ic_id = $booth_devices->ic_id;
                $model->status = $booth_devices->status;
                $model->ic_assigned_on = $booth_devices->ic_assigned_on;
                $model->ic_assigned_by = $booth_devices->ic_assigned_by;
              
                $model->save();
            }
            Yii::$app->getSession()->setFlash('success', 'Assigned');
            return $this->redirect(['view', 'id' => $booth_devices->ic_id]);
           

           
        }
        //add users to the ic
        $post_data=Yii::$app->request->post();
       // print_r($post_data['IcUsers']['operation_type']);die;
       

        
        if ( $add_ic_user_model->load(Yii::$app->request->post()) && $post_data['IcUsers']['operation_type']==1) {

           
            foreach ($add_ic_user_model['user_id'] as $user_id) {

                /*$model = new IcUsers();
                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = IcUsers::STATUS_ACTIVE;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;              
                $model->save();*/
                
                
                $model = new IcUsers();
                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = IcUsers::STATUS_ACTIVE;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcUsers table.');
                    return $this->redirect(['view', 'id' => $id]);
                }
                
            }
            Yii::$app->getSession()->setFlash('success', 'Teller Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }
        // add head tellers to the ic
        if ( $ic_head_teller_model->load(Yii::$app->request->post()) ) {

           
            foreach ($ic_head_teller_model['user_id'] as $user_id) {

                /*$model = new IcHeadTellers();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->created_on = date("Y-m-d H:i:s");
                $model->created_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcHeadTellers();
                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->created_on = date("Y-m-d H:i:s");
                $model->created_by = Yii::$app->user->id;
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcHeadTaller table.');
                    return $this->redirect(['view', 'id' => $id]);
                }

             
            }
            Yii::$app->getSession()->setFlash('success', 'Head Teller(HT) Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }
          // add head tellers Weekend to the ic
          if ( $ic_head_teller_model_weekend->load(Yii::$app->request->post()) ) {

           
            foreach ($ic_head_teller_model_weekend['user_id'] as $user_id) {

                /*$model = new IcHeadTellersWeekend();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->created_on = date("Y-m-d H:i:s");
                $model->created_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcHeadTellersWeekend();
                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->created_on = date("Y-m-d H:i:s");
                $model->created_by = Yii::$app->user->id;
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcHeadTellersWeekend table.');
                    return $this->redirect(['view', 'id' => $id]);
                }

             
            }
            Yii::$app->getSession()->setFlash('success', 'Head Teller - WKND Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }
          // add ATS to the ic
          if ( $ic_ats_model->load(Yii::$app->request->post()) ) {

           
            foreach ($ic_ats_model['user_id'] as $user_id) {

                /*$model = new IcAts();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcAts();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;             
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAts table.');
                    return $this->redirect(['view', 'id' => $id]);
                }

             
            }
            Yii::$app->getSession()->setFlash('success', 'ATS Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }
           // add TS to the ic
           if ( $ic_ts_model->load(Yii::$app->request->post()) ) {
              
            foreach ($ic_ts_model['user_id'] as $user_id) {

                /*$model = new IcTs();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcTs();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;            
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcTs table.');
                    return $this->redirect(['view', 'id' => $id]);
                }


            }
            Yii::$app->getSession()->setFlash('success', 'TS Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }

         // add ATS ALL to the ic
         if ( $ic_ats_model_all->load(Yii::$app->request->post()) ) {
           

           
            foreach ($ic_ats_model_all['user_id'] as $user_id) {

                /*$model7 = new IcAtsAll();

                $model7->user_id = $user_id;
                $model7->ic_id = $id;
                $model7->status = 1;
                $model7->added_on = date("Y-m-d H:i:s");
                $model7->added_by = Yii::$app->user->id;             
                $model7->save();*/
                
                $model7 = new IcAtsAll();

                $model7->user_id = $user_id;
                $model7->ic_id = $id;
                $model7->status = 1;
                $model7->added_on = date("Y-m-d H:i:s");
                $model7->added_by = Yii::$app->user->id;            
                
                if ($model7->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAtsAll table.');
                    return $this->redirect(['view', 'id' => $id]);
                }

             
            }
            Yii::$app->getSession()->setFlash('success', 'ATS All Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }

         // add ATS Weekend to the ic
         if ( $ic_ats_model_weekend->load(Yii::$app->request->post()) ) {

           
            foreach ($ic_ats_model_weekend['user_id'] as $user_id) {

                /*$model = new IcAtsWeekend();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcAtsWeekend();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;            
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAtsWeekend table.');
                    return $this->redirect(['view', 'id' => $id]);
                }


             
            }
            Yii::$app->getSession()->setFlash('success', 'ATS Weekend Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }

        // add IC MANAGER to the ic
        if ($ic_manager_model->load(Yii::$app->request->post()) ) {
              
            foreach ($ic_manager_model['user_id'] as $user_id) {

                /*$model = new IcManager();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;             
                $model->save();*/
                
                $model = new IcManager();

                $model->user_id = $user_id;
                $model->ic_id = $id;
                $model->status = 1;
                $model->added_on = date("Y-m-d H:i:s");
                $model->added_by = Yii::$app->user->id;            
                
                if ($model->save()) {
                    $model2 = new IcUsersAll();
                    $model2->user_id = $user_id;
                    $model2->ic_id = $id;
                    $model2->status = IcUsersAll::STATUS_ACTIVE;
                    $model2->added_on = date("Y-m-d H:i:s");
                    $model2->added_by = Yii::$app->user->id;
                    
                    if (!$model2->save()) {
                        Yii::$app->getSession()->setFlash('error', 'Error saving data in IcAllUsers table.');
                        return $this->redirect(['view', 'id' => $id]);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Error saving data in IcManager table.');
                    return $this->redirect(['view', 'id' => $id]);
                }

            }
            Yii::$app->getSession()->setFlash('success', 'IC Manager Assigned');
            return $this->redirect(['view', 'id' => $id]);
           

           
        }


        //remove users from ic
        if ($remove_ic_user_model->load(Yii::$app->request->post()) && $post_data['IcUsers']['operation_type'] ==0) {

           
            foreach ($remove_ic_user_model['user_id'] as $user_id) {
                $res=Yii::$app->db->createCommand('UPDATE  tbl_ic_users SET status= 0 ,removed_on="'.date("Y-m-d H:i:s").'" ,removed_by="'.Yii::$app->user->id.'" WHERE  ic_id = '.$id.' AND user_id='.$user_id.' AND status=1')->execute();
               
            }
            return $this->redirect(['view', 'id' => $id]);
           

           
        }


        $data_head_tellers = IcHeadTellers::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_head_tellers_weekend = IcHeadTellersWeekend::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_ats = IcAts::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_ats_all = IcAtsAll::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_ats_weekend = IcAtsWeekend::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_ts = IcTs::find()->where(['ic_id'=>$id,'status'=>1])->all();
        $data_ic_manager = IcManager::find()->where(['ic_id'=>$id,'status'=>1])->all();
     
        return $this->render('view', [
            'model' => $this->findModel($id),
            'searchBoothModel' => $searchBoothModel,
            'dataBoothProvider' => $dataBoothProvider,
            'countOfBooths' => $countOfBooths,
            'BoothModel'=>$BoothModel,
            'all_devices'=>$all_devices,
            'devices_for_ic'=>$devices_for_ic,
            'booth_devices'=>$booth_devices,
            'devices_for_ic_count'=>$devices_for_ic_count,
            'ic_user'=>$ic_user,
            'searchIcUsersModel'=>$searchIcUsersModel,
            'countOfUsers'=>$countOfUsers,
            'dataUsersProvider'=>$dataUsersProvider,
            'add_ic_user_model'=>$add_ic_user_model,
            'remove_ic_user_model'=>$remove_ic_user_model,
            'user_list_for_add'=>$user_list_for_add,
            'user_list_for_remove'=>$user_list_for_remove,

            'ic_head_teller_model'=>$ic_head_teller_model,
            'head_teller_list'=>$head_teller_list,
            'data_head_tellers'=>$data_head_tellers,

           

            'ic_head_teller_model_weekend'=>$ic_head_teller_model_weekend,
            'head_teller_list_weekend'=>$head_teller_list_weekend,
            'data_head_tellers_weekend'=>$data_head_tellers_weekend,

            'ic_ats_model'=>$ic_ats_model,
            'ats_list'=>$ats_list,
            'data_ats'=>$data_ats,

            'ic_ats_model_all'=>$ic_ats_model_all,
            'ats_list_all'=>$ats_list_all,
            'data_ats_all'=>$data_ats_all,

            'ic_ats_model_weekend'=>$ic_ats_model_weekend,
            'ats_list_weekend'=>$ats_list_weekend,
            'data_ats_weekend'=>$data_ats_weekend,

            'ic_ts_model'=>$ic_ts_model,
            'ts_list'=>$ts_list,
            'data_ts'=>$data_ts,

            'ic_manager_model'=>$ic_manager_model,
            'ic_manager_list'=>$ic_manager_list,
            'data_ic_manager'=>$data_ic_manager,
            'buttenEnable'=>$buttenEnable



        ]);
    }

    /**
     * Creates a new Ic model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Ic();


        if ($model->load(Yii::$app->request->post()) && $model->save()) {

            $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();

            for ($i = 0; $i < sizeof($ic); $i++) {
    
                for ($j = 0; $j < $i; $j++) {
                    $route_id = Yii::$app->db->createCommand('SELECT id FROM tbl_route WHERE source=' . $ic[$i]['id'] . ' AND destination=' . $ic[$j]['id'] . '')->queryOne();
                    $route_id_2 = Yii::$app->db->createCommand('SELECT id FROM tbl_route WHERE source=' . $ic[$j]['id'] . ' AND destination=' . $ic[$i]['id'] . '')->queryOne();
                  
                    if(!(isset($route_id["id"])) && !(isset($route_id_2["id"]))){
                        
                        $route = new Route();
                        $route->source=$ic[$i]['id'];
                        $route->destination=$ic[$j]['id'];
                        $route->direction = 1;
                        $route->distance=100;
                        $route->max_time = "04:30:00";
                        $route->save();

                        $route_2 = new Route();
                        $route_2->source=$ic[$j]['id'];
                        $route_2->destination=$ic[$i]['id'];
                        $route_2->direction = 2;
                        $route_2->distance=100;
                        $route_2->max_time = "04:30:00";
                        $route_2->save();
                    }
                }
            }
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Ic model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Ic model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {   
       
      //  $res = Yii::$app->db->createCommand('DELETE FROM tbl_route WHERE source=' . $id . ' OR destination=' . $id . '')->execute();
                 
      try {
        $this->findModel($id)->delete();
        Yii::$app->getSession()->setFlash('success', 'Success - deleted');
        return $this->redirect(['index']);
    } catch (\Exception $e) {
        Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
        return $this->redirect(['index']);
    } catch (\Throwable $e) {
        Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
        return $this->redirect(['index']);
    }
       
    }

    /**
     * Finds the Ic model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Ic the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {

            if (($model = Ic::findOne($id)) !== null) {
                return $model;
            }
    
            throw new NotFoundHttpException('The requested page does not exist.');
            
        }

        if (User::hasRole('90')) {
            
            $regions = RegionAdmin::find()
                ->select('region_id')
                ->where(['user_id' => Yii::$app->user->identity->id, 'status' => 1])
                ->column();  
    
            $model = Ic::find()
                ->where(['region_id' => $regions, 'id' => $id])
                ->one();
    
            if ($model !== null) {
                return $model;
            }
    
            
            throw new NotFoundHttpException('The requested page does not exist for your assigned ic.');
        }

        if (User::hasRole('91')) {
            
            $regions = RegionManager::find()
                ->select('region_id')
                ->where(['user_id' => Yii::$app->user->identity->id, 'status' => 1])
                ->column();  
    
            
            $model = Ic::find()
                ->where(['region_id' => $regions, 'id' => $id])
                ->one();
    
            if ($model !== null) {
                return $model;
            }
    
            
            throw new NotFoundHttpException('The requested page does not exist for your assigned ic.');
        }

        if (User::hasRole('85')) {
            
            $ic_manager = IcManager::find()
                ->select('ic_id')
                ->where([
                    'user_id' => \Yii::$app->user->identity->id, 
                    'status' => 1
                ])
                ->column(); 
        
            
            if (in_array($id, $ic_manager)) {
                $model = Ic::findOne(['id' => $id]); 
                if ($model !== null) {
                    return $model; 
                }
            }
        
            throw new NotFoundHttpException('The requested page does not exist for your assigned IC.');
        }
        
        // ATS 40, TS 30, HEAD TELLERS 25
        $roleToClassMapping = [
            '40' => [IcAts::class, IcAtsWeekend::class],
            '30' => [IcAtsAll::class, IcTs::class],
            '25' => [IcHeadTellers::class, IcHeadTellersWeekend::class]
        ];

        foreach ($roleToClassMapping as $roleId => $classes) {
            if (User::hasRole($roleId)) {
                $ic_ids = [];
                foreach ($classes as $class) {
                    $ic_ids = array_merge($ic_ids, $class::find()->select('ic_id')->where(['user_id' => \Yii::$app->user->identity->id, 'status' => 1])->column());
                }

                if (in_array($id, $ic_ids)) {
                    $model = Ic::findOne(['id' => $id]);
                    if ($model !== null) {
                        return $model;
                    }
                }

                throw new NotFoundHttpException('The requested page does not exist for your assigned IC.');
            }
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    
       
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }

    public function actionUnassignedDevice()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $device_id = ($_POST['device_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;
        $res = Yii::$app->db->createCommand('UPDATE tbl_booth_devices SET status=0,ic_remove_on="'.$today.'",ic_remove_by='.$user_id.' WHERE device_id=' . $device_id .' AND ic_id='.$interchange_id .' AND booth_id is NULL')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'Device unassiged');
            return true;
        }
       
    }
    public function actionRemoveHt()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ht_row_id = ($_POST['ic_ht_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_head_tellers SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ht_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'Head Teller(HT) removed');
            return true;
        }
       
    }
    public function actionRemoveHtw()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ht_row_id = ($_POST['ic_ht_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_head_tellers_weekend SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ht_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'Head Teller - WKND removed');
            return true;
        }
       
    }
    

    public function actionGetImmediateIc($ic_id) {
        
        // $ic = Ic::find()
        //     ->where(['id'=>$ic_id])
        //     ->one();

        
        // if(is_null($ic->previous_ic) && is_null($ic->next_ic)){
            
        //     echo "<option value = ''>This interchange has no immediate interchange</option>";
            
        // }else{
            
        //     if(!is_null($ic->previous_ic)){
        //         echo "<option value='".$ic->previous_ic."'>".Ic::find()->where(['id' => $ic->previous_ic])->one()->name."</option>";
        //     }
        //     if(!is_null($ic->next_ic)){
        //         echo "<option value='".$ic->next_ic."'>".Ic::find()->where(['id' => $ic->next_ic])->one()->name."</option>";
        //     }
        // }

     $ic = Ic::find()
            ->where(['not in','id',[$ic_id]])
            ->all();
     foreach($ic as $i){
        echo "<option value='".$i->id."'>".Ic::find()->where(['id' => $i->id])->one()->name."</option>";
     }   
        
        
    }
    public function actionGetImmediateIcForSelect() {
        $ic_id = $_POST['icId'];
        
        // $ic = Ic::find()
        //     ->where(['id'=>$ic_id])
        //     ->one();
        
        // if(is_null($ic->previous_ic) && is_null($ic->next_ic)){
            
        //     echo "<option value = ''> <-- Please select 1st interchange</option>";
            
        // }else{
            
        //     if(!is_null($ic->previous_ic)){
        //         echo "<option value='".$ic->previous_ic."'>".Ic::find()->where(['id' => $ic->previous_ic])->one()->name."</option>";
        //     }
        //     if(!is_null($ic->next_ic)){
        //         echo "<option value='".$ic->next_ic."'>".Ic::find()->where(['id' => $ic->next_ic])->one()->name."</option>";
        //     }
        // }

        $ic = Ic::find()
        ->where(['not in','id',[$ic_id]])
        ->all();
 foreach($ic as $i){
    echo "<option value='".$i->id."'>".Ic::find()->where(['id' => $i->id])->one()->name."</option>";
 }   
    }   
    public function actionRemoveAts()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ats_row_id = ($_POST['ic_ats_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_ats SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ats_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'ATS removed');
            return true;
        }
       
    }
    public function actionRemoveAtsAll()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ats_row_id = ($_POST['ic_ats_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_ats_all SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ats_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'ATS removed');
            return true;
        }
       
    }
    public function actionRemoveAtsw()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ats_row_id = ($_POST['ic_ats_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_ats_weekend SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ats_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'ATS removed');
            return true;
        }
       
    }
    public function actionRemoveTs()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_ats_row_id = ($_POST['ic_ats_row_id']);
        $interchange_id = ($_POST['interchange_id']);
        $today = date("Y-m-d H:i:s");
        $user_id = Yii::$app->user->id;

        $res = Yii::$app->db->createCommand('UPDATE tbl_ic_ts SET status=0,removed_on="'.$today.'",removed_by='.$user_id.' WHERE id=' . $ic_ats_row_id .' ')->execute();
        if($res){
            Yii::$app->getSession()->setFlash('success', 'TS removed');
            return true;
        }
       
    }
    
    public function actionRemoveIcm()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
    $ic_manager_row_id = Yii::$app->request->post('ic_manager_row_id');
    $interchange_id = Yii::$app->request->post('interchange_id');
    $today = date("Y-m-d H:i:s");
    $user_id = Yii::$app->user->id;

    // Begin transaction to ensure data consistency
    $transaction = Yii::$app->db->beginTransaction();
    try {
        // Update the IC Manager status
        $res = Yii::$app->db->createCommand()->update('tbl_ic_manager', [
            'status' => 0,
            'removed_on' => $today,
            'removed_by' => $user_id
        ], 'id=:id', [':id' => $ic_manager_row_id])->execute();

        if ($res) {
            $transaction->commit();
            Yii::$app->session->setFlash('success', 'IC Manager removed');
            return ['status' => 'success', 'message' => 'IC Manager removed successfully.'];
        } else {
            $transaction->rollBack();
            return ['status' => 'error', 'message' => 'Failed to remove IC Manager.'];
        }
    } catch (\Exception $e) {
        $transaction->rollBack();
        return ['status' => 'error', 'message' => $e->getMessage()];
    }
       
    }
    
    public function actionShiftList() {

        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic_id = ($_POST['ic_id']);
        $date1 = ($_POST['date']);

     //   echo $date;
        $schedule = Schedule::find()
        ->where(['date'=>$date1])
        ->one();
      
      
        if($schedule->id){
            $shifts = Shift::find()
            ->where(['ic_id'=>$ic_id,'schedule_id'=>$schedule->id])
            ->all();

            return ['success'=>true,'data'=>$shifts];
      
        
       
        }else{
            return ['success'=>false,'data'=>""];
      
        }
        
    }


    public function actionIcList($region_id)
    
    {
    
    $db = Yii::$app->db;
    
    Yii::$app->response->format = Response::FORMAT_HTML;
    
    if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }

    if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
        if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }
    } elseif (User::hasRole('25')) {
        $user_id = Yii::$app->user->identity->id;
        $icList = $db->createCommand("SELECT IC.id, IC.name FROM tbl_ic_head_tellers HT
            INNER JOIN tbl_ic IC ON HT.ic_id = IC.id
            WHERE IC.region_id = :region_id AND HT.user_id = :user_id
            UNION
            SELECT IC.id, IC.name FROM tbl_ic_head_tellers_weekend HTW
            INNER JOIN tbl_ic IC ON HTW.ic_id = IC.id
            WHERE IC.region_id = :region_id AND HTW.user_id = :user_id")
            ->bindValue(':region_id', $region_id)
            ->bindValue(':user_id', $user_id)
            ->queryAll();
        $icList = ArrayHelper::map($icList, 'id', 'name');
    } elseif (User::hasRole('30')) {
        $user_id = Yii::$app->user->identity->id;
        $icList = $db->createCommand("SELECT IC.id, IC.name FROM tbl_ic_ts TS
            INNER JOIN tbl_ic IC ON TS.ic_id = IC.id
            WHERE IC.region_id = :region_id AND TS.user_id = :user_id
            UNION
            SELECT IC.id, IC.name FROM tbl_ic_ats_all ATSA
            INNER JOIN tbl_ic IC ON ATSA.ic_id = IC.id
            WHERE IC.region_id = :region_id AND ATSA.user_id = :user_id")
            ->bindValue(':region_id', $region_id)
            ->bindValue(':user_id', $user_id)
            ->queryAll();
        $icList = ArrayHelper::map($icList, 'id', 'name');
    } elseif (User::hasRole('40')) {
        $user_id = Yii::$app->user->identity->id;
        $icList = $db->createCommand("SELECT IC.id, IC.name FROM tbl_ic_ats ATS
            INNER JOIN tbl_ic IC ON ATS.ic_id = IC.id
            WHERE IC.region_id = :region_id AND ATS.user_id = :user_id
            UNION
            SELECT IC.id, IC.name FROM tbl_ic_ats_weekend ATSW
            INNER JOIN tbl_ic IC ON ATSW.ic_id = IC.id
            WHERE IC.region_id = :region_id AND ATSW.user_id = :user_id")
            ->bindValue(':region_id', $region_id)
            ->bindValue(':user_id', $user_id)
            ->queryAll();
        $icList = ArrayHelper::map($icList, 'id', 'name');
    } elseif (User::hasRole('85')) {
        $user_id = Yii::$app->user->identity->id;
        $icList = $db->createCommand("SELECT IC.id, IC.name FROM tbl_ic_manager IM
            INNER JOIN tbl_ic IC ON IM.ic_id = IC.id
            WHERE IC.region_id = :region_id AND IM.user_id = :user_id")
            ->bindValue(':region_id', $region_id)
            ->bindValue(':user_id', $user_id)
            ->queryAll();
        $icList = ArrayHelper::map($icList, 'id', 'name');
    } elseif (User::hasRole('90')) { 
        if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }
    } elseif (User::hasRole('91')) { 
        if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }
    }elseif (User::hasRole('95')) { 
        if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }
    }elseif (User::hasRole('65')) { 
        if ($region_id === 'Any') {
            $icList = Ic::getIcWithDefault();
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
        }
    }
        
    
    /*if ($region_id === 'Any') {
        $icList = Ic::getIcWithDefault();
    } else {
        if (User::hasRole('85')) {
            $user_id = Yii::$app->user->identity->id; 
            
            $icList = $db->createCommand("SELECT IC.id,IC.name FROM tbl_ic_users ICU 
            INNER JOIN tbl_ic IC ON ICU.ic_id = IC.id
            WHERE IC.region_id = $region_id AND ICU.user_id = $user_id ")->queryAll();
             $icList = ArrayHelper::map($icList, 'id', 'name');
            
            print_r($icList);die;
        } else {
            $icList = Ic::find()->where(['region_id' => $region_id])->all();
            $icList = ArrayHelper::map($icList, 'id', 'name');
            $icList = ['Any' => 'Any'] + $icList;
            
            //print_r($icList);die;
        }
    }*/

        // $options = "<option value=''>Any</option>";
        // if ($icList) {
        //     foreach ($icList as $ic) {
        //         echo "<option value='{$ic->id}'>{$ic->name}</option>";
        //     }
        
        
        
        if (!empty($icList)) {
        foreach ($icList as $key => $value) {
            $options .= "<option value='{$key}'>{$value}</option>";
        }
        } else {
            echo "<option value=''>No IC found</option>";
        }
        echo $options;

    }

    




}
