<?php

namespace app\controllers;

use Yii;
use app\models\IcUsers;
use app\models\IcUsersSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;

/**
 * IcUsersController implements the CRUD actions for IcUsers model.
 */
class IcUsersController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update','delete','get-tellers-by-ic',
                        'get-tellers-by-ic-with-default'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all IcUsers models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new IcUsersSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single IcUsers model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new IcUsers model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new IcUsers();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing IcUsers model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing IcUsers model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the IcUsers model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return IcUsers the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = IcUsers::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionGetTellersByIc($ic_id) {
        
        $db = new yii\db\Connection(Yii::$app->db);

        $tellers = $db->createCommand("SELECT U.id,U.username
            FROM `user` U
            INNER JOIN tbl_ic_users IU ON IU.user_id = U.id AND IU.ic_id = $ic_id and IU.status = 1")->queryAll();

        if(!empty($tellers)){
            foreach($tellers as $teller){
                echo "<option value='".$teller['id']."'>".$teller['username']."</option>";
            }
        }else{
            echo "<option value = ''>This interchange has no tellers</option>";
        }
        
    }

    public function actionGetTellersByIcWithDefault($ic_id) {
        
        $db = new yii\db\Connection(Yii::$app->db);
        if($ic_id == 'Any'){
            $tellers = $db->createCommand("SELECT U.id,U.username
                FROM `user` U
                INNER JOIN tbl_ic_users IU ON IU.user_id = U.id AND IU.status = 1")->queryAll();
        }else{
            $tellers = $db->createCommand("SELECT U.id,U.username
                FROM `user` U
                INNER JOIN tbl_ic_users IU ON IU.user_id = U.id AND IU.ic_id = $ic_id and IU.status = 1")->queryAll();
        }

        
        if(!empty($tellers)){
            echo "<option value='Any'>Any</option>";
            foreach($tellers as $teller){
                echo "<option value='".$teller['id']."'>".$teller['username']."</option>";
            }
        }else{
            echo "<option value='Any'>Any</option>";
        }
        
    }
}
