<?php

namespace app\controllers;

use Yii;
use app\models\NfcCard;
use app\models\NfcCardSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\NfcAllocation;
use yii\web\Response;
use yii\filters\AccessControl;

/**
 * NfcCardController implements the CRUD actions for NfcCard model.
 */
class NfcCardController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view','update','delete','unassigned','create' ],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all NfcCard models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new NfcCardSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single NfcCard model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {

       $allocation_data =  NfcAllocation::find()->where(['nfc_card_id'=>$id])->orderBy([
        'id' => SORT_DESC,
       
      ])->all();
      $nfc_allocated_on =  NfcAllocation::find()->where(['nfc_card_id'=>$id,'status'=>NfcAllocation::STATUS_ACTIVE])->orderBy([
        'id' => SORT_DESC,
       
      ])->one();
      $AllocationModel = new NfcAllocation();
      if ($AllocationModel->load(Yii::$app->request->post()) ) {

           
       
        if($AllocationModel->save()){
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_nfc"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_nfc"')->execute();
            
             return $this->redirect(['view', 'id' => $AllocationModel->nfc_card_id]);
        }

       
    }

   
      
       return $this->render('view', [
            'model' => $this->findModel($id),
            'allocation_data'=>$allocation_data,

            'nfc_allocated_on'=>$nfc_allocated_on,
            'AllocationModel'=>$AllocationModel
        ]);
    }

    /**
     * Creates a new NfcCard model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new NfcCard();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_nfc"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_nfc"')->execute();
              $update_created_by = Yii::$app->db->createCommand('UPDATE tbl_nfc_card SET created_by='.\Yii::$app->user->identity->id.' WHERE id='.$model->id.'')->execute();   
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing NfcCard model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_nfc"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_nfc"')->execute();
              $update_created_by = Yii::$app->db->createCommand('UPDATE tbl_nfc_card SET created_by='.\Yii::$app->user->identity->id.' WHERE id='.$model->id.'')->execute();   
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing NfcCard model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        try {
            $this->findModel($id)->delete();
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_nfc"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_nfc"')->execute();
            
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the NfcCard model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return NfcCard the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = NfcCard::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
    public function actionUnassigned()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $nfc_card_id = ($_POST['nfc_card_id']);
      
         $today = date("Y-m-d H:i:s");
         $user_id = Yii::$app->user->id;

      
       //$res =  NfcAllocation::updateAll(['status' => 0], ['AND',['=', 'nfc_card_id', $nfc_card_id],['=','status',1]]);
       $res = NfcAllocation::updateAll(
        [
            'status' => 0,
            'removed_on' => $today,
            'removed_by' => $user_id
        ], 
        [
            'nfc_card_id' => $nfc_card_id,
            'status' => 1
        ]
    );
      
       
        if( $res){
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_nfc"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_nfc_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_nfc"')->execute();
            
            return true;
        }else{
            return false;
        }
        
    }
}
