<?php

namespace app\controllers;

use Yii;
use app\models\Roster;
use app\models\RosterSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\Schedule;
use app\models\Ic;
use app\models\Shift;
use app\models\Session;
use app\models\SessionUsers;
use app\models\ShiftUsers;
use yii\helpers\ArrayHelper;
use webvimark\modules\UserManagement\models\User;
use app\models\Booth;
use yii\filters\AccessControl;
use app\models\IcHeadTellers;
use app\models\IcHeadTellersWeekend;
use app\models\IcAts;
use app\models\IcAtsAll;
use app\models\IcAtsWeekend;
use app\models\IcTs;
use app\models\RegionAdmin;
use app\models\RegionManager;
use app\models\IcManager;





/**
 * RosterController implements the CRUD actions for Roster model.
 */
class RosterController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update',
                        'delete','booth','delete-user','redirect-schedule',
                        'delete-session','shift','booth-quick','shift-quick','real-time','no-roster'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Roster models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new RosterSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }
    public function actionRealTime()
    {
        $today = date('Y-m-d');
        $roster=Roster::find()->where(['AND',['<=','from',$today],['>=','to',$today]])->one();
        if($roster->id){
            return $this->redirect(['view','id'=>$roster->id]);
        }else{
            Yii::$app->getSession()->setFlash('danger', 'No Roster for current date');
            return $this->redirect(['no-roster']);
        }
       

    }
    public function actionNoRoster()
    {
        return $this->render('no-roster', [
         
        ]);
       

    }


    /**
     * Displays a single Roster model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        $model = $this->findModel($id);
        $interchanges = Ic::find()->where(['status'=>Ic::STATUS_ACTIVE])->all();


         //HEAD TELLERS 25
      if(User::hasRole('25')){

        $head_teller = IcHeadTellers::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        $head_teller_weekend = IcHeadTellersWeekend::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        if($head_teller){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller]])->all();  }
        if($head_teller_weekend){
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller_weekend]])->all();
        }
       
    }
      //ATS 40
      if(User::hasRole('40')){

        $head_teller = IcAts::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        $head_teller_weekend = IcAtsWeekend::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        if($head_teller){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller]])->all();  }
        if($head_teller_weekend){
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller_weekend]])->all();
        }
       
    }

     //TS 30
     if(User::hasRole('30')){

        $head_teller = IcAtsAll::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        $head_teller_weekend = IcTs::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        if($head_teller){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller]])->all();  }
        if($head_teller_weekend){
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$head_teller_weekend]])->all();
        }
       
    }
    
    //IC MANAGER 85
    if(User::hasRole('85')){

        $ic_manager = IcManager::find()->select('ic_id as id')->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        if($ic_manager){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$ic_manager]])->all(); 
        }

    }
    
     //REGION ADMIN 90
     if(User::hasRole('90')){

        $regions = RegionAdmin::find()->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        $ar_region = [];
                 foreach ($regions as $region) {
                     array_push($ar_region, $region->region_id);
                 }          
        $ics = Ic::find()->select('id')->where(['region_id'=>$ar_region] )->all();
        $ar =[];
        foreach($ics as $ic){
            array_push($ar,$ic->id);
        }
            if($ics){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$ar]])->all(); 
         }
        
       
    }
    
    //REGION MANAGER 91
    if(User::hasRole('91')){

        $regions = RegionManager::find()->where(['user_id' =>\Yii::$app->user->identity->id,'status'=>1] )->all();
        $ar_region = [];
                 foreach ($regions as $region) {
                     array_push($ar_region, $region->region_id);
                 }          
        $ics = Ic::find()->select('id')->where(['region_id'=>$ar_region] )->all();
        $ar =[];
        foreach($ics as $ic){
            array_push($ar,$ic->id);
        }
            if($ics){
          
            $interchanges = Ic::find()->where(['AND',['status'=>Ic::STATUS_ACTIVE],['in','id',$ar]])->all(); 
         }
        
       
    }
    
    
    
    
        $roster = Roster::findOne($roster_id);    
        $today = date('Y-m-d');
        if($model->from <= $today && $model->to >= $today){
            $schedule = Schedule::find()->where(['roster_id'=>$id,'date'=>$today])->one();

        }else{
            $schedule = Schedule::find()->where(['roster_id'=>$id,'date'=>$model->from])->one();
        }


        return $this->render('view', [
            'model' => $model,
            'interchanges'=>$interchanges,
            'schedule'=> $schedule
        ]);
    }
    public function actionBooth($roster_id,$interchange_id)
    {

        $booths = Booth::find()->where(['ic_id'=>$interchange_id,'status'=>Booth::STATUS_ACTIVE])->all();
         $roster = Roster::findOne($roster_id);
        $interchange = Ic::findOne($interchange_id);
         $schedule = Schedule::find()->where(['roster_id'=>$roster->id,'date'=>$roster->from])->one();


        return $this->render('booth', [
           // 'model' => $this->findModel($id),
            'booths'=>$booths,
            'interchange'=>$interchange,
            'roster_id'=>$roster_id,
            'schedule'=>$schedule,
            'roster'=> $roster,
        ]);
    }
   
   

    public function actionShift($roster_id,$interchange_id,$schedule_id)
    {


        $session_model = new Session();
        $session_user_model = new ShiftUsers();
        
        $user_data = ArrayHelper::map(Yii::$app->db->createCommand('SELECT * FROM user WHERE id IN(SELECT user_id FROM tbl_ic_users WHERE tbl_ic_users.ic_id='.$interchange_id.' AND tbl_ic_users.status=1) AND superadmin=0 AND status=1')->queryAll(), 'id', 'username');

    //    print_r($user_dataa);
    //    die;

        $interchange = Ic::findOne($interchange_id);
     // $booth = Booth::findOne($booth_id);

       $schedule = Schedule::findOne($schedule_id);
        $roster = Roster::findOne($roster_id);
      
     
        $Alllist = array();
          $shifts = Yii::$app->db->createCommand('SELECT tbl_shift.* FROM tbl_shift INNER JOIN tbl_master_shift ON tbl_master_shift.id=tbl_shift.name WHERE schedule_id="' . $schedule->id . '" AND ic_id="' . $interchange_id . '" ')->queryAll();
        
     //   print_r( $shifts);die;

        
        foreach($shifts as $shift){

        
                $users = Yii::$app->db->createCommand('SELECT 
                tbl_shift_users.*,
                user.username,
                user.nic,
                user.profile_log,
                user.contact_number,
                user.file_no,
                user.email,
                user.full_name
                FROM tbl_shift_users INNER JOIN user ON user.id=tbl_shift_users.user_id WHERE shift_id="' . $shift['id'] . '"')->queryAll();
               
                $user_count = Yii::$app->db->createCommand('SELECT COUNT(*) as user_count  FROM tbl_shift_users WHERE shift_id="' . $shift['id'] . '"')->queryOne();
               
                $shift["users"]=$users;
                $shift["user_count"]=$user_count["user_count"];
            
           
           
           array_push($Alllist,$shift);


        }
      
        // echo json_encode($Alllist);
        // die;  
     //   print_r($shifts );die;


        // if ($session_model->load(Yii::$app->request->post())) {
        //     $model = new Session();
           
        //     $model->start_time = $session_model->start_time;
        //     $model->end_time = $session_model->end_time;
        //     $model->shift_id = $session_model->shift_id;
        //     $model->booth_id = $session_model->booth_id;
          
        //     $shift_info = Yii::$app->db->createCommand('SELECT tbl_shift.*,tbl_shift.shift_start,tbl_shift.shift_end FROM tbl_shift  WHERE tbl_shift.id="' . $session_model->shift_id . '"  ')->queryOne();
        //     $shift_start = date("H:i:s",strtotime($shift_info['shift_start']));
        //     $shift_end = date("H:i:s",strtotime($shift_info['shift_end']));

        //     $model->save();        
        //     return $this->redirect(['shift','roster_id'=>$roster_id,'interchange_id'=>$interchange_id,'booth_id'=>$booth->id,'schedule_id'=>$schedule->id]);


        //     if(($session_model->start_time < $session_model->end_time) && ($shift_start <= $session_model->start_time && $shift_end >= $session_model->end_time )){
               
        //     }else{
        //         Yii::$app->getSession()->setFlash('danger', 'Please consider your inputs');

        //     }
            
        // }
        if ($session_user_model->load(Yii::$app->request->post()) && isset($session_user_model['user_id'])) {
           
            foreach ($session_user_model['user_id'] as $user_id) {

                $model = new ShiftUsers();
                $model->shift_id = $session_user_model->shift_id;
                $model->user_id = $user_id;
                $exist = ShiftUsers::find()->where(['shift_id'=>$session_user_model->shift_id,'user_id'=>$user_id])->one();
                if($exist->id){
                    Yii::$app->getSession()->setFlash('danger', 'User already exists');
                    return $this->redirect(['shift','roster_id'=>$roster_id,'interchange_id'=>$interchange_id,'booth_id'=>$booth->id,'schedule_id'=>$schedule->id]);

                }else{
                    $model->save();
                  
                }
               
            }
            return $this->redirect(['shift','roster_id'=>$roster_id,'interchange_id'=>$interchange_id,'booth_id'=>$booth->id,'schedule_id'=>$schedule->id]);

           
        }


        return $this->render('shift', [
            
            'interchange'=>$interchange,
            'booth'=>$booth,
            'roster'=>$roster,
            'schedule'=>$schedule,         
            'session_model'=>$session_model,
            'session_user_model'=>$session_user_model,
            'Alllist'=>$Alllist,
          //  'user_data'=>$user_data
         
        ]);
    }
   
    /**
     * Creates a new Roster model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Roster();
        $last_exist_roster = Roster::find()->orderBy(['id'=> SORT_DESC])->one();
     // print_r($last_exist_roster);die;
    
        if ($model->load(Yii::$app->request->post())  ) {
            //* $model->from = "2020-02-29" ;
            //* $model->to ="2020-03-31";
            //* $model->id =18;
            if($model->from < $model->to ){
                if($model->save()){
                   //* if(true){

                    //call to component for get all dates between two dates
                    $all_dates = Yii::$app->Dates->getDatesFromRange($model->from, $model->to);            
                    //call to component for loop schedules and shifts
                    $results = Yii::$app->Roster->createScheduleAndShifts($all_dates, $model->id);  

                    Yii::$app->getSession()->setFlash('success', 'Roster is created');          
                    return $this->redirect(['view', 'id' => $model->id]);


                }
                
                
            }else{
                Yii::$app->getSession()->setFlash('danger', 'Please consider your end date');

            }
            
        }

        return $this->render('create', [
            'model' => $model,
            'last_exist_roster'=>$last_exist_roster
        ]);
    }

    /**
     * Updates an existing Roster model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Roster model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {   
         //call to component for loop schedules and shifts
         $results = Yii::$app->Roster->deleteScheduleAndShifts($id);
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Roster model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Roster the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Roster::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
    public function actionBoothQuick()
    {
        $id = ($_POST['id']);
        $roster_id = ($_POST['roster_id']);
   
        $this->redirect(['booth', 'roster_id' => $roster_id,'interchange_id'=> $id]);
    }
    public function actionShiftQuick()
    {
        $id = ($_POST['id']);
        $ic = Ic::find()->where(['id'=>$id])->one();
        $roster_id = ($_POST['roster_id']);
        $schedule_id = ($_POST['schedule_id']);
     


        $this->redirect(['shift', 'roster_id' => $roster_id,'interchange_id'=> $id,'schedule_id'=>$schedule_id]);
    }
    public function actionDeleteUser() {

        $id = ($_POST['id']);       
        $result = Yii::$app->db->createCommand('DELETE FROM tbl_shift_users WHERE id=' . $id . '')->execute();
        if ($result) {       
         
            return true;
        }
       
    }
    public function actionRedirectSchedule() {

        $schedule_date = ($_POST['schedule_date']);     
        $roster_id = ($_POST['roster_id']);  
        $interchange_id = ($_POST['interchange_id']);  
        $booth_id = ($_POST['booth_id']);    
        $schedule = Yii::$app->db->createCommand('SELECT id FROM tbl_schedule WHERE date="'.$schedule_date.'"')->queryOne();
      
        return $this->redirect(['shift','roster_id'=>$roster_id,'interchange_id'=>$interchange_id,'booth_id'=>$booth_id,'schedule_id'=>$schedule['id']]);

       
    }
    public function actionDeleteSession() {


        $shift_id = ($_POST['shift_id']);
       
        $result = Yii::$app->db->createCommand('DELETE FROM tbl_shift_users WHERE shift_id=' . $shift_id . '')
        ->execute(); 
        // $result = Yii::$app->db->createCommand('DELETE FROM tbl_session WHERE id=' . $session_id . '')
        //         ->execute();
              
       
        return true;
        
       
    }

}