<?php

namespace app\controllers;

use Yii;
use app\models\Route;
use app\models\RouteSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Url;
use yii\web\Response;
use yii\filters\AccessControl;
use webvimark\modules\UserManagement\models\User;

/**
 * RouteController implements the CRUD actions for Route model.
 */
class RouteController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update', 'delete', 'distance', 'distance-save', 'max-time', 'max-time-save', 'direction', 'direction-save'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Route models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new RouteSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Route model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Route model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new Route();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
                    'model' => $model,
        ]);
    }

    /**
     * Updates an existing Route model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id) {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
                    'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Route model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Route model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Route the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = Route::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionQuick() {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }

    public function actionDistance() {

        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();
       
        $buttenEnable = 1;
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $buttenEnable = 1;
        } else if (User::hasRole('95')) {
            $buttenEnable = 0;
        } else if (User::hasRole('65')) {
            $buttenEnable = 0;
        } else if (User::hasRole('90')) {
            $buttenEnable = 0;
        }else if (User::hasRole('91')) {
            $buttenEnable = 0;
        }else if (User::hasRole('85')) {
            $buttenEnable = 0;
        }


        return $this->render('distance', [
                    'ic' => $ic,
                    'buttenEnable' => $buttenEnable,
        ]);
    }

    public function actionDirection() {

        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();
        $directions = Yii::$app->db->createCommand('SELECT * FROM tbl_direction')->queryAll();

        $buttenEnable = 1;
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $buttenEnable = 1;
        } else if (User::hasRole('95')) {
            $buttenEnable = 0;
        } else if (User::hasRole('65')) {
            $buttenEnable = 0;
        } else if (User::hasRole('90')) {
            $buttenEnable = 0;
        }else if (User::hasRole('91')) {
            $buttenEnable = 0;
        }else if (User::hasRole('85')) {
            $buttenEnable = 0;
        }
        
        // print_r($saveButtenEnable);die;
        return $this->render('direction', [
                    'ic' => $ic,
                    'directions' => $directions,
                    'buttenEnable' => $buttenEnable,
        ]);
    }

    public function actionDistanceSave() {


        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();
        $distances = $_POST["distances"];
        for ($i = 0; $i < sizeof($ic); $i++) {

            for ($j = 0; $j < $i; $j++) {
                $single_route = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$i]['id'] . ' AND destination=' . $ic[$j]['id'] . '')->queryOne();
                $single_route_2 = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$j]['id'] . ' AND destination=' . $ic[$i]['id'] . '')->queryOne();
                if (!($single_route['distance'] == $distances["val" . $ic[$i]['id'] . $ic[$j]['id']])) {
                    Yii::$app->db->createCommand('UPDATE tbl_route SET distance=' . $distances["val" . $ic[$i]['id'] . $ic[$j]['id']] . '  WHERE id=' . $single_route['id'] . '')->execute();
                    Yii::$app->db->createCommand('UPDATE tbl_route SET distance=' . $distances["val" . $ic[$i]['id'] . $ic[$j]['id']] . '  WHERE id=' . $single_route_2['id'] . '')->execute();
                }
            }
        }
        $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_fare"')->queryOne();
        $new_version = $master_version['tbl_version'] + 1;
        $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_fare"')->execute();

        $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_route"')->queryOne();
        $new_version = $master_version['tbl_version'] + 1;
        $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_route"')->execute();

        Yii::$app->response->format = Response::FORMAT_JSON;
        return true;
    }

    public function actionDirectionSave() {

        $distances = json_decode($_POST["directions"], true);
        Yii::$app->response->format = Response::FORMAT_JSON;
        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();

        for ($i = 0; $i < sizeof($ic); $i++) {

            for ($j = 0; $j < sizeof($ic); $j++) {

                if ($ic[$i]['id'] != $ic[$j]['id']) {

                    $single_route = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$i]['id'] . ' AND destination=' . $ic[$j]['id'] . '')->queryOne();
                    $single_route2 = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$j]['id'] . ' AND destination=' . $ic[$i]['id'] . '')->queryOne();

                    $dir_val_1 = $distances[$single_route['id']];
                    $dir_val_2 = $distances[$single_route2['id']];

                    if (!empty($single_route) and!empty($single_route2)) {

                        try {

                            Yii::$app->db->createCommand('UPDATE tbl_route SET direction=' . $dir_val_1 . ' WHERE id=' . $single_route['id'] . '')->execute();
                            Yii::$app->db->createCommand('UPDATE tbl_route SET direction=' . $dir_val_2 . ' WHERE id=' . $single_route2['id'] . '')->execute();
                        } catch (\yii\db\Exception $e) {

                            Yii::error("Failed to insert data: " . $e->getMessage());
                            return "An error occurred: " . $e->getMessage();
                        }
                    } else {

                        $model1 = new Route();
                        $model1->source = $ic[$i]['id'];
                        $model1->destination = $ic[$j]['id'];
                        $model1->direction = 1;
                        $model1->distance = 0;
                        $model1->max_time = '23:59:00';
                        $model1->save();

                        $model2 = new Route();
                        $model2->source = $ic[$j]['id'];
                        $model2->destination = $ic[$i]['id'];
                        $model2->direction = 1;
                        $model2->distance = 0;
                        $model2->max_time = '23:59:00';
                        $model2->save();
                    }
                }
            }
        }


        Yii::$app->response->format = Response::FORMAT_JSON;
        return true;
    }

    public function actionMaxTime() {
        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();

        $saveButtenEnable = 1;

        $buttenEnable = 1;
        if (User::hasRole(Yii::$app->user->isSuperAdmin)) {
            $buttenEnable = 1;
        } else if (User::hasRole('95')) {
            $buttenEnable = 0;
        } else if (User::hasRole('65')) {
            $buttenEnable = 0;
        } else if (User::hasRole('90')) {
            $buttenEnable = 0;
        }else if (User::hasRole('91')) {
            $buttenEnable = 0;
        }else if (User::hasRole('85')) {
            $buttenEnable = 0;
        }

        return $this->render('max-time', [
                    'ic' => $ic,
                    'buttenEnable' => $buttenEnable,
        ]);
    }

    public function actionMaxTimeSave() {
        $ic = Yii::$app->db->createCommand('SELECT * FROM tbl_ic')->queryAll();

        $max_times = $_POST["max_times"];
        for ($i = 0; $i < sizeof($ic); $i++) {

            for ($j = 0; $j < $i; $j++) {
                $single_route = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$i]['id'] . ' AND destination=' . $ic[$j]['id'] . '')->queryOne();
                $single_route_2 = Yii::$app->db->createCommand('SELECT * FROM tbl_route WHERE source=' . $ic[$j]['id'] . ' AND destination=' . $ic[$i]['id'] . '')->queryOne();
                if (!($single_route['max_time'] == $max_times["val" . $ic[$i]['id'] . $ic[$j]['id']])) {
                    Yii::$app->db->createCommand('UPDATE tbl_route SET max_time="' . $max_times["val" . $ic[$i]['id'] . $ic[$j]['id']] . '"  WHERE id=' . $single_route['id'] . '')->execute();
                    Yii::$app->db->createCommand('UPDATE tbl_route SET max_time="' . $max_times["val" . $ic[$i]['id'] . $ic[$j]['id']] . '"  WHERE id=' . $single_route_2['id'] . '')->execute();
                }
            }
        }
        $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_fare"')->queryOne();
        $new_version = $master_version['tbl_version'] + 1;
        $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_fare"')->execute();
        $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_route"')->queryOne();
        $new_version = $master_version['tbl_version'] + 1;
        $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version=' . $new_version . ' WHERE tbl_name="tbl_route"')->execute();

        Yii::$app->response->format = Response::FORMAT_JSON;
        return true;
    }

}
