<?php

namespace app\controllers;

use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;
use yii\filters\VerbFilter;
use webvimark\modules\UserManagement\models\User;
use machour\yii2\notifications\NotificationsModule;
use app\common\components\Notification;
use app\models\Region;
use app\models\Ic;
use app\models\Booth;
use app\models\Devices;
use app\models\NfcCard;
use app\models\ExitTicket;
use app\models\EntranceTicket;
use yii\web\NotFoundHttpException;

class SiteController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['logout', 'login'],
                'rules' => [
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions() {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */
    public function actionIndex() {

        if (!Yii::$app->user->isGuest) {

            return $this->render('index', [
            ]);
        } else {
            return $this->redirect(['user-management/auth/login']);
        }
    }

    public function actionDashboard() {
        if (!User::hasRole('25')) {

            throw new NotFoundHttpException('You are not allowed to perform this action.');
        } else {
            $counts = Yii::$app->db->createCommand('SELECT * FROM  dashboard_counts LIMIT 1')->queryOne();

            $users_count = $counts['user_count'];
            $region_count = $counts['region_count'];
            $ic_count = $counts['ic_count'];
            $booth_count = $counts['booth_count'];
            $device_count = $counts['device_count'];
            $entrance_ticket_count = $counts['entrance_ticket_count'];
            $exit_ticket_count = $counts['exit_ticket_count'];
            $nfc_count = $counts['nfc_count'];

            $en_details = Yii::$app->db->createCommand('SELECT * FROM dashboard_entrance_summary')->queryAll();

            $ex_details = Yii::$app->db->createCommand('SELECT * FROM dashboard_exit_summary')->queryAll();

            return $this->render('dashboard', [
                        'users_count' => $users_count,
                        'region_count' => $region_count,
                        'ic_count' => $ic_count,
                        'booth_count' => $booth_count,
                        'device_count' => $device_count,
                        'nfc_count' => $nfc_count,
                        'entrance_ticket_count' => $entrance_ticket_count,
                        'exit_ticket_count' => $exit_ticket_count,
                        'en_details' => $en_details,
                        'ex_details' => $ex_details,
            ]);
            throw new NotFoundHttpException('The requested page does not exist for your assigned booth.');
        }
    }

    /**
     * Login action.
     *
     * @return Response|string
     */
    public function actionLogin() {
        if (!Yii::$app->user->isGuest) {
            return $this->render('index');
        } else {
            return $this->redirect(['user-management/auth/login']);
        }
    }

    /**
     * Logout action.
     *
     * @return Response
     */
    public function actionLogout() {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    /**
     * Displays contact page.
     *
     * @return Response|string
     */
    public function actionContact() {
        if (!Yii::$app->user->isGuest) {
            return $this->render('index');
        } else {
            return $this->redirect(['user-management/auth/login']);
        }
    }

    /**
     * Displays about page.
     *
     * @return string
     */
    public function actionAbout() {
        if (!Yii::$app->user->isGuest) {
            return $this->render('index');
        } else {
            return $this->redirect(['user-management/auth/login']);
        }
    }

}
