<?php

namespace app\controllers;

use Yii;
use app\models\TicketTypeStatus;
use app\models\TicketTypeStatusSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\helpers\ArrayHelper;
use yii\web\Response;

/**
 * TicketTypeStatusController implements the CRUD actions for TicketTypeStatus model.
 */
class TicketTypeStatusController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all TicketTypeStatus models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new TicketTypeStatusSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single TicketTypeStatus model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new TicketTypeStatus model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new TicketTypeStatus();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
                    'model' => $model,
        ]);
    }

    /**
     * Updates an existing TicketTypeStatus model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id) {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
                    'model' => $model,
        ]);
    }

    /**
     * Deletes an existing TicketTypeStatus model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id) {
        try {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the TicketTypeStatus model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return TicketTypeStatus the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = TicketTypeStatus::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionQuick() {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }

}
