<?php

namespace app\controllers;

use Yii;
use app\models\TimeSlot;
use app\models\TimeSlotSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;

/**
 * TimeSlotController implements the CRUD actions for TimeSlot model.
 */
class TimeSlotController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update','delete'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all TimeSlot models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new TimeSlotSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single TimeSlot model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new TimeSlot model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new TimeSlot();
        $first_exist_shift = TimeSlot::find()->orderBy(['id'=> SORT_ASC])->one();
        
        $last_exist_shift = TimeSlot::find()->orderBy(['id'=> SORT_DESC])->one();
      

        $all_shift = TimeSlot::find()->all();
        $all_hours =0;
        foreach($all_shift as $shift){
           $hours =  strtotime($shift->end_time) - strtotime($shift->start_time);
           $duration_in_hours= $hours/60/60;
       
          
           // print_r( "*");
          $all_hours = $all_hours+$duration_in_hours;

        }
        $rest_hours = 24-$all_hours;

     

        if ($model->load(Yii::$app->request->post())) {
            
         $hours =  strtotime($model->end_time) - strtotime($model->start_time);
            $duration_in_hours= $hours/60/60;
            if ($duration_in_hours < 0)
            {
                $duration_in_hours = $duration_in_hours+24;
                // echo "The profitloss is negative";
            }
           // print_r($duration_in_hours);die;
            if($rest_hours >= $duration_in_hours ){
                if($model->save()){
                    $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_fare"')->queryOne();
                    $new_version = $master_version['tbl_version']+1;
                    $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_fare"')->execute();
                   
                    return $this->redirect(['view', 'id' => $model->id]);
                }
            }else{
                Yii::$app->getSession()->setFlash('danger', '24 hr exceed');
            }
        }

        return $this->render('create', [
            'model' => $model,
            'last_exist_shift'=>$last_exist_shift,
            'first_exist_shift'=>$first_exist_shift
        ]);
    }

    /**
     * Updates an existing TimeSlot model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_fare"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_fare"')->execute();
           
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing TimeSlot model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        try {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the TimeSlot model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return TimeSlot the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = TimeSlot::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
}
