<?php

namespace app\controllers;

use Yii;
use app\models\Towing;
use app\models\TowingValue;
use app\models\TowingSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\Shift;
use app\models\LoginLogoutSession;
use app\models\CashCollect;
use webvimark\modules\UserManagement\models\User;
use yii\db\Query;
use yii\filters\AccessControl;

/**
 * TowingController implements the CRUD actions for Towing model.
 */
class TowingController extends Controller {

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete','get-session-list','towing-reciept-pdf'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all Towing models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new TowingSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Towing model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id) {
        setcookie("towing_record_id", $id, time() + 86400);

        return $this->render('view', [
                    'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Towing model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new Towing();
        $sessionList = [];

        $activeTovingValue = TowingValue::find()->where(['status' => 1])->one();

        if ($model->load(Yii::$app->request->post())) {
            $req_time = date('Y-m-d H:i:s');
            $shift = Shift::find()->where(['AND', ['<=', 'shift_start', $req_time], ['>=', 'shift_end', $req_time], ['=', 'ic_id', (string) $model->ic_id]])->one();

            $sessionList = Towing::getSessionList($model->ic_id);

            // Generate the next receipt number
            $lastReceipt = Towing::find()->select('receipt_no')->orderBy(['id' => SORT_DESC])->limit(1)->one();
            $nextReceiptNumber = $lastReceipt ? intval($lastReceipt->receipt_no) + 1 : 1;
            //$model->receipt_no = str_pad($nextReceiptNumber, 5, '0', STR_PAD_LEFT);
            $receipt_number = $shift->id . "-" . $model->ic_id . "-" . date('YmdHis');
            $model->receipt_no = $receipt_number;
            $model->vehicle_no = strtoupper($model->vehicle_no);
            $model->customer_name = ucwords($model->customer_name);

            $model->created_on = $req_time;

            if ($shift->id) {
                $model->shift_id = $shift->id;
                if ($model->save()) {
                    return $this->redirect(['view', 'id' => $model->id]);
                }
            } else {
                Yii::$app->getSession()->setFlash('danger', 'No shift available for today');
            }
        }

        return $this->render('create', [
                    'model' => $model,
                    'sessionList' => $sessionList,
                    'activeTovingValue' => $activeTovingValue,
        ]);
    }

    /**
     * Updates an existing Towing model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    // public function actionUpdate($id)
    // {
    //     $model = $this->findModel($id);
    //     if ($model->load(Yii::$app->request->post()) && $model->save()) {
    //         return $this->redirect(['view', 'id' => $model->id]);
    //     }
    //     // if ($model->load(Yii::$app->request->post())) {
    //     //     $model->updated_at = Yii::$app->formatter->asDatetime('now', 'yyyy-MM-dd HH:mm:ss');
    //     //     if ($model->save()) {
    //     //         return $this->redirect(['view', 'id' => $model->id]);
    //     //     }
    //     // }
    //     return $this->render('update', [
    //         'model' => $model,
    //     ]);
    // }

    public function actionUpdate($id) {
        $model = $this->findModel($id);

        $model->updated_on = date("Y-m-d H:i:s");
        $model->updated_by = Yii::$app->user->id;

        $actual_session_model = LoginLogoutSession::findOne(['session_uuid' => $model->session_uuid]);

        if (!$actual_session_model) {
            Yii::$app->session->setFlash('error', 'Session not found.');
            return $this->redirect(['index']);
        }

        $cash_collected_data = CashCollect::findOne(['status' => 1, 'actual_session_id' => $actual_session_model->id]);

        if ($cash_collected_data !== null) {
            Yii::$app->session->setFlash('warning', 'Session already processed. Edit is not allowed!');
            return $this->redirect(['index']);
        }

        // if ($model->load(Yii::$app->request->post()) && $model->save()) {
        //     Yii::$app->session->setFlash('success', 'Record updated successfully.');
        //     return $this->redirect(['view', 'id' => $model->id]);
        // }

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->save()) {
                Yii::$app->session->setFlash('success', 'Record updated successfully.');
            } else {
                Yii::$app->session->setFlash('error', 'Failed to update the record.');
            }
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
                    'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Towing model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    // public function actionDelete($id)
    // {
    //     $this->findModel($id)->delete();
    //     return $this->redirect(['index']);
    // }

    public function actionDelete($id) {
        $model = $this->findModel($id);

        $actual_session_model = LoginLogoutSession::findOne(['session_uuid' => $model->session_uuid]);

        if (!$actual_session_model) {
            Yii::$app->session->setFlash('error', 'Session not found. Deletion cannot proceed.');
            return $this->redirect(['index']);
        }

        $cash_collected_data = CashCollect::findOne(['status' => 1, 'actual_session_id' => $actual_session_model->id]);

        if ($cash_collected_data === null) {

            if ($model->delete() !== false) {
                Yii::$app->session->setFlash('success', 'Record deleted successfully.');
            } else {
                Yii::$app->session->setFlash('error', 'Failed to delete the record.');
            }
        } else {

            Yii::$app->session->setFlash('warning', 'Session already processed. Cannot delete this record!');
        }

        return $this->redirect(['index']);
    }

    /**
     * Finds the Towing model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Towing the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = Towing::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    public function actionGetSessionList() {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;

        $ic_id = Yii::$app->request->post('ic_id');

        if (!$ic_id) {
            return ['success' => false, 'message' => 'Invalid IC ID.'];
        }

        try {
            $sessionList = Towing::getSessionList($ic_id);

            if (empty($sessionList)) {
                return ['success' => false, 'message' => 'No sessions found.'];
            }

            return ['success' => true, 'data' => $sessionList];
        } catch (\Exception $e) {

            Yii::error('Error in actionGetSessionList: ' . $e->getMessage(), __METHOD__);
            return ['success' => false, 'message' => 'An error occurred.'];
        }
    }

    public function actionTowingRecieptPdf() {

        $id = $_COOKIE["towing_record_id"];
        $model = $this->findModel($id);

        $sessionUuid = $model->session_uuid;
        $sql = "SELECT user.id, user.username
                FROM tbl_login_logout_session s
                INNER JOIN user ON s.user_id = user.id
                WHERE s.session_uuid = :session_uuid";

        $command = Yii::$app->db->createCommand($sql);
        $command->bindValue(':session_uuid', $sessionUuid);
        $result = $command->queryAll();
        $telerId;
        if (!empty($result)) {
            $telerId = $result[0]['username'];
        }

        $mpdf = new \Mpdf\Mpdf();
        $mpdf->WriteHTML($this->renderPartial('towing-pdf', [
                    'model' => $this->findModel($id),
                    'telerId' => $telerId,
        ]));
        $mpdf->Output('towing reciept ' . date('Y-m-d H:i:s') . '.pdf', 'D');
        exit();
    }

}
