<?php

namespace app\controllers;

use Yii;
use app\models\VehicleCategory;
use app\models\VehicleCategorySearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;

/**
 * VehicleCategoryController implements the CRUD actions for VehicleCategory model.
 */
class VehicleCategoryController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        $requestroute = Yii::$app->requestedRoute;
        $editurl = '/' . $requestroute;
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'quick', 'view', 'create', 'update','delete','vehicle-category'],
                        'roles' => [$editurl],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all VehicleCategory models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new VehicleCategorySearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }
    public function actionVehicleCategory()
    {
        $searchModel = new VehicleCategorySearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }
    /**
     * Displays a single VehicleCategory model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new VehicleCategory model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new VehicleCategory();

        if ($model->load(Yii::$app->request->post()) ) {

            if( $model->type == "MAIN"){
                $last_vehicle_category = VehicleCategory::find()->where(['type'=>'MAIN'])->orderBy(['id' => SORT_DESC])->one();
                if($last_vehicle_category){
                    $model->id = $last_vehicle_category->id +1;
                }else{
                    $model->id = 1;
                }
                
            }elseif( $model->type == "SERVICE"){
                $last_vehicle_category = VehicleCategory::find()->where(['type'=>'SERVICE'])->orderBy(['id' => SORT_DESC])->one();
                if($last_vehicle_category){
                    $model->id = $last_vehicle_category->id +1;
                }else{
                    $model->id = 20;
                }

            }elseif( $model->type == "SPECIAL"){
                $last_vehicle_category = VehicleCategory::find()->where(['type'=>'SPECIAL'])->orderBy(['id' => SORT_DESC])->one();
                if($last_vehicle_category){
                    $model->id = $last_vehicle_category->id +1;
                }else{
                    $model->id = 50;
                }

            }else{

            }
            // $model->validate();
            // print_r( $model->getErrors());
            // die;
            
            if($model->save()){
                $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_vehicle_category"')->queryOne();
                $new_version = $master_version['tbl_version']+1;
                $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_vehicle_category"')->execute();
                
                return $this->redirect(['view', 'id' => $model->id]);

            }

           
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing VehicleCategory model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_vehicle_category"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_vehicle_category"')->execute();
           
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing VehicleCategory model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        try {
            $this->findModel($id)->delete();
            $master_version = Yii::$app->db->createCommand('SELECT * FROM tbl_version WHERE tbl_name="tbl_vehicle_category"')->queryOne();
            $new_version = $master_version['tbl_version']+1;
            $increament_master_fare_version = Yii::$app->db->createCommand('UPDATE tbl_version SET tbl_version='.$new_version.' WHERE tbl_name="tbl_vehicle_category"')->execute();
           
            Yii::$app->getSession()->setFlash('success', 'Success - deleted');
            return $this->redirect(['index']);
        } catch (\Exception $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        } catch (\Throwable $e) {
            Yii::$app->getSession()->setFlash('danger', 'You are not allowed to delete this item.Item is used in another place ');
            return $this->redirect(['index']);
        }
    }

    /**
     * Finds the VehicleCategory model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return VehicleCategory the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = VehicleCategory::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionQuick()
    {
        $id = ($_POST['id']);
        $this->redirect(['view', 'id' => $id]);
    }
}
